/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor;

import java.util.Collection;
import java.util.HashSet;
import java.util.UUID;

import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.MPartStack;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.EPartService.PartState;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeNode;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;
import eu.etaxonomy.taxeditor.bulkeditor.e4.BulkEditorE4;
import eu.etaxonomy.taxeditor.bulkeditor.input.TaxonEditorInput;
import eu.etaxonomy.taxeditor.editor.descriptiveDataSet.DescriptiveDataSetEditor;
import eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.CharacterMatrixPart;
import eu.etaxonomy.taxeditor.editor.e4.TaxonEditorInputE4;
import eu.etaxonomy.taxeditor.editor.group.authority.CdmAuthorityEditorInput;
import eu.etaxonomy.taxeditor.editor.group.authority.e4.CdmAuthorityEditorE4;
import eu.etaxonomy.taxeditor.editor.internal.TaxeditorEditorPlugin;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.view.checklist.e4.DistributionEditorPart;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateViewEditorInput;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.workbench.WorkbenchUtility;
import eu.etaxonomy.taxeditor.workbench.part.IE4SavablePart;

/**
 * Utility for the editor package
 *
 * @author n.hoffmann
 * @created 20.01.2009
 * @version 1.0
 */
public class EditorUtil extends AbstractUtility {

    private static final String NAME_EDITOR_ID = "eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4";

    public static void openDescriptiveDataSetEditor(UUID descriptiveDataSetUuid, EModelService modelService, EPartService partService, MApplication application){
        Collection<MPart> parts = partService.getParts();
        for (MPart part : parts) {
            if(part.getObject() instanceof DescriptiveDataSetEditor
                && ((DescriptiveDataSetEditor) part.getObject()).getDescriptiveDataSet().getUuid().equals(descriptiveDataSetUuid)){
                partService.showPart(part, PartState.ACTIVATE);
                return;
            }
        }
        String partId = AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_EDITOR_VIEW_DESCRIPTIVEDATASET_DESCRIPTIVEDATASETEDITOR;
        MPart part = showPart(partId, modelService, partService, application);
        DescriptiveDataSetEditor editor = (DescriptiveDataSetEditor) part.getObject();
        editor.init(descriptiveDataSetUuid);
    }

    public static void openCharacterMatrix(UUID descriptiveDataSetUuid, EModelService modelService, EPartService partService, MApplication application){
        Collection<MPart> parts = partService.getParts();
        for (MPart part : parts) {
            if(part.getObject() instanceof CharacterMatrixPart
                && ((CharacterMatrixPart) part.getObject()).getDescriptiveDataSet().getUuid().equals(descriptiveDataSetUuid)){
                partService.showPart(part, PartState.ACTIVATE);
                return;
            }
        }
        String partId = AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_EDITOR_DESCRIPTIVEDATASET_MATRIX_CHARACTERMATRIXPART;
        MPart part = showPart(partId, modelService, partService, application);
        CharacterMatrixPart editor = (CharacterMatrixPart) part.getObject();
        editor.init(descriptiveDataSetUuid, true);
    }

    public static void openDistributionEditor(UUID parentTaxonUuid, EModelService modelService, EPartService partService, MApplication application){
        Collection<MPart> parts = partService.getParts();
        String partId = AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_EDITOR_VIEW_CHECKLIST_E4_DISTRIBUTIONEDITORPART;
        String partIdFactualData = AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_EDITOR_VIEW_DESCRIPTIVE_E4_FACTUALDATAPARTE4;
        String partIdMedia = AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_EDITOR_VIEW_MEDIA_E4_MEDIAVIEWPARTE4;
        closePart(partIdMedia, partService);
        closePart(partIdFactualData, partService);
        MPart part = showPart(partId, modelService, partService, application);
        DistributionEditorPart editor = (DistributionEditorPart) part.getObject();
        editor.init(parentTaxonUuid);
    }


    public static void openSpecimenEditor(DerivateViewEditorInput input, EModelService modelService, EPartService partService, MApplication application){
        String partId = AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_EDITOR_VIEW_DERIVATE_DERIVATEVIEW;
        MPart part = showPart(partId, modelService, partService, application);
        DerivateView derivateView = (DerivateView) part.getObject();
        derivateView.init(input);
    }

    public static void openRightsEditor(CdmAuthorityEditorInput input, EModelService modelService, EPartService partService, MApplication application){
        Collection<MPart> parts = partService.getParts();
        for (MPart part : parts) {
            if(part.getObject() instanceof CdmAuthorityEditorE4
                && ((CdmAuthorityEditorE4) part.getObject()).getInput().getGroup().equals(input.getGroup())){
                partService.showPart(part, PartState.ACTIVATE);
                return;
            }
        }
        String partId = AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_EDITOR_GROUP_AUTHORITY_E4_CDMAUTHORITYEDITORE4;
        MPart part = showPart(partId, modelService, partService, application);
        CdmAuthorityEditorE4 authorityView = (CdmAuthorityEditorE4) part.getObject();
        authorityView.init(input);
    }

    public static MPart showPart(String partId, EModelService modelService, EPartService partService, MApplication application){
        MPart part = partService.findPart(partId);
        if(part==null || modelService.getPartDescriptor(partId).isAllowMultiple()){
            part = partService.createPart(partId);
        }
        MPartStack editorAreaPartStack = WorkbenchUtility.getEditorAreaPartStack(application, modelService);
        if(editorAreaPartStack!=null){
            editorAreaPartStack.getChildren().add(part);
        }
        return partService.showPart(part, PartState.ACTIVATE);
    }

    public static MPart showPart(String partId, EModelService modelService, EPartService partService){
        MPart part = partService.findPart(partId);
        if(part==null || modelService.getPartDescriptor(partId).isAllowMultiple()){
            part = partService.createPart(partId);
            partService.activate(part);
        }else{
            partService.activate(part);
        }

       return part;
    }

	public static void openTaxonNodeE4(UUID taxonNodeUuid, EModelService modelService, EPartService partService, MApplication application) {
	    TaxonEditorInputE4 input = TaxonEditorInputE4.NewInstance(taxonNodeUuid);
	    openNameEditor_internal(input, modelService, partService, application);
	}

	public static void openTaxonBaseE4(UUID taxonBaseUuid, EModelService modelService, EPartService partService, MApplication application) {
	    TaxonEditorInputE4 input = TaxonEditorInputE4.NewInstanceFromTaxonBase(taxonBaseUuid);
	    openNameEditor_internal(input, modelService, partService, application);
	}

    public static void openTaxonBaseE4(UUID taxonBaseUuid) {
        //FIXME E4 this can probably be removed when fully migrated
        TaxonEditorInputE4 input = TaxonEditorInputE4.NewInstanceFromTaxonBase(taxonBaseUuid);

        EPartService partService = TaxeditorEditorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getService(EPartService.class);
        EModelService modelService = TaxeditorEditorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getService(EModelService.class);
        openNameEditor_internal(input, modelService, partService, null);
    }

    private static void openNameEditor_internal(TaxonEditorInputE4 input, EModelService modelService, EPartService partService, MApplication application) {
        TaxonBase taxonBase = input.getTaxon();
        if(taxonBase==null){
            return;
        }
        if (taxonBase.isOrphaned()) {
            if(taxonBase.isInstanceOf(Synonym.class)){
                MessagingUtils.warningDialog(Messages.EditorUtil_ORPHAN_ACCEPTED_TAXON, TaxonEditorInputE4.class, Messages.EditorUtil_ORPHAN_ACCEPTED_TAXON_MESSAGE);
                return;
            }
            else{
                MessagingUtils.warningDialog(Messages.EditorUtil_ORPHAN_TAXON, TaxonEditorInputE4.class, Messages.EditorUtil_ORPHAN_TAXON_MESSAGE);
                return;
            }
        }

        Collection<MPart> parts = partService.getParts();
        //check if part is already opened
        for (MPart part : parts) {
        	if(part.getObject() instanceof TaxonNameEditorE4
                    && ((TaxonNameEditorE4) part.getObject()).getTaxon()!=null
                    && ((TaxonNameEditorE4) part.getObject()).getTaxon().getUuid().equals(input.getTaxon().getUuid())){
        	    if (part.isDirty()){
        	        forceUserSaveE4Editor(((TaxonNameEditorE4) part.getObject()), getShell());
        	    }
                partService.hidePart(part);
                break;
            }
        }
        MPart part = showPart(NAME_EDITOR_ID, modelService, partService, application);

        TaxonNameEditorE4 editor = (TaxonNameEditorE4) part.getObject();
        editor.init(input);
        String partIdFactualData = AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_EDITOR_VIEW_DESCRIPTIVE_E4_FACTUALDATAPARTE4;
        String partIdMedia = AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_EDITOR_VIEW_MEDIA_E4_MEDIAVIEWPARTE4;
        showPart(partIdMedia, modelService, partService);
        showPart(partIdFactualData, modelService, partService);
        editor.setFocus();
    }

    public static Collection<MPart> checkForChanges(UUID taxonUUID, EPartService partService ){
        Collection<MPart> parts = partService.getParts();
        Collection<MPart> dirtyParts = new HashSet();
        //check if part is already opened
        boolean isDirty = false;
        for (MPart part : parts) {
            if(part.getObject() instanceof TaxonNameEditorE4
                    && ((TaxonNameEditorE4) part.getObject()).getTaxon()!=null
                    && ((TaxonNameEditorE4) part.getObject()).getTaxon().getUuid().equals(taxonUUID)){
                if (part.isDirty()){
                    dirtyParts.add(part);
                }

                break;
            }else if (taxonUUID == null){
                if (part.isDirty()){
                    dirtyParts.add(part);
                }
            }
        }
        return dirtyParts;
    }

    public static Collection<IE4SavablePart> checkForTaxonChanges(UUID taxonUUID, EPartService partService ){
        Collection<MPart> parts = partService.getParts();
        Collection<IE4SavablePart> dirtyParts = new HashSet<>();
        //check if part is already opened
        for (MPart part : parts) {
            if(part.getObject() instanceof TaxonNameEditorE4
                    && ((TaxonNameEditorE4) part.getObject()).getTaxon()!=null
                    && ((TaxonNameEditorE4) part.getObject()).getTaxon().getUuid().equals(taxonUUID)){
                if (part.isDirty()){
                    dirtyParts.add((IE4SavablePart) part);
                }

                break;
            }else if (taxonUUID == null){
                if (part.isDirty() && (part.getObject() instanceof TaxonNameEditorE4 || (part.getObject() instanceof BulkEditorE4 && ((BulkEditorE4)part.getObject()).getEditorInput() instanceof TaxonEditorInput))){
                    dirtyParts.add((IE4SavablePart) part);
                }
            }
        }
        return dirtyParts;
    }

	/**
	 * An uninitialized taxon is one that hasn't been saved yet. As such, it
	 * should appear in neither the list of recent names nor in the taxonomic
	 * tree when opened.
	 *
	 * @param parentNodeUuid
	 *            a {@link java.util.UUID} object.
	 */
	public static void openEmptyE4(UUID parentNodeUuid) {
		TaxonEditorInputE4 input = TaxonEditorInputE4
				.NewEmptyInstance(parentNodeUuid);
        EPartService partService = TaxeditorEditorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getService(EPartService.class);
        MPart part = partService.createPart(NAME_EDITOR_ID);
        part = partService.showPart(part, PartState.ACTIVATE);
        TaxonNameEditorE4 editor = (TaxonNameEditorE4) part.getObject();
        editor.init(input);
	}

	/**
	 * <p>
	 * getUndoContext
	 * </p>
	 *
	 * @return a {@link org.eclipse.core.commands.operations.IUndoContext}
	 *         object.
	 */
	public static IUndoContext getUndoContext() {
		return IOperationHistory.GLOBAL_UNDO_CONTEXT;
	}

	/**
	 * <p>
	 * forceUserSave
	 * </p>
	 *
	 * @param editor
	 *            a {@link org.eclipse.ui.IEditorPart} object.
	 * @param shell
	 *            a {@link org.eclipse.swt.widgets.Shell} object.
	 * @return a boolean.
	 */
	public static boolean forceUserSave(IEditorPart editor, Shell shell) {
		if (editor.isDirty()) {

			boolean doSave = MessageDialog
					.openConfirm(shell, Messages.EditorUtil_COMFIRM_SAVE,
							Messages.EditorUtil_CONFIRM_SAVE_MESSAGE);

			if (!doSave) {
				return false;
			}

			editor.doSave(AbstractUtility.getMonitor());
		}
		return true;
	}

	public static boolean forceUserSaveE4Editor(TaxonNameEditorE4 editor, Shell shell) {
	    if (editor.isDirty()) {

	        boolean doSave = MessageDialog
	                .openConfirm(shell, Messages.EditorUtil_COMFIRM_SAVE,
	                        Messages.EditorUtil_CONFIRM_SAVE_MESSAGE);

	        if (!doSave) {
	            return false;
	        }

	        editor.save(AbstractUtility.getMonitor());
	    }
	    return true;
	}

	/**
	 * <p>
	 * getSelection
	 * </p>
	 *
	 * @param event
	 *            a {@link org.eclipse.core.commands.ExecutionEvent} object.
	 * @return a {@link org.eclipse.jface.viewers.IStructuredSelection} object.
	 */
	public static IStructuredSelection getSelection(ExecutionEvent event) {
		IEditorPart activeEditor = HandlerUtil.getActiveEditor(event);

		return (IStructuredSelection) activeEditor.getSite()
				.getSelectionProvider().getSelection();
	}

	/**
	 * <p>
	 * getPluginId
	 * </p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	public static String getPluginId() {
		return TaxeditorEditorPlugin.PLUGIN_ID;
	}

	/**
	 * Iterates recursively over all originals having the given specimen as a derivate.
	 * The first {@link DerivedUnit} with no more originals or the first {@link FieldUnit} is returned
	 * @param specimen the start element for which the originals are iterated recursively
	 * @return either a FieldUnit or a the topmost DerivedUnit (which can be itself)
	 */
	public static SpecimenOrObservationBase<?> getTopMostDerivate(SpecimenOrObservationBase<?> specimen){
	    if(specimen==null){
	        return null;
	    }
	    if(specimen.isInstanceOf(FieldUnit.class)){
	        return specimen;
	    }
	    else if(specimen.isInstanceOf(DerivedUnit.class)){
	        DerivedUnit derivedUnit = HibernateProxyHelper.deproxy(specimen, DerivedUnit.class);
	        if(derivedUnit.getOriginals()!=null
	                && !(derivedUnit.getOriginals().isEmpty())){
	            for(SpecimenOrObservationBase<?> original:((DerivedUnit) specimen).getOriginals()){
	                return getTopMostDerivate(original);
	            }
	        }
	    }
	    return specimen;
	}

    /**
     * If the current selection is a single {@link TreeNode} it will be returned.
     * @param selection the selection to check
     * @return the selected TreeNode or <code>null</code> if no TreeNode selected
     */
    public static TreeNode getTreeNodeOfSelection(ISelection selection){
        if(selection instanceof IStructuredSelection
                && ((IStructuredSelection) selection).size()==1
                && ((IStructuredSelection) selection).getFirstElement() instanceof TreeNode){
            return (TreeNode) ((IStructuredSelection) selection).getFirstElement();

        }
        return null;
    }

    public static void closeObsoleteEditor(TaxonNodeDto taxonNode, EPartService partService){
        String treeIndex = taxonNode.getTreeIndex();
        Collection<MPart> parts = partService.getParts();
        for (MPart part : parts) {
            Object object = part.getObject();
            if(object instanceof TaxonNameEditorE4){
                TaxonNameEditorE4 taxonEditor = (TaxonNameEditorE4)object;
                TaxonNode node = taxonEditor.getEditorInput().getTaxonNode();
                if (node.treeIndex()!= null){
	                if(node.treeIndex().startsWith(treeIndex)){
	                    if (part.isDirty()){
	                        forceUserSaveE4Editor(taxonEditor, getShell());
	                    }
	                    partService.hidePart(part);
	                }
                }else{
                	logger.debug("The taxonnode of taxon " + node.getTaxon().getTitleCache() + " uuid: " + node.getUuid() + " has no treeindex");;
                }
            }
        }
    }


    public static void closePart(String partID, EPartService partService){

        Collection<MPart> parts = partService.getParts();
        for (MPart part : parts) {
            String elementId = part.getElementId();
            if (elementId.equals(partID)){
                partService.hidePart(part);
            }
        }
    }

    public static void updateEditor(TaxonNode taxonNode, TaxonNameEditorE4 editor){
        String treeIndex = taxonNode.treeIndex();
        TaxonNode node = editor.getEditorInput().getTaxonNode();
        if(node.treeIndex().equals(treeIndex)){
        	TaxonEditorInputE4 input = TaxonEditorInputE4.NewInstance(node.getUuid());
        	editor.init(input);


        }
    }

}
