/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.editor.definedterm.DefinedTermSorter;
import eu.etaxonomy.taxeditor.editor.definedterm.TermDtoContentProvider;
import eu.etaxonomy.taxeditor.editor.definedterm.TermDtoLabelProvider;
import eu.etaxonomy.taxeditor.preference.wizard.CheckBoxTreeComposite;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author pplitzner
 * @since Oct 29, 2018
 *
 */
public class AreasSelectionDialog extends Dialog{

    private CheckBoxTreeComposite treeComposite;

    private List<TermDto> selectedAreas;

    private Collection<TermVocabularyDto> areaVocabularies;


    protected AreasSelectionDialog(Shell parentShell, List<TermDto> selectedAreas) {
        super(parentShell);
        this.selectedAreas = selectedAreas;
        this.areaVocabularies = CdmStore.getService(IVocabularyService.class).findVocabularyDtoByTermType(TermType.NamedArea);
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        treeComposite = new CheckBoxTreeComposite(parent, new TermDtoContentProvider(), new TermDtoLabelProvider(), SWT.NONE);
        treeComposite.getViewer().setComparator(new DefinedTermSorter());
        treeComposite.getViewer().setInput(areaVocabularies);
        Collections.sort(selectedAreas, (o1, o2)->o1.getOrderIndex()-o2.getOrderIndex());
        treeComposite.setCheckedElements(selectedAreas.toArray());
        GridLayoutFactory.fillDefaults().applyTo(treeComposite);
        return treeComposite;
    }

    @Override
    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText("Choose areas");
        newShell.setSize(400, 600);
    }

    @Override
    protected void okPressed() {
        selectedAreas.clear();
        List<Object> checkedElements = Arrays.asList(treeComposite.getViewer().getCheckedElements());
        checkedElements = checkedElements.stream()
                .filter(element->element instanceof TermDto && !treeComposite.getViewer().getGrayed(element))
                .collect(Collectors.toList());
        selectedAreas = new ArrayList(checkedElements);
        super.okPressed();
    }

    @Override
    protected boolean isResizable() {
        return true;
    }

    public List<TermDto> getSelectedAreas(){
        return selectedAreas;
    }
}
