/**
 * Copyright (C) 2017 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;

import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.description.Character;
import eu.etaxonomy.cdm.model.term.FeatureTree;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.remote.l10n.TermRepresentation_L10n;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.featuretree.e4.FeatureTreeEditorComposite;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.ui.combo.TermUuidComboViewer;
import eu.etaxonomy.taxeditor.util.TaxonTreeNodeContentProvider;
import eu.etaxonomy.taxeditor.util.TaxonTreeNodeLabelProvider;

/**
 * @author pplitzner
 * @since Nov 21, 2017
 *
 */
public class DescriptiveDataSetComposite extends Composite {
    private Text txtDescriptiveDataSet;
    private TermUuidComboViewer comboRankMin;
    private TermUuidComboViewer comboRankMax;
    private FeatureTreeEditorComposite<Character> featureTreeEditorComposite;
    private TreeViewer taxonNodeTree;
    private List<TermDto> areas = new ArrayList<>();
    private Text textAreaText;
    private Button btnChooseArea;
    private Button btnRemoveRankMin;
    private Button btnRemoveRankMax;

    public DescriptiveDataSetComposite(Composite parent, int style) {
        super(parent, style);

        setLayout(new GridLayout(3, false));

        Label lblNewLabel = new Label(this, SWT.NONE);
        lblNewLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblNewLabel.setText(Messages.DescriptiveDataSetComposite_LABEL);

        txtDescriptiveDataSet = new Text(this, SWT.BORDER);
        txtDescriptiveDataSet.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

        Label lblNewLabel_1 = new Label(this, SWT.NONE);
        lblNewLabel_1.setText(Messages.DescriptiveDataSetComposite_TAXON_FILTER);

        Label lblNewLabel_2 = new Label(this, SWT.NONE);
        lblNewLabel_2.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblNewLabel_2.setText(Messages.DescriptiveDataSetComposite_RANK_MIN);

        Composite composite_1 = new Composite(this, SWT.NONE);
        composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        GridLayout gl_composite_1 = new GridLayout(2, false);
        gl_composite_1.horizontalSpacing = 0;
        gl_composite_1.verticalSpacing = 0;
        gl_composite_1.marginWidth = 0;
        gl_composite_1.marginHeight = 0;
        composite_1.setLayout(gl_composite_1);

        comboRankMin = new TermUuidComboViewer(composite_1, SWT.NONE);
        comboRankMin.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        comboRankMin.setInput(TermType.Rank);

        btnRemoveRankMin = new Button(composite_1, SWT.NONE);
        btnRemoveRankMin.setImage(ImageResources.getImage(ImageResources.TRASH_ICON));

        Tree tree = new Tree(this, SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);
        tree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 4));
        taxonNodeTree = new TreeViewer(tree);

        taxonNodeTree.setContentProvider(new TaxonTreeNodeContentProvider());
        taxonNodeTree.setLabelProvider(new TaxonTreeNodeLabelProvider());

        Label lblNewLabel_3 = new Label(this, SWT.NONE);
        lblNewLabel_3.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblNewLabel_3.setText(Messages.DescriptiveDataSetComposite_RANK_MAX);

        Composite composite_2 = new Composite(this, SWT.NONE);
        composite_2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        GridLayout gl_composite_2 = new GridLayout(2, false);
        gl_composite_2.horizontalSpacing = 0;
        gl_composite_2.verticalSpacing = 0;
        gl_composite_2.marginWidth = 0;
        gl_composite_2.marginHeight = 0;
        composite_2.setLayout(gl_composite_2);

        comboRankMax = new TermUuidComboViewer(composite_2, SWT.NONE);
        comboRankMax.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        comboRankMax.setInput(TermType.Rank);

        btnRemoveRankMax = new Button(composite_2, SWT.NONE);
        btnRemoveRankMax.setImage(ImageResources.getImage(ImageResources.TRASH_ICON));

        Label lblNewLabel_4 = new Label(this, SWT.NONE);
        lblNewLabel_4.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblNewLabel_4.setText(Messages.DescriptiveDataSetComposite_AREA);

        Composite composite = new Composite(this, SWT.NONE);
        composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
        GridLayout gl_composite = new GridLayout(2, false);
        gl_composite.horizontalSpacing = 0;
        gl_composite.verticalSpacing = 0;
        gl_composite.marginHeight = 0;
        gl_composite.marginWidth = 0;
        composite.setLayout(gl_composite);

        textAreaText = new Text(composite, SWT.BORDER);
        textAreaText.setEditable(false);
        textAreaText.setText(Messages.DescriptiveDataSetComposite_CHOOSE);
        textAreaText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

        btnChooseArea = new Button(composite, SWT.NONE);
        btnChooseArea.setImage(ImageResources.getImage(ImageResources.BROWSE_ICON));

        featureTreeEditorComposite = new FeatureTreeEditorComposite<Character>(this, SWT.NONE);
        featureTreeEditorComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
    }

    @Override
    protected void checkSubclass() {
        // Disable the check that prevents subclassing of SWT components
    }
    public Text getTxt_label() {
        return txtDescriptiveDataSet;
    }

    public TermUuidComboViewer getRankMin() {
        return comboRankMin;
    }
    public TermUuidComboViewer getRankMax() {
        return comboRankMax;
    }

    public void setRankMin(Rank min) {
        comboRankMin.setElement(min);
    }
    public void setRankMax(Rank max) {
        comboRankMax.setElement(max);
    }

    public FeatureTreeEditorComposite<Character> getFeatureTreeEditorComposite() {
        return featureTreeEditorComposite;
    }

    public FeatureTree<Character> getCharacters(){
        return featureTreeEditorComposite.getFeatureTree();
    }
    public void setCharacters(FeatureTree<Character> characters) {
        featureTreeEditorComposite.setSelectedTree(characters);
    }

    public TreeViewer getTaxonNodeTree() {
        return taxonNodeTree;
    }
    public List<TermDto> getAreas() {
        return areas;
    }
    public void setAreas(List<TermDto> areas) {
        this.areas = areas;
        textAreaText.setText(areas.stream().map(area->{
            area.localize(new TermRepresentation_L10n());
            return area.getRepresentation_L10n();
        }).collect(Collectors.joining(", ")));
    }
    public Button getBtnChooseArea() {
        return btnChooseArea;
    }
    public Button getBtnRemoveRankMin() {
        return btnRemoveRankMin;
    }
    public Button getBtnRemoveRankMax() {
        return btnRemoveRankMax;
    }
}
