/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet;

import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.TransferData;

import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;

/**
 * @author pplitzner
 * @since Dec 19, 2017
 *
 */
public class TaxonNodeDropAdapter extends ViewerDropAdapter {

    private DescriptiveDataSetEditor descriptiveDataSetEditor;

    public TaxonNodeDropAdapter(DescriptiveDataSetEditor descriptiveDataSetEditor) {
        super(descriptiveDataSetEditor.getTaxonTreeViewer());
        this.descriptiveDataSetEditor = descriptiveDataSetEditor;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean performDrop(Object data) {
        TaxonNodeDto taxonNodeDto = (TaxonNodeDto) ((ITreeSelection) data).getFirstElement();
        descriptiveDataSetEditor.addTaxonNode(taxonNodeDto);
        descriptiveDataSetEditor.setFocus();
        return true;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean validateDrop(Object target, int operation, TransferData transferType) {
        ISelection selection = LocalSelectionTransfer.getTransfer().getSelection();
        if(selection instanceof ITreeSelection
                && ((IStructuredSelection) selection).size()==1
                && ((IStructuredSelection) selection).getFirstElement() instanceof TaxonNodeDto){
            return true;
        }
        return false;
    }


}
