/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.character;

import java.util.Collection;
import java.util.stream.Collectors;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.nebula.widgets.nattable.util.GUIHelper;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.model.description.Character;
import eu.etaxonomy.cdm.model.term.FeatureNode;
import eu.etaxonomy.taxeditor.featuretree.CharacterTransfer;
import eu.etaxonomy.taxeditor.featuretree.e4.FeatureNodeDragListener;

/**
 * @author pplitzner
 * @since Jun 2, 2017
 *
 */
public class CharacterDragListener extends FeatureNodeDragListener {

    private TreeViewer structureViewer;
    private TreeViewer propertyViewer;
    private Collection<Character> characters;
    private Image image;
    private GC gc;

    public CharacterDragListener(TreeViewer structureViewer, TreeViewer propertyViewer) {
        super(propertyViewer);
        this.structureViewer = structureViewer;
        this.propertyViewer = propertyViewer;
    }

    @Override
    public void dragFinished(DragSourceEvent event) {
        characters = null;
        if(image!=null){
            image.dispose();
            image = null;
        }
        if(gc!=null){
            gc.dispose();
            gc = null;
        }
        super.dragFinished(event);
    }

    @Override
    public void dragSetData(DragSourceEvent event) {
        if (CharacterTransfer.getInstance().isSupportedType(event.dataType)) {
            event.data = characters;
        }
    }

    @Override
    public void dragStart(DragSourceEvent event) {
        event.doit =
                !structureViewer.getSelection().isEmpty()
                && !propertyViewer.getSelection().isEmpty()
                && ((IStructuredSelection) structureViewer.getSelection()).toList()
                .stream().allMatch(element -> element instanceof FeatureNode)
                && ((IStructuredSelection) propertyViewer.getSelection()).toList()
                .stream().allMatch(element -> element instanceof FeatureNode);
        if(!event.doit){
            return;
        }
        TreeSelection structureSelection = (TreeSelection) structureViewer.getSelection();
        TreeSelection propertySelection = (TreeSelection) propertyViewer.getSelection();
        if(structureSelection!=null && propertySelection!=null
                && !structureSelection.isEmpty() && !propertySelection.isEmpty()){
            //create all possible characters
            characters = CharacterEditor.createCharacters(structureSelection, propertySelection);

        }
        String text = characters.stream().map(character->character.generateTitle()).collect(Collectors.joining("\n"));

        // creating temporary GC for size calculation
        gc = new GC(Display.getDefault());
        // Calculate text extent
        Point compositeSize = gc.textExtent(text);
        //remove temporary gc
        gc.dispose();
        gc = null;
        // Creating new Image
        image = new Image(Display.getCurrent(),compositeSize.x+10,compositeSize.y+10);
        //create gc for image to be able to draw on it
        gc = new GC(image);
        //set and fill with bg color
        gc.setBackground(GUIHelper.COLOR_LIST_BACKGROUND);
        gc.fillRectangle(0, 0, compositeSize.x+10, compositeSize.y+10);
        //draw text
        gc.drawText(text, 5, 5, true);
        //draw border
        gc.drawRectangle(0, 0, compositeSize.x+9, compositeSize.y+9);
        // Setting widget to DnD image
        event.image = image;

    }

}
