/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.character;

import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TransferData;

import eu.etaxonomy.cdm.api.service.IFeatureNodeService;
import eu.etaxonomy.cdm.model.description.Character;
import eu.etaxonomy.cdm.model.term.FeatureNode;
import eu.etaxonomy.cdm.model.term.FeatureTree;
import eu.etaxonomy.taxeditor.featuretree.CharacterTransfer;
import eu.etaxonomy.taxeditor.featuretree.e4.FeatureTreeDropAdapter;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 *
 * @author pplitzner
 * @date Jun 1, 2017
 *
 */
public class CharacterDropAdapter extends FeatureTreeDropAdapter {

	private CharacterEditor characterEditor;

    protected CharacterDropAdapter(CharacterEditor characterEditor, Viewer viewer, UISynchronize sync) {
	    super(characterEditor, viewer, sync);
	    this.characterEditor = characterEditor;
	}

	@Override
	protected Object determineTarget(DropTargetEvent event) {
        return event.item == null ? ((FeatureTree)getViewer().getInput()).getRoot() : event.item.getData();
	}

	@Override
	public boolean performDrop(Object data) {
	    if(!CharacterTransfer.getInstance().isSupportedType(getCurrentEvent().currentDataType)){
	        return super.performDrop(data);
	    }
	    if(StoreUtil.promptCheckIsDirty(editor)){
	        return false;
	    }
	    if(!characterEditor.isVocabularySet()){
	        return false;
	    }
	    FeatureNode target = (FeatureNode) getCurrentTarget();
	    if(data instanceof Object[]){
	        Object[] transferObjects = (Object[])data;
	        for(int i=0;i<transferObjects.length;i++){
	            Object object = transferObjects[i];
	            if(object instanceof Character){
	                Character character = (Character)object;
	                if(target==null){
                        //drop on feature tree
                        target = ((FeatureTree)getViewer().getInput()).getRoot();
                    }
                    CdmStore.getService(IFeatureNodeService.class).createChildFeatureNode(target.getUuid(), character, characterEditor.getVocabulary().getUuid());
	            }
	        }
	        getViewer().refresh();
	        return true;
	    }
	    return false;
	}

	@Override
	public boolean validateDrop(Object target, int operation,
			TransferData transferData) {
		if(CharacterTransfer.getInstance().isSupportedType(transferData)){
		    return true;
		}
		return super.validateDrop(target, operation, transferData);
	}

}
