/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.math3.stat.Frequency;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.State;

/**
 * @author pplitzner
 * @since Jul 12, 2018
 *
 */
public class CategoricalDataHistogram {

    private Feature feature;
    private Frequency frequency;
    private List<State> states;

    public CategoricalDataHistogram(Feature feature) {
        this.feature = feature;
        this.states = new ArrayList<>();
        this.frequency = new Frequency();
        feature.getSupportedCategoricalEnumerations()
        .forEach(voc->voc.getTerms()
                .forEach(state->states.add(state)));
    }

    public void addState(State state){
        frequency.addValue(state.getLabel());
    }

    public long getCount(State state){
        return frequency.getCount(state.getLabel());
    }

    public List<State> getStates() {
        return states;
    }

    public Feature getFeature() {
        return feature;
    }
}
