// $Id$
/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.nebula.widgets.nattable.config.AbstractRegistryConfiguration;
import org.eclipse.nebula.widgets.nattable.config.CellConfigAttributes;
import org.eclipse.nebula.widgets.nattable.config.IConfigRegistry;
import org.eclipse.nebula.widgets.nattable.config.IEditableRule;
import org.eclipse.nebula.widgets.nattable.edit.EditConfigAttributes;
import org.eclipse.nebula.widgets.nattable.edit.gui.ICellEditDialog;
import org.eclipse.nebula.widgets.nattable.style.DisplayMode;
import org.eclipse.swt.graphics.Point;

import eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.categorical.CategoricalComboBoxDataProvider;
import eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.categorical.CategoricalDataCellEditor;
import eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.categorical.CategoricalDataDisplayConverter;
import eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.quantitative.QuantitativeDataDialogEditor;
import eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.quantitative.QuantitativeDataEditModeDisplayConverter;
import eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.quantitative.QuantitativeDataNormalModeDisplayConverter;
import eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.supplementalInfo.SupplementalInfoDisplayConverter;

/**
 * @author pplitzner
 * @date 09.07.2018
 *
 */
final class CellEditorDataConversionConfiguration extends AbstractRegistryConfiguration {

    private CharacterMatrix matrix;

    public CellEditorDataConversionConfiguration(CharacterMatrix matrix) {
        super();
        this.matrix = matrix;
    }

    @Override
    public void configureRegistry(IConfigRegistry configRegistry) {

        //add display converter for string representation
        configRegistry.registerConfigAttribute(
                CellConfigAttributes.DISPLAY_CONVERTER,
                new SupplementalInfoDisplayConverter(),
                DisplayMode.NORMAL,
                CharacterMatrix.TAXON_COLUMN);
        configRegistry.registerConfigAttribute(
                CellConfigAttributes.DISPLAY_CONVERTER,
                new SupplementalInfoDisplayConverter(),
                DisplayMode.NORMAL,
                CharacterMatrix.COLLECTOR_COLUMN);
        configRegistry.registerConfigAttribute(
                CellConfigAttributes.DISPLAY_CONVERTER,
                new SupplementalInfoDisplayConverter(),
                DisplayMode.NORMAL,
                CharacterMatrix.IDENTIFIER_COLUMN);
        configRegistry.registerConfigAttribute(
                CellConfigAttributes.DISPLAY_CONVERTER,
                new SupplementalInfoDisplayConverter(),
                DisplayMode.NORMAL,
                CharacterMatrix.COUNTRY_COLUMN);

        //---register cell editors---

        /**
         * categorical cell editor
         */
        //make cell editable
        configRegistry.registerConfigAttribute(
                EditConfigAttributes.CELL_EDITABLE_RULE,
                IEditableRule.ALWAYS_EDITABLE,
                DisplayMode.EDIT,
                CharacterMatrixConfigLabelAccumulator.CATEGORICAL_EDITABLE
                );
        //add display converter for string representation
        configRegistry.registerConfigAttribute(
                CellConfigAttributes.DISPLAY_CONVERTER,
                new CategoricalDataDisplayConverter(),
                DisplayMode.NORMAL,
                CharacterMatrixConfigLabelAccumulator.CATEGORICAL
                );
        //register categorical editor
        configRegistry.registerConfigAttribute(EditConfigAttributes.CELL_EDITOR,
                new CategoricalDataCellEditor(matrix, new CategoricalComboBoxDataProvider(matrix)),
                DisplayMode.EDIT,
                CharacterMatrixConfigLabelAccumulator.CATEGORICAL_EDITABLE
                );


        /**
         * quantitative cell editor
         */
        //make cell editable
        configRegistry.registerConfigAttribute(
                EditConfigAttributes.CELL_EDITABLE_RULE,
                IEditableRule.ALWAYS_EDITABLE,
                DisplayMode.EDIT,
                CharacterMatrixConfigLabelAccumulator.QUANTITATIVE_EDITABLE
                );
        //add display converter for string representation
        configRegistry.registerConfigAttribute(
                CellConfigAttributes.DISPLAY_CONVERTER,
                new QuantitativeDataEditModeDisplayConverter(),
                DisplayMode.EDIT,
                CharacterMatrixConfigLabelAccumulator.QUANTITATIVE_EDITABLE);
        configRegistry.registerConfigAttribute(
                CellConfigAttributes.DISPLAY_CONVERTER,
                new QuantitativeDataNormalModeDisplayConverter(),
                DisplayMode.NORMAL,
                CharacterMatrixConfigLabelAccumulator.QUANTITATIVE);

        //Open cell editor in dialog
        configRegistry.registerConfigAttribute(
                EditConfigAttributes.OPEN_IN_DIALOG,
                Boolean.TRUE,
                DisplayMode.EDIT,
                CharacterMatrixConfigLabelAccumulator.QUANTITATIVE_EDITABLE);
        //register quantitative editor
        configRegistry.registerConfigAttribute(
                EditConfigAttributes.CELL_EDITOR,
                new QuantitativeDataDialogEditor(matrix),
                DisplayMode.EDIT,
                CharacterMatrixConfigLabelAccumulator.QUANTITATIVE_EDITABLE);
        // configure custom dialog settings
        Map<String, Object> editDialogSettings = new HashMap<>();
        editDialogSettings.put(ICellEditDialog.DIALOG_SHELL_TITLE, "Quantitative Data");
        editDialogSettings.put(ICellEditDialog.DIALOG_SHELL_RESIZABLE, Boolean.TRUE);

        Point size = new Point(250, 300);
        editDialogSettings.put(ICellEditDialog.DIALOG_SHELL_SIZE, size);

        //FIXME: this is not displayed because the cell editor
        //is already a dialog. Maybe the cell editor and the
        //dialog can be merged
        editDialogSettings.put(ICellEditDialog.DIALOG_MESSAGE, "Enter exact values:");

        configRegistry.registerConfigAttribute(
                EditConfigAttributes.EDIT_DIALOG_SETTINGS,
                editDialogSettings,
                DisplayMode.EDIT,
                CharacterMatrixConfigLabelAccumulator.QUANTITATIVE_EDITABLE);
    }

}
