// $Id$
/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix;

import java.util.Collection;

import org.eclipse.e4.ui.workbench.modeling.ESelectionService;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.nebula.widgets.nattable.data.IRowDataProvider;
import org.eclipse.nebula.widgets.nattable.extension.e4.selection.E4SelectionListener;
import org.eclipse.nebula.widgets.nattable.layer.cell.ILayerCell;
import org.eclipse.nebula.widgets.nattable.layer.event.ILayerEvent;
import org.eclipse.nebula.widgets.nattable.selection.SelectionLayer;
import org.eclipse.nebula.widgets.nattable.selection.event.CellSelectionEvent;
import org.eclipse.nebula.widgets.nattable.selection.event.RowSelectionEvent;

import eu.etaxonomy.cdm.api.service.dto.RowWrapperDTO;

/**
 * @author pplitzner
 * @date 09.07.2018
 *
 */
final class CellSelectionListener extends E4SelectionListener<Object> {

    private CharacterMatrixPart part;

    public CellSelectionListener(
            ESelectionService service,
            SelectionLayer selectionLayer,
            IRowDataProvider<Object> rowDataProvider,
            CharacterMatrixPart part) {
        super(service, selectionLayer, rowDataProvider);
        this.part = part;
    }

    @Override
    public void handleLayerEvent(ILayerEvent event) {
        if(event instanceof CellSelectionEvent){
            CellSelectionEvent cellSelectionEvent = (CellSelectionEvent)event;
            int columnPosition = cellSelectionEvent.getColumnPosition();
            if(columnPosition>=CharacterMatrix.LEADING_COLUMN_COUNT){
                Collection<ILayerCell> selectedCells = cellSelectionEvent.getSelectionLayer().getSelectedCells();
                if(selectedCells.size()==1){
                    ILayerCell cell = selectedCells.iterator().next();
                    Object dataValue = cell.getDataValue();
                    if(dataValue!=null){
                        part.getSelectionService().setSelection(new StructuredSelection(dataValue));
                        return;
                    }
                }
            }
        }
        else if(event instanceof RowSelectionEvent){
            RowSelectionEvent rowSelectionEvent = (RowSelectionEvent) event;
            int[] fullySelectedRowPositions = rowSelectionEvent.getSelectionLayer().getFullySelectedRowPositions();
            if(fullySelectedRowPositions.length==1){
                Object rowObject = part.getMatrix().getBodyDataProvider().getRowObject(fullySelectedRowPositions[0]);
                if(rowObject instanceof RowWrapperDTO){
                    part.getSelectionService().setSelection(new StructuredSelection(((RowWrapperDTO) rowObject).getDescription()));
                    return;
                }
            }
        }
        part.getSelectionService().setSelection(new StructuredSelection());
    }
}
