/**
 * Copyright (C) 2017 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.UUID;
import java.util.stream.Collectors;

import javax.inject.Inject;

import org.apache.commons.collections4.map.LinkedMap;
import org.eclipse.core.runtime.ICoreRunnable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.services.EMenuService;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.nebula.widgets.nattable.NatTable;
import org.eclipse.nebula.widgets.nattable.config.AbstractUiBindingConfiguration;
import org.eclipse.nebula.widgets.nattable.config.ConfigRegistry;
import org.eclipse.nebula.widgets.nattable.config.DefaultNatTableStyleConfiguration;
import org.eclipse.nebula.widgets.nattable.coordinate.PositionCoordinate;
import org.eclipse.nebula.widgets.nattable.coordinate.Range;
import org.eclipse.nebula.widgets.nattable.copy.command.InternalCopyDataCommandHandler;
import org.eclipse.nebula.widgets.nattable.copy.command.InternalPasteDataCommandHandler;
import org.eclipse.nebula.widgets.nattable.data.IDataProvider;
import org.eclipse.nebula.widgets.nattable.data.ListDataProvider;
import org.eclipse.nebula.widgets.nattable.export.command.ExportCommandHandler;
import org.eclipse.nebula.widgets.nattable.extension.e4.selection.E4SelectionListener;
import org.eclipse.nebula.widgets.nattable.extension.glazedlists.GlazedListsEventLayer;
import org.eclipse.nebula.widgets.nattable.extension.glazedlists.GlazedListsSortModel;
import org.eclipse.nebula.widgets.nattable.extension.glazedlists.tree.GlazedListTreeData;
import org.eclipse.nebula.widgets.nattable.extension.glazedlists.tree.GlazedListTreeRowModel;
import org.eclipse.nebula.widgets.nattable.freeze.CompositeFreezeLayer;
import org.eclipse.nebula.widgets.nattable.freeze.FreezeHelper;
import org.eclipse.nebula.widgets.nattable.freeze.FreezeLayer;
import org.eclipse.nebula.widgets.nattable.grid.GridRegion;
import org.eclipse.nebula.widgets.nattable.grid.command.ClientAreaResizeCommand;
import org.eclipse.nebula.widgets.nattable.grid.data.DefaultColumnHeaderDataProvider;
import org.eclipse.nebula.widgets.nattable.grid.data.DefaultCornerDataProvider;
import org.eclipse.nebula.widgets.nattable.grid.data.DefaultRowHeaderDataProvider;
import org.eclipse.nebula.widgets.nattable.grid.data.FixedSummaryRowHeaderLayer;
import org.eclipse.nebula.widgets.nattable.grid.layer.ColumnHeaderLayer;
import org.eclipse.nebula.widgets.nattable.grid.layer.CornerLayer;
import org.eclipse.nebula.widgets.nattable.grid.layer.DefaultRowHeaderDataLayer;
import org.eclipse.nebula.widgets.nattable.grid.layer.GridLayer;
import org.eclipse.nebula.widgets.nattable.layer.AbstractLayer;
import org.eclipse.nebula.widgets.nattable.layer.CompositeLayer;
import org.eclipse.nebula.widgets.nattable.layer.DataLayer;
import org.eclipse.nebula.widgets.nattable.layer.ILayer;
import org.eclipse.nebula.widgets.nattable.layer.cell.ColumnOverrideLabelAccumulator;
import org.eclipse.nebula.widgets.nattable.layer.stack.DefaultBodyLayerStack;
import org.eclipse.nebula.widgets.nattable.selection.SelectionLayer;
import org.eclipse.nebula.widgets.nattable.sort.SortHeaderLayer;
import org.eclipse.nebula.widgets.nattable.sort.config.SingleClickSortConfiguration;
import org.eclipse.nebula.widgets.nattable.style.HorizontalAlignmentEnum;
import org.eclipse.nebula.widgets.nattable.style.theme.ModernNatTableThemeConfiguration;
import org.eclipse.nebula.widgets.nattable.summaryrow.FixedSummaryRowLayer;
import org.eclipse.nebula.widgets.nattable.summaryrow.SummaryRowLayer;
import org.eclipse.nebula.widgets.nattable.tree.ITreeRowModel;
import org.eclipse.nebula.widgets.nattable.tree.TreeLayer;
import org.eclipse.nebula.widgets.nattable.ui.binding.UiBindingRegistry;
import org.eclipse.nebula.widgets.nattable.ui.matcher.MouseEventMatcher;
import org.eclipse.nebula.widgets.nattable.ui.menu.PopupMenuAction;
import org.eclipse.nebula.widgets.nattable.ui.menu.PopupMenuBuilder;
import org.eclipse.nebula.widgets.nattable.util.GUIHelper;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;

import ca.odell.glazedlists.BasicEventList;
import ca.odell.glazedlists.EventList;
import ca.odell.glazedlists.SortedList;
import ca.odell.glazedlists.TreeList;
import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.api.service.dto.RowWrapperDTO;
import eu.etaxonomy.cdm.api.service.dto.SpecimenRowWrapperDTO;
import eu.etaxonomy.cdm.common.monitor.IRemotingProgressMonitor;
import eu.etaxonomy.cdm.model.description.Character;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptiveDataSet;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.term.FeatureNode;
import eu.etaxonomy.cdm.model.term.FeatureTree;
import eu.etaxonomy.cdm.model.description.MeasurementUnit;
import eu.etaxonomy.cdm.model.description.State;
import eu.etaxonomy.cdm.persistence.dto.SpecimenNodeWrapper;
import eu.etaxonomy.taxeditor.editor.internal.TaxeditorEditorPlugin;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.session.ICdmEntitySession;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.workbench.WorkbenchUtility;

/**
 * Character matrix editor for editing specimen/taxon descriptions in a table
 * @author pplitzner
 * @since Nov 26, 2017
 *
 */
public class CharacterMatrix extends Composite {

    private static final String CHARACTER_MATRIX_STATE_PROPERTIES = "characterMatrixState.properties"; //$NON-NLS-1$

    static final int LEADING_COLUMN_COUNT = 4;
    static final String TAXON_COLUMN = "taxon_column"; //$NON-NLS-1$
    static final String COLLECTOR_COLUMN = "collector_column"; //$NON-NLS-1$
    static final String IDENTIFIER_COLUMN = "identifier_column"; //$NON-NLS-1$
    static final String COUNTRY_COLUMN = "country_column"; //$NON-NLS-1$

    static final String LABEL_TAXON_ROW = "TAXON_ROW"; //$NON-NLS-1$
    static final String LABEL_TAXON_AGGREGATED_DESCRIPTION = "TAXON_AGGREGATED_DESCRIPTION"; //$NON-NLS-1$
    static final String LABEL_TAXON_AGGREGATED_DESCRIPTION_ICON = "TAXON_AGGREGATED_DESCRIPTION_ICON"; //$NON-NLS-1$
    static final String LABEL_TAXON_DEFAULT_DESCRIPTION = "TAXON_DEFAULT_DESCRIPTION"; //$NON-NLS-1$
    static final String LABEL_TAXON_DEFAULT_DESCRIPTION_ICON = "TAXON_DEFAULT_DESCRIPTION_ICON"; //$NON-NLS-1$
    static final String LABEL_TAXON_LITERATURE_DESCRIPTION = "TAXON_LITERATURE_DESCRIPTION"; //$NON-NLS-1$
    static final String LABEL_TAXON_LITERATURE_DESCRIPTION_ICON = "TAXON_LITERATURE_DESCRIPTION_ICON"; //$NON-NLS-1$
    static final String LABEL_TAXON_DESCRIPTION = "LABEL_TAXON_DESCRIPTION"; //$NON-NLS-1$
    static final String LABEL_DESCRIPTION_HAS_SUPPLEMENTAL_DATA = "LABEL_DESCRIPTION_HAS_SUPPLEMENTAL_DATA"; //$NON-NLS-1$

    @Inject
    private UISynchronize sync;

    @Inject
    private EMenuService menuService;

    private DescriptiveDataSet descriptiveDataSet;

    private NatTable natTable;

    private Map<Integer, Character> indexToCharacterMap = new HashMap<>();

    private Map<Feature, List<State>> categoricalFeatureToStateMap = new HashMap<>();

    private LinkedMap<String, String> propertyToLabelMap = new LinkedMap<>();

    private EventList<Object> descriptions;

    private Collection<SpecimenNodeWrapper> specimenCache = null;

    private Map<Feature, CategoricalDataHistogram> featureToHistogramMap = new HashMap<>();

    private Map<Feature, QuantitativeDataStatistics> featureToQuantDataStatisticsMap = new HashMap<>();

    private ListDataProvider<Object> bodyDataProvider;

    private FreezeLayer freezeLayer;

    private List<Character> characters;

    private CharacterMatrixPart part;

    private AbstractLayer topMostLayer;

    private FixedSummaryRowLayer summaryRowLayer;

    private ConfigRegistry configRegistry;

    private DefaultBodyLayerStack bodyLayer;

    private boolean isTreeView = true;

    private CharacterMatrixToolbar toolbar;

    private boolean isShowTooltips = true;

    public CharacterMatrix(Composite parent, CharacterMatrixPart part) {
        super(parent, SWT.NONE);
        this.part = part;
        this.setLayout(new GridLayout());

        createToolBar();

        natTable = new NatTable(this, false);

        createBottomToolbar();

    }

    private void createToolBar(){
        toolbar = new CharacterMatrixToolbar(this, SWT.NONE);
    }

    @SuppressWarnings("unused")
    private void createBottomToolbar() {
        new CharacterMatrixBottomToolbar(this, SWT.NONE);
    }


    private void applyStyles(){
        ModernNatTableThemeConfiguration configuration = new ModernNatTableThemeConfiguration();
        configuration.summaryRowHAlign = HorizontalAlignmentEnum.CENTER;
        // NOTE: Getting the colors and fonts from the GUIHelper ensures that
        // they are disposed properly (required by SWT)
        configuration.cHeaderBgColor = GUIHelper.getColor(211, 211, 211);
        configuration.rHeaderBgColor = GUIHelper.getColor(211, 211, 211);
        natTable.addConfiguration(configuration);

    }

    void toggleTreeFlat(boolean isTree, Button btnToggleFlat, Button btnToggleTree, Button btnCollapseAll, Button btnExpandAll, Button btnFreezeSuppInfo) {
        isTreeView = isTree;
        createTable(isTree, freezeLayer.isFrozen());
        btnToggleFlat.setEnabled(isTree);
        btnToggleTree.setEnabled(!isTree);
        btnCollapseAll.setEnabled(isTree);
        btnExpandAll.setEnabled(isTree);
    }

    public boolean isTreeView() {
        return isTreeView;
    }

    public void createTable(boolean treeView, boolean freezeSupplementalColumns){
        /**
         * layers
         */
        createLayers(treeView);

        /**
         * configuration
         */
        configureNatTable(treeView, configRegistry, topMostLayer, summaryRowLayer);

        /**
         * handlers and listeners
         */
        registerHandlersAndListeners(topMostLayer);

        //grab all space
        GridDataFactory.fillDefaults().grab(true, true).applyTo(natTable);

        //update label to current data set
        toolbar.getWsLabel().setText(descriptiveDataSet.getLabel());
        toolbar.getWsLabel().setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        toolbar.getWsLabel().getParent().layout();

        //initial freeze of supplemental columns
        freezeSupplementalColumns(freezeSupplementalColumns);


        //add tooltip to table
        new CategoricalChartTooltip(this);
        new QuantitativeChartTooltip(this);

        this.layout();
        natTable.doCommand(new ClientAreaResizeCommand(natTable));
    }

    private List<Character> initCharacterList(FeatureNode<Character> node){
        List<Character> characters = new ArrayList<>();
        node.getChildNodes().forEach(childNode->
                {
                    characters.add(childNode.getTerm());
                    characters.addAll(initCharacterList(childNode));
                });
        return characters;
    }

    public void initDescriptiveDataSet(DescriptiveDataSet descriptiveDataSet){
        this.descriptiveDataSet = descriptiveDataSet;
        //get features/columns stored in descriptive data set
        FeatureTree<Character> tree = descriptiveDataSet.getDescriptiveSystem();
        characters = initCharacterList(tree.getRoot());

        //init state data for categorical features
        characters.forEach(character->
        {
            if(character.isSupportsCategoricalData()){
                List<State> supportedStates = new ArrayList<>();
                character.getSupportedCategoricalEnumerations().forEach(voc->supportedStates.addAll(voc.getTerms()));
                categoricalFeatureToStateMap.put(character, supportedStates);
            }
        });
        descriptions = new BasicEventList<>();

    }

    private void createLayers(boolean treeView) {
        SortedList<Object> sortedList = new SortedList<>(descriptions, new MatrixRowComparator());
        // wrap the SortedList with the TreeList
        TreeList<Object> treeList = new TreeList(sortedList, new DescriptionTreeFormat(descriptiveDataSet), TreeList.NODES_START_EXPANDED);
        /**
         * data provider
         */
        SpecimenColumnPropertyAccessor columnPropertyAccessor = new SpecimenColumnPropertyAccessor(this);
        bodyDataProvider = treeView?new ListDataProvider<>(treeList, columnPropertyAccessor):new ListDataProvider<>(sortedList, columnPropertyAccessor);

        configRegistry = new ConfigRegistry();


        /**
         * BODY layer
         *
         *

        CompositeLayer
         - (top) SummaryRowLayer
         - (bottom) ViewportLayer

             ^
        ViewportLayer

             ^
        TreeLayer (default visible)

             ^
        CompositeFreezeLayer
         - viewportLayer
         - selectionLayer
         - freezeLayer

             ^
        FreezeLayer

             ^
        SelectionLayer

             ^
        ColumnHideShowLayer

             ^
        ColumnReorderLayer

             ^
        DataLayer

         *

         */
        DataLayer bodyDataLayer = new DataLayer(bodyDataProvider);
        bodyDataLayer.registerCommandHandler(new CopyPasteUpdateDataCommandHandler(bodyDataLayer));

        //register labels
        CharacterMatrixConfigLabelAccumulator labelAccumulator = new CharacterMatrixConfigLabelAccumulator(this);
        bodyDataLayer.setConfigLabelAccumulator(labelAccumulator);


        propertyToLabelMap.put(TAXON_COLUMN, Messages.CharacterMatrix_TAXON);
        propertyToLabelMap.put(COLLECTOR_COLUMN, Messages.CharacterMatrix_COLLECTOR_NO);
        propertyToLabelMap.put(IDENTIFIER_COLUMN, Messages.CharacterMatrix_IDENTIFIER);
        propertyToLabelMap.put(COUNTRY_COLUMN, Messages.CharacterMatrix_COUNTRY);
        for(int i=0;i<characters.size();i++){
            Character character = characters.get(i);
            initLabels(i, character);
        }

        // layer for event handling of GlazedLists and PropertyChanges
        GlazedListsEventLayer eventLayer = new GlazedListsEventLayer<>(bodyDataLayer, treeList);
        GlazedListTreeData treeData = new GlazedListTreeData<>(treeList);
        ITreeRowModel treeRowModel = new GlazedListTreeRowModel<>(treeData);

        bodyLayer = new DefaultBodyLayerStack(eventLayer);
        final SelectionLayer selectionLayer = bodyLayer.getSelectionLayer();
        freezeLayer = new FreezeLayer(selectionLayer);
        final CompositeFreezeLayer compositeFreezeLayer = new CompositeFreezeLayer(
                freezeLayer, bodyLayer.getViewportLayer(), selectionLayer);
        TreeLayer treeLayer = new TreeLayer(compositeFreezeLayer, treeRowModel);

        topMostLayer = treeView?treeLayer:compositeFreezeLayer;

        summaryRowLayer = new FixedSummaryRowLayer(bodyDataLayer, topMostLayer, configRegistry, false);
        //regoster labels with summary prefix for summary layer
        ColumnOverrideLabelAccumulator summaryColumnLabelAccumulator =new ColumnOverrideLabelAccumulator(bodyDataLayer);
        summaryRowLayer.setConfigLabelAccumulator(summaryColumnLabelAccumulator);
        for(int i=0;i<characters.size();i++){
            Character character = characters.get(i);
            summaryColumnLabelAccumulator.registerColumnOverrides(
                    i+LEADING_COLUMN_COUNT,
                    SummaryRowLayer.DEFAULT_SUMMARY_COLUMN_CONFIG_LABEL_PREFIX+MatrixUtility.getProperty(character));
        }
        // because the horizontal dependency is the ViewportLayer
        // we need to set the composite dependency to false
        summaryRowLayer.setHorizontalCompositeDependency(false);

        CompositeLayer composite = new CompositeLayer(1, 2);
        composite.setChildLayer("SUMMARY", summaryRowLayer, 0, 0); //$NON-NLS-1$
        composite.setChildLayer(GridRegion.BODY, topMostLayer, 0, 1);


        /**
         * column header layer
         */
        IDataProvider columnHeaderDataProvider = new DefaultColumnHeaderDataProvider(
                propertyToLabelMap.values().toArray(new String[] {}), propertyToLabelMap);
        DataLayer columnHeaderDataLayer = new DataLayer(columnHeaderDataProvider);
        ColumnHeaderLayer columnHeaderLayer = new ColumnHeaderLayer(columnHeaderDataLayer, topMostLayer, selectionLayer);

        // add the SortHeaderLayer to the column header layer stack
        // as we use GlazedLists, we use the GlazedListsSortModel which
        // delegates the sorting to the SortedList
        final SortHeaderLayer<DescriptionBase> sortHeaderLayer = new SortHeaderLayer<>(
                columnHeaderLayer,
                new GlazedListsSortModel<>(
                        sortedList,
                        columnPropertyAccessor,
                        configRegistry,
                        columnHeaderDataLayer));


        /**
         * row header layer
         */
        IDataProvider rowHeaderDataProvider = new DefaultRowHeaderDataProvider(bodyDataProvider);
        DefaultRowHeaderDataLayer rowHeaderDataLayer = new DefaultRowHeaderDataLayer(rowHeaderDataProvider);
        FixedSummaryRowHeaderLayer fixedSummaryRowHeaderLayer = new FixedSummaryRowHeaderLayer(rowHeaderDataLayer,
                composite, selectionLayer);
        fixedSummaryRowHeaderLayer.setSummaryRowLabel("\u2211"); //$NON-NLS-1$


        /**
         * corner layer
         */
        ILayer cornerLayer = new CornerLayer(
                new DataLayer(new DefaultCornerDataProvider(columnHeaderDataProvider, rowHeaderDataProvider)),
                fixedSummaryRowHeaderLayer, sortHeaderLayer);


        /**
         * GRID layer (composition of all other layers)
         */
        GridLayer gridLayer = new GridLayer(composite, sortHeaderLayer, fixedSummaryRowHeaderLayer, cornerLayer);

        natTable.setLayer(gridLayer);

    }

    private void registerHandlersAndListeners(AbstractLayer topMostLayer) {
        // add the ExportCommandHandler to the ViewportLayer in order to make
        // exporting work
        topMostLayer.registerCommandHandler(new ExportCommandHandler(topMostLayer));

        //selection listener
        E4SelectionListener selectionListener = new CellSelectionListener(part.getSelectionService(),
                bodyLayer.getSelectionLayer(), bodyDataProvider, part);
        bodyLayer.getSelectionLayer().addLayerListener(selectionListener);
        selectionListener.setFullySelectedRowsOnly(false);

        //register handler for view configuration menu
        natTable.registerCommandHandler(toolbar.getDisplayPersistenceDialogCommandHandler());

        //register handlers for copy&paste
        natTable.registerCommandHandler(
                new InternalPasteDataCommandHandler(bodyLayer.getSelectionLayer(), natTable.getInternalCellClipboard()));
        natTable.registerCommandHandler(
                new InternalCopyDataCommandHandler(bodyLayer.getSelectionLayer(), natTable.getInternalCellClipboard()));
    }

    private void configureNatTable(boolean treeView,
            ConfigRegistry configRegistry,
            AbstractLayer topMostLayer,
            FixedSummaryRowLayer summaryRowLayer) {
        /**
         * CONFIGURATION
         */
        natTable.setConfigRegistry(configRegistry);

        applyStyles();

        //add default configuration because autoconfigure is set to false in constructor
        natTable.addConfiguration(new DefaultNatTableStyleConfiguration());

        // this is for DEBUG ONLY
        //        natTable.addConfiguration(new DebugMenuConfiguration(natTable));

        // override the default sort configuration and change the mouse bindings
        // to sort on a single click
        if(!treeView){
            natTable.addConfiguration(new SingleClickSortConfiguration());
        }

        natTable.addConfiguration(new CharacterMatrixLabelStyleConfiguration());

        // add the header menu configuration for adding the column header menu
        // with hide/show actions
        natTable.addConfiguration(new CharacterMatrixHeaderMenuConfiguration(natTable));

        // add custom configuration for data conversion and add column labels to viewport layer
        topMostLayer.addConfiguration(new CellEditorDataConversionConfiguration(this));

        //register aggregation configuration
        summaryRowLayer.addConfiguration(new AggregationConfiguration(this));

        //copy&paste configuration
        natTable.addConfiguration(new CopyPasteEditBindings(bodyLayer.getSelectionLayer(), natTable.getInternalCellClipboard()));

      //+++CONTEXT MENU+++
        menuService.registerContextMenu(natTable, "eu.etaxonomy.taxeditor.editor.popupmenu.charactermatrix"); //$NON-NLS-1$
        // get the menu registered by EMenuService
        final Menu e4Menu = natTable.getMenu();
        // remove the menu reference from NatTable instance
        natTable.setMenu(null);
        natTable.addConfiguration(
                new AbstractUiBindingConfiguration() {
            @Override
            public void configureUiBindings(
                    UiBindingRegistry uiBindingRegistry) {
                // add e4 menu to NatTable
                new PopupMenuBuilder(natTable, e4Menu)
                    .build();

                // register the UI binding for header, corner and body region
                uiBindingRegistry.registerMouseDownBinding(
                        new MouseEventMatcher(
                                SWT.NONE,
                                null,
                                MouseEventMatcher.RIGHT_BUTTON),
                        new PopupMenuAction(e4Menu));
            }
        });

        natTable.configure();
    }

    void freezeSupplementalColumns(boolean freeze){
        if(freeze){
            FreezeHelper.freeze(freezeLayer, bodyLayer.getViewportLayer(),
                    new PositionCoordinate(bodyLayer.getViewportLayer(), 0, 0),
                    new PositionCoordinate(bodyLayer.getViewportLayer(), LEADING_COLUMN_COUNT-1, -1));
        }
        else{
            FreezeHelper.unfreeze(freezeLayer, bodyLayer.getViewportLayer());
        }
    }

    void selectStateItem(ComboViewer comboStates, String stateName){
        String[] items = comboStates.getCombo().getItems();
        for(int i=0;i<items.length;i++){
            if(items[i].equals(stateName)){
                comboStates.getCombo().select(i);
                break;
            }
        }
    }

    private void initLabels(int index, Character character) {
        indexToCharacterMap.put(index+LEADING_COLUMN_COUNT, character);

        String label = character.getLabel();
        String property = label;
        //show unit for quantitative data
        if(character.isSupportsQuantitativeData()){
            Set<MeasurementUnit> recommendedMeasurementUnits = character.getRecommendedMeasurementUnits();
            if(recommendedMeasurementUnits.size()>1){
                MessagingUtils.warningDialog(Messages.CharacterMatrix_INIT_PROBLEM, CharacterMatrix.class,
                        String.format(Messages.CharacterMatrix_INIT_PROBLEM_MESSAGE, character.getLabel()));
            }
            if(recommendedMeasurementUnits.size()==1){
                MeasurementUnit unit = recommendedMeasurementUnits.iterator().next();
                label += " ["+unit.getIdInVocabulary()+"]"; //$NON-NLS-1$ //$NON-NLS-2$
            }
        }
        propertyToLabelMap.put(property, label);
    }

    public void loadDescriptions(DescriptiveDataSet descriptiveDataSet) {
        UUID monitorUuid =  CdmApplicationState.getLongRunningTasksService().monitGetRowWrapper(descriptiveDataSet);

        final Collection<RowWrapperDTO> wrappers = new ArrayList<>();
        String jobLabel = Messages.CharacterMatrix_LOAD_CHARACTER_DATA;
        Job job = Job.create(jobLabel, (ICoreRunnable) monitor -> {
            SubMonitor subMonitor = SubMonitor.convert(monitor);
            subMonitor.beginTask(jobLabel, IProgressMonitor.UNKNOWN);
            IRemotingProgressMonitor remotingMonitor;
            try {
                 remotingMonitor = CdmStore.getProgressMonitorClientManager()
                .pollMonitor(jobLabel,
                        monitorUuid,
                        50,
                        null,
                        (List)null,
                        subMonitor);
            } catch (InterruptedException e) {
                MessagingUtils.informationDialog(Messages.CharacterMatrix_LOADING_FAILED_TITLE,
                        Messages.CharacterMatrix_LOADING_FAILED_MESSAGE);
                return;
            }
            Object result = remotingMonitor.getResult();
            if(result instanceof Collection){
                wrappers.addAll((Collection<RowWrapperDTO>) result);
            }
            if(result instanceof Exception){
                MessagingUtils.errorDialog("Exception during description loading", this.getClass(), "An exception occured during loading", TaxeditorEditorPlugin.PLUGIN_ID, (Throwable) result, true);
            }
            else if(wrappers.isEmpty()){
                MessagingUtils.informationDialog(Messages.CharacterMatrix_NO_DESCRIPTION_TITLE,
                        Messages.CharacterMatrix_NO_DESCRIPTION_MESSAGE);
            }
            monitor.done();
        });
        job.addJobChangeListener(new JobChangeAdapter(){
            @Override
            public void done(IJobChangeEvent event) {
                sync.syncExec(()->{
                    List<RowWrapperDTO> rowsWithoutTaxonNode = wrappers.stream().filter(row->row.getTaxonNode()==null).collect(Collectors.toList());
                    if(!rowsWithoutTaxonNode.isEmpty()){
                        String collect = rowsWithoutTaxonNode.stream().
                        map(row->row.getDescription().toString())
                        .collect(Collectors.joining("\n\n - ")); //$NON-NLS-1$
                        MessagingUtils.warningDialog(
                                Messages.CharacterMatrix_NO_NODE_FOUND_TITLE,
                                this.getClass(),
                                String.format(Messages.CharacterMatrix_NO_NODE_FOUND_MESSAGE, collect)
                                );
                    }
                    wrappers.stream().filter(row->row.getTaxonNode()!=null).forEach(wrapper->CharacterMatrix.this.descriptions.add(wrapper));
                    loadingDone();
                });
            }
        });
        job.schedule();
    }

    public IStructuredSelection getSelection(){
        Set<Range> selectedRowPositions = bodyLayer.getSelectionLayer().getSelectedRowPositions();
        List<Object> selectedObjects = new ArrayList<>();
        for (Range range : selectedRowPositions) {
            for(int i=range.start;i<range.end;i++){
                selectedObjects.add(bodyDataProvider.getRowObject(i));
            }
        }
        return new StructuredSelection(selectedObjects);
    }

    private void loadingDone() {
        this.part.loadingDone();
        createTable(isTreeView, freezeLayer.isFrozen());
    }

    public List<State> getSupportedStatesForCategoricalFeature(Feature feature){
        return categoricalFeatureToStateMap.get(feature);
    }

    public Map<Integer, Character> getIndexToCharacterMap() {
        return indexToCharacterMap;
    }

    public LinkedMap<String, String> getPropertyToLabelMap() {
        return propertyToLabelMap;
    }

    public void setDirty() {
        part.setDirty();
    }

    public CharacterMatrixPart getPart() {
        return part;
    }

    public NatTable getNatTable() {
        return natTable;
    }

    public EventList<Object> getDescriptions() {
        return descriptions;
    }

    public DescriptiveDataSet getDescriptiveDataSet() {
        return descriptiveDataSet;
    }

    public Collection<SpecimenNodeWrapper> getSpecimenCache() {
        return specimenCache;
    }

    public void setSpecimenCache(Collection<SpecimenNodeWrapper> specimenCache) {
        this.specimenCache = specimenCache.stream()
                .filter(wrapper ->
        //map descriptions on a list of uuids of the described specimen
        !this.descriptions.stream()
        .filter(rowWrapper->rowWrapper instanceof SpecimenRowWrapperDTO)
        .map(specimenRowWrapper->((SpecimenRowWrapperDTO) specimenRowWrapper).getSpecimen().getUuid())
        .collect(Collectors.toList())
        //an check if the specimen to add is already contained
        .contains(wrapper.getUuidAndTitleCache().getUuid())
        )
        .collect(Collectors.toList());
    }

    public Properties getNatTableState() {
        return toolbar.getNatTableState();
    }

    public ListDataProvider<Object> getBodyDataProvider() {
        return bodyDataProvider;
    }

    DefaultBodyLayerStack getBodyLayer() {
        return bodyLayer;
    }

    File getStatePropertiesFile() {
        return new File(WorkbenchUtility.getBaseLocation(), CHARACTER_MATRIX_STATE_PROPERTIES);
    }

    public List<Character> getFeatures() {
        return characters;
    }

    public Map<Feature, CategoricalDataHistogram> getFeatureToHistogramMap() {
        return featureToHistogramMap;
    }

    public Map<Feature, QuantitativeDataStatistics> getFeatureToQuantDataStatisticsMap() {
        return featureToQuantDataStatisticsMap;
    }

    public void toogleIsShowTooltips() {
        this.isShowTooltips = !this.isShowTooltips;
    }

    public boolean isShowTooltips() {
        return isShowTooltips;
    }

    public ICdmEntitySession getCdmEntitiySession(){
        return part.getCdmEntitySession();
    }

}
