/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix;

import org.eclipse.nebula.widgets.nattable.layer.LabelStack;
import org.eclipse.nebula.widgets.nattable.layer.cell.IConfigLabelAccumulator;

import eu.etaxonomy.cdm.api.service.dto.TaxonRowWrapperDTO;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;

/**
 * @author pplitzner
 * @since Jul 6, 2018
 *
 */
public class CharacterMatrixConfigLabelAccumulator implements IConfigLabelAccumulator {

    public static final String QUANTITATIVE = "QUANTITATIVE";
    public static final String CATEGORICAL = "CATEGORICAL";
    public static final String CATEGORICAL_EDITABLE = CATEGORICAL+"_EDITABLE";
    public static final String QUANTITATIVE_EDITABLE = QUANTITATIVE+"_EDITABLE";

    private CharacterMatrix matrix;

    public CharacterMatrixConfigLabelAccumulator(CharacterMatrix matrix) {
        super();
        this.matrix = matrix;
    }

    @Override
    public void accumulateConfigLabels(LabelStack configLabels, int columnPosition, int rowPosition) {
        boolean isEditable = true;

        Object rowObject = matrix.getBodyDataProvider().getRowObject(rowPosition);
        if(rowObject instanceof TaxonNodeDto){
            configLabels.addLabel(CharacterMatrix.LABEL_TAXON_ROW);
        }
        else if(rowObject instanceof TaxonRowWrapperDTO){
            TaxonRowWrapperDTO taxonRowWrapper = (TaxonRowWrapperDTO) rowObject;
            if(MatrixUtility.isAggregatedTaxonDescription(taxonRowWrapper)){
                configLabels.addLabel(CharacterMatrix.LABEL_TAXON_AGGREGATED_DESCRIPTION);
                isEditable = false;
                if(columnPosition==0){
                    configLabels.addLabel(CharacterMatrix.LABEL_TAXON_AGGREGATED_DESCRIPTION_ICON);
                }
            }
            else if(MatrixUtility.isDefaultTaxonDescription(taxonRowWrapper)){
                configLabels.addLabel(CharacterMatrix.LABEL_TAXON_DEFAULT_DESCRIPTION);
                if(columnPosition==0){
                    configLabels.addLabel(CharacterMatrix.LABEL_TAXON_DEFAULT_DESCRIPTION_ICON);
                }
            }
            else if(MatrixUtility.isLiteratureTaxonDescription(taxonRowWrapper)){
                configLabels.addLabel(CharacterMatrix.LABEL_TAXON_LITERATURE_DESCRIPTION);
                if(columnPosition==0){
                    configLabels.addLabel(CharacterMatrix.LABEL_TAXON_LITERATURE_DESCRIPTION_ICON);
                }
            }
            //check for supplemental data
            if(!taxonRowWrapper.getDescription().getSources().isEmpty() && columnPosition==0){
                configLabels.addLabel(CharacterMatrix.LABEL_DESCRIPTION_HAS_SUPPLEMENTAL_DATA);
            }
            configLabels.addLabel(CharacterMatrix.LABEL_TAXON_DESCRIPTION);
        }

        if(columnPosition==0){
            configLabels.addLabel(CharacterMatrix.TAXON_COLUMN);
        }
        else if(columnPosition==1){
            configLabels.addLabel(CharacterMatrix.COLLECTOR_COLUMN);
        }
        else if(columnPosition==2){
            configLabels.addLabel(CharacterMatrix.IDENTIFIER_COLUMN);
        }
        else if(columnPosition==3){
            configLabels.addLabel(CharacterMatrix.COUNTRY_COLUMN);
        }
        else{
            Feature feature = matrix.getFeatures().get(columnPosition-CharacterMatrix.LEADING_COLUMN_COUNT);
            configLabels.addLabel(MatrixUtility.getProperty(feature));
            if(feature.isSupportsCategoricalData()){
                configLabels.addLabel(CATEGORICAL);
                if(isEditable){
                    configLabels.addLabel(CATEGORICAL_EDITABLE);
                }
            }
            else if(feature.isSupportsQuantitativeData()){
                configLabels.addLabel(QUANTITATIVE);
                if(isEditable){
                    configLabels.addLabel(QUANTITATIVE_EDITABLE);
                }
            }
        }
    }
}
