/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix;

import org.eclipse.nebula.widgets.nattable.config.AbstractRegistryConfiguration;
import org.eclipse.nebula.widgets.nattable.config.CellConfigAttributes;
import org.eclipse.nebula.widgets.nattable.config.IConfigRegistry;
import org.eclipse.nebula.widgets.nattable.painter.cell.ImagePainter;
import org.eclipse.nebula.widgets.nattable.painter.cell.TextPainter;
import org.eclipse.nebula.widgets.nattable.painter.cell.decorator.CellPainterDecorator;
import org.eclipse.nebula.widgets.nattable.style.CellStyleAttributes;
import org.eclipse.nebula.widgets.nattable.style.DisplayMode;
import org.eclipse.nebula.widgets.nattable.style.Style;
import org.eclipse.nebula.widgets.nattable.ui.util.CellEdgeEnum;
import org.eclipse.nebula.widgets.nattable.util.GUIHelper;

import eu.etaxonomy.taxeditor.model.ImageResources;

/**
 * @author pplitzner
 * @since Jul 6, 2018
 *
 */
public final class CharacterMatrixLabelStyleConfiguration extends AbstractRegistryConfiguration {
    @Override
    public void configureRegistry(IConfigRegistry configRegistry) {
        //different background color styles
        Style taxonRowStyle = new Style();
        taxonRowStyle.setAttributeValue(
                CellStyleAttributes.BACKGROUND_COLOR,
                GUIHelper.COLOR_WIDGET_BACKGROUND);
        configRegistry.registerConfigAttribute(
                CellConfigAttributes.CELL_STYLE,
                taxonRowStyle,
                DisplayMode.NORMAL,
                CharacterMatrix.LABEL_TAXON_ROW);

        Style taxonAggregationStyle = new Style();
        taxonAggregationStyle.setAttributeValue(
                CellStyleAttributes.BACKGROUND_COLOR,
                GUIHelper.getColor(255, 255, 153));
        configRegistry.registerConfigAttribute(
                CellConfigAttributes.CELL_STYLE,
                taxonAggregationStyle,
                DisplayMode.NORMAL,
                CharacterMatrix.LABEL_TAXON_AGGREGATED_DESCRIPTION);

        Style taxonDefaultDescriptionStyle = new Style();
        taxonDefaultDescriptionStyle.setAttributeValue(
                CellStyleAttributes.BACKGROUND_COLOR,
                GUIHelper.COLOR_WIDGET_BACKGROUND);
        configRegistry.registerConfigAttribute(
                CellConfigAttributes.CELL_STYLE,
                taxonDefaultDescriptionStyle,
                DisplayMode.NORMAL,
                CharacterMatrix.LABEL_TAXON_DEFAULT_DESCRIPTION);

        Style taxonLiteratureDescriptionStyle = new Style();
        taxonLiteratureDescriptionStyle.setAttributeValue(
                CellStyleAttributes.BACKGROUND_COLOR,
                GUIHelper.getColor(231, 231, 231));
        configRegistry.registerConfigAttribute(
                CellConfigAttributes.CELL_STYLE,
                taxonLiteratureDescriptionStyle,
                DisplayMode.NORMAL,
                CharacterMatrix.LABEL_TAXON_LITERATURE_DESCRIPTION);

        //cell painter for supplemental data
        configRegistry.registerConfigAttribute(
                CellConfigAttributes.CELL_PAINTER,
                new CellPainterDecorator(
                        new TextPainter(),
                        CellEdgeEnum.BOTTOM_RIGHT,
                        new ImagePainter(GUIHelper.getImage("plus")),
                        false),
                DisplayMode.NORMAL,
                CharacterMatrix.LABEL_DESCRIPTION_HAS_SUPPLEMENTAL_DATA);

        /*
         * cell painter for description icons
         */
        //literature
        configRegistry.registerConfigAttribute(
                CellConfigAttributes.CELL_PAINTER,
                new CellPainterDecorator(
                        new TextPainter(),
                        CellEdgeEnum.LEFT,
                        new ImagePainter(ImageResources.getImage(ImageResources.HELP_TOPIC)),
                        true),
                DisplayMode.NORMAL,
                CharacterMatrix.LABEL_TAXON_LITERATURE_DESCRIPTION_ICON);
        //aggregation
        configRegistry.registerConfigAttribute(
                CellConfigAttributes.CELL_PAINTER,
                new CellPainterDecorator(
                        new TextPainter(),
                        CellEdgeEnum.LEFT,
                        new ImagePainter(ImageResources.getImage(ImageResources.FUNNEL_ICON)),
                        true),
                DisplayMode.NORMAL,
                CharacterMatrix.LABEL_TAXON_AGGREGATED_DESCRIPTION_ICON);
        //default
        configRegistry.registerConfigAttribute(
                CellConfigAttributes.CELL_PAINTER,
                new CellPainterDecorator(
                        new TextPainter(),
                        CellEdgeEnum.LEFT,
                        new ImagePainter(ImageResources.getImage(ImageResources.VALIDATE_ICON)),
                        true),
                DisplayMode.NORMAL,
                CharacterMatrix.LABEL_TAXON_DEFAULT_DESCRIPTION_ICON);


    }
}