/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix;

import org.eclipse.nebula.widgets.nattable.copy.InternalCellClipboard;
import org.eclipse.nebula.widgets.nattable.copy.action.ClearClipboardAction;
import org.eclipse.nebula.widgets.nattable.copy.action.PasteDataAction;
import org.eclipse.nebula.widgets.nattable.copy.action.PasteOrMoveSelectionAction;
import org.eclipse.nebula.widgets.nattable.edit.action.DeleteSelectionAction;
import org.eclipse.nebula.widgets.nattable.edit.config.DefaultEditBindings;
import org.eclipse.nebula.widgets.nattable.selection.SelectionLayer;
import org.eclipse.nebula.widgets.nattable.ui.binding.UiBindingRegistry;
import org.eclipse.nebula.widgets.nattable.ui.matcher.KeyEventMatcher;
import org.eclipse.swt.SWT;

/**
 * @author pplitzner
 * @since Dec 20, 2018
 *
 */
public class CopyPasteEditBindings extends DefaultEditBindings {

    private SelectionLayer selectionLayer;

    private InternalCellClipboard clipboard;

    public CopyPasteEditBindings(SelectionLayer selectionLayer, InternalCellClipboard clipboard) {
        this.selectionLayer = selectionLayer;
        this.clipboard = clipboard;
    }

    @Override
    public void configureUiBindings(UiBindingRegistry uiBindingRegistry) {
        super.configureUiBindings(uiBindingRegistry);
        // ui binding for deleting a cell value on pressing DEL
        uiBindingRegistry.registerFirstKeyBinding(
                new KeyEventMatcher(SWT.DEL),
                new DeleteSelectionAction());

        // ui binding to perform a paste action on pressing CTRL+V
        uiBindingRegistry.registerFirstKeyBinding(
                new KeyEventMatcher(SWT.MOD1, 'v'),
                new PasteDataAction());

        // ui binding to perform paste or selection movement on ENTER
        uiBindingRegistry.registerFirstKeyBinding(
                new KeyEventMatcher(SWT.NONE, SWT.CR),
                new PasteOrMoveSelectionAction(this.clipboard));

        // ui binding to clear the InternalCellClipboard
        uiBindingRegistry.registerFirstKeyBinding(
                new KeyEventMatcher(SWT.NONE, SWT.ESC),
                new ClearClipboardAction(this.clipboard));

        // Mouse drag
        // trigger the handle drag operations
        // Note: we ensure a FillHandleLayerPainter is set in configureLayer
//        uiBindingRegistry.registerFirstMouseDragMode(
//                new FillHandleEventMatcher((FillHandleLayerPainter) this.selectionLayer.getLayerPainter()),
//                new FormulaFillHandleDragMode(this.selectionLayer, this.clipboard, this.dataProvider));
    }
}
