// $Id$
/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix;

import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import ca.odell.glazedlists.TreeList;
import eu.etaxonomy.cdm.api.service.ITaxonNodeService;
import eu.etaxonomy.cdm.api.service.dto.RowWrapperDTO;
import eu.etaxonomy.cdm.model.description.DescriptiveDataSet;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * The tree format defines how to build the hierarchy path for the
 * taxon column in the character matrix
 *
 * @author pplitzner
 * @date 23.01.2018
 *
 */
 public class DescriptionTreeFormat implements TreeList.Format<Object> {

     private Rank maxRank;

     private Map<UUID, TaxonNodeDto> uuidToTaxonNodeDtoMap = new HashMap<>();

     public DescriptionTreeFormat(DescriptiveDataSet descriptiveDataSet) {
         this.maxRank = descriptiveDataSet.getMaxRank();
     }

     @Override
     public void getPath(List path, Object element) {
         if(element instanceof RowWrapperDTO){
             //TODO: check for multiple taxon nodes in multiple classifications
             TaxonNodeDto taxonNode = ((RowWrapperDTO) element).getTaxonNode();
             //use dto from map cache because the elements have to be
             //exactly the same for the tree format to work
             if(uuidToTaxonNodeDtoMap.get(taxonNode.getUuid())!=null){
                 taxonNode = uuidToTaxonNodeDtoMap.get(taxonNode.getUuid());
             }
             else{
                 uuidToTaxonNodeDtoMap.put(taxonNode.getUuid(), taxonNode);
             }
             addPathRecursive(path, taxonNode);
         }
         path.add(element);
     }

     private void addPathRecursive(List path, TaxonNodeDto node){
         if(node!=null){
             if(maxRank!=null && node.getRankOrderIndex()==maxRank.getOrderIndex()){
                 path.add(node);
                 return;
             }
             TaxonNodeDto parentNodeDto = uuidToTaxonNodeDtoMap.get(node.getParentUUID());
             if(parentNodeDto==null){
                 TaxonNode parent = CdmStore.getService(ITaxonNodeService.class).load(node.getParentUUID());
                 if(parent!=null){
                     parentNodeDto = new TaxonNodeDto(parent);
                     uuidToTaxonNodeDtoMap.put(node.getParentUUID(), parentNodeDto);
                 }
             }
             if(parentNodeDto!=null){
                 addPathRecursive(path, parentNodeDto);
                 path.add(node);
             }
         }
     }

     @Override
     public boolean allowsChildren(Object element) {
         return true;
     }

     @Override
     public Comparator<Object> getComparator(int depth) {
         return new MatrixRowComparator();
     }

 }
