/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix;

import java.util.Comparator;

import eu.etaxonomy.cdm.api.service.ITaxonNodeService;
import eu.etaxonomy.cdm.api.service.dto.RowWrapperDTO;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.SpecimenDescription;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDtoByNameComparator;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDtoByRankAndNameComparator;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDtoNaturalComparator;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author pplitzner
 * @since Jan 25, 2018
 *
 */
public class MatrixRowComparator implements Comparator<Object>{

    private Comparator<TaxonNodeDto> comparator;
    private ITaxonNodeService taxonNodeService;

    public MatrixRowComparator() {
        taxonNodeService = CdmStore.getService(ITaxonNodeService.class);
        if (PreferencesUtil.isNodesSortedNaturally()){
            comparator = new TaxonNodeDtoNaturalComparator();
        } else if (PreferencesUtil.isNodesSortedByName()){
            comparator = new TaxonNodeDtoByNameComparator();
        }else {
            comparator = new TaxonNodeDtoByRankAndNameComparator();
        }
    }

    @Override
    public int compare(Object o1, Object o2) {
        if(o1 instanceof TaxonNodeDto){
            if(o2 instanceof RowWrapperDTO){
                return 1;
            }
        }
        else{
            if(o2 instanceof TaxonNodeDto){
                return -1;
            }
        }
        if(o1 instanceof TaxonNodeDto && o2 instanceof TaxonNodeDto){
            return comparator.compare((TaxonNodeDto)o1, (TaxonNodeDto)o2);
        }
        if(o1 instanceof RowWrapperDTO && o2 instanceof RowWrapperDTO){
            RowWrapperDTO rowWrapper1 = (RowWrapperDTO)o1;
            RowWrapperDTO rowWrapper2 = (RowWrapperDTO)o2;
            //check for equality
            if(rowWrapper1.equals(rowWrapper2)){
                return 0;
            }
            TaxonNodeDto node1 = rowWrapper1.getTaxonNode();
            TaxonNodeDto node2 = rowWrapper2.getTaxonNode();
            if(node1!=null && node2!=null){
                //compare by taxon node
                int compare = comparator.compare(node1, node2);
                if(compare!=0){
                    return compare;
                }
                DescriptionBase description1 = rowWrapper1.getDescription();
                DescriptionBase description2 = rowWrapper2.getDescription();
                //compare by taxon description type
                if(description1 instanceof TaxonDescription){
                    if(description2 instanceof SpecimenDescription){
                        return -1;
                    }
                }
                else{
                    if(description2 instanceof TaxonDescription){
                        return 1;
                    }
                }
                //both descriptions are of the same type
                if(description1 instanceof SpecimenDescription){
                    //description2 has to also be a SpecimenDescription
                    SpecimenDescription specimenDescription1 = (SpecimenDescription)description1;
                    SpecimenDescription specimenDescription2 = (SpecimenDescription)description2;
                    int id1 = specimenDescription1.getDescribedSpecimenOrObservation().getId();
                    int id2 = specimenDescription2.getDescribedSpecimenOrObservation().getId();
                    return id1-id2;
                }
                else if(description1 instanceof TaxonDescription){
                    //description2 has to also be a TaxonDescription
                    TaxonDescription taxonDescription1 = (TaxonDescription)description1;
                    TaxonDescription taxonDescription2 = (TaxonDescription)description2;
                    boolean isComputed1 = taxonDescription1.getMarkers().stream()
                            .anyMatch(marker -> marker.getMarkerType().equals(MarkerType.COMPUTED()));
                    boolean isComputed2 = taxonDescription2.getMarkers().stream()
                            .anyMatch(marker -> marker.getMarkerType().equals(MarkerType.COMPUTED()));
                    if(isComputed1){
                        if(!isComputed2){
                            return -1;
                        }
                    }
                    if(isComputed2){
                        return 1;
                    }
                    //TODO: implement compare for different description types
                }
            }
        }
        return o1.hashCode()-o2.hashCode();
    }

}
