/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.commons.math3.stat.descriptive.DescriptiveStatistics;

import eu.etaxonomy.cdm.model.description.QuantitativeData;
import eu.etaxonomy.cdm.model.description.StatisticalMeasure;

/**
 * @author pplitzner
 * @since Jul 18, 2018
 *
 */
public class QuantitativeDataStatistics {

    private List<DescriptiveStatistics> statistics = new ArrayList<>();

    public void addQuantitativeData(QuantitativeData quantitativeData){
        double[] exactValues = quantitativeData.getStatisticalValues().stream()
                .filter(value -> value.getType().equals(StatisticalMeasure.EXACT_VALUE()))
                .mapToDouble(value -> value.getValue()).toArray();
        //if there are exact values we use those
        if (exactValues.length > 0) {
            statistics.add(new DescriptiveStatistics(exactValues));
        }
        //otherwise we use the aggregated values (min, max, etc.)
        else{
            DescriptiveStatistics aggregatedStatistics = new DescriptiveStatistics();
            if(quantitativeData.getMin()!=null){
                aggregatedStatistics.addValue(quantitativeData.getMin().doubleValue());
            }
            if(quantitativeData.getMax()!=null){
                aggregatedStatistics.addValue(quantitativeData.getMax().doubleValue());
            }
            statistics.add(aggregatedStatistics);
        }
        Collections.sort(statistics, (s1, s2)->Double.compare(s1.getMean(), s2.getMean()));
    }

    public List<DescriptiveStatistics> getStatistics() {
        return statistics;
    }

}
