/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix;

import org.eclipse.nebula.widgets.nattable.data.IColumnPropertyAccessor;

import eu.etaxonomy.cdm.api.service.dto.RowWrapperDTO;
import eu.etaxonomy.cdm.api.service.dto.SpecimenRowWrapperDTO;
import eu.etaxonomy.cdm.api.service.dto.TaxonRowWrapperDTO;
import eu.etaxonomy.cdm.model.description.Feature;

/**
 * Property accessor class which maps setting and getting data for
 * each row in the character matrix
 * @author pplitzner
 * @since Nov 26, 2017
 *
 */
public class SpecimenColumnPropertyAccessor implements IColumnPropertyAccessor<Object>{

    private CharacterMatrix matrix;

    public SpecimenColumnPropertyAccessor(CharacterMatrix matrix) {
        this.matrix = matrix;
    }

    @Override
    public Object getDataValue(Object rowObject, int columnIndex) {
        if(rowObject instanceof SpecimenRowWrapperDTO){
            SpecimenRowWrapperDTO rowWrapper = (SpecimenRowWrapperDTO) rowObject;
            switch (columnIndex) {
            case 0:
                if(matrix.isTreeView()){
                    return "#"+rowWrapper.getSpecimen().getId();
                }
                else{
                    return rowWrapper.getTaxonNode();
                }
            case 1:
                return rowWrapper.getFieldUnit();
            case 2:
                return rowWrapper.getIdentifier();
            case 3:
                return rowWrapper.getCountry();

            default:
                break;
            }
            Feature feature = matrix.getIndexToCharacterMap().get(columnIndex);
            return rowWrapper.getDataValueForFeature(feature);
        } else if(rowObject instanceof TaxonRowWrapperDTO){
            TaxonRowWrapperDTO taxonWrapper = (TaxonRowWrapperDTO)rowObject;
            if(columnIndex==0){
                return taxonWrapper.getDescription();
            }
            Feature feature = matrix.getIndexToCharacterMap().get(columnIndex);
            return taxonWrapper.getDataValueForFeature(feature);

        }
        else if (columnIndex == 0) {
            return rowObject;
        }
        return null;
    }

    @Override
    public void setDataValue(Object rowObject, int columnIndex, Object newValue) {
        if(rowObject instanceof RowWrapperDTO){
            RowWrapperDTO rowWrapper = (RowWrapperDTO)rowObject;
            Feature feature = matrix.getIndexToCharacterMap().get(columnIndex);
            rowWrapper.setDataValueForFeature(feature, newValue);
        }
    }

    @Override
    public int getColumnCount() {
        return matrix.getPropertyToLabelMap().size();
    }

    @Override
    public String getColumnProperty(int columnIndex) {
        return matrix.getPropertyToLabelMap().get(columnIndex);
    }

    @Override
    public int getColumnIndex(String propertyName){
        return matrix.getPropertyToLabelMap().indexOf(propertyName);
    }

}
