/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.categorical;

import org.eclipse.nebula.widgets.nattable.edit.editor.ComboBoxCellEditor;
import org.eclipse.nebula.widgets.nattable.widget.NatCombo;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.api.service.dto.RowWrapperDTO;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.CharacterMatrix;

/**
 * Cell editor for CategoricalData
 * @author pplitzner
 * @since Dec 7, 2017
 *
 */
public class CategoricalDataCellEditor extends ComboBoxCellEditor{

    private CharacterMatrix matrix;

    public CategoricalDataCellEditor(CharacterMatrix matrix, CategoricalComboBoxDataProvider categoricalComboBoxDataProvider) {
        super(categoricalComboBoxDataProvider, categoricalComboBoxDataProvider.getMaxVisibleItems());
        this.matrix = matrix;
        setUseCheckbox(true);
        setMultiselect(true);
    }

    @Override
    public void setCanonicalValue(Object canonicalValue) {
        if(canonicalValue==null){
            Object rowWrapper = matrix.getBodyDataProvider().getRowObject(this.getRowIndex());
            if(rowWrapper instanceof RowWrapperDTO){
                Feature feature = matrix.getIndexToCharacterMap().get(getColumnIndex());
                ((RowWrapperDTO) rowWrapper).addCategoricalData(feature);
            }
        }
        super.setCanonicalValue(canonicalValue);
    }

    @Override
    protected Control activateCell(Composite parent, Object originalCanonicalValue) {
        if(matrix.getBodyDataProvider().getRowObject(this.getRowIndex()) instanceof RowWrapperDTO){
            NatCombo natCombo = (NatCombo) super.activateCell(parent, originalCanonicalValue);
            natCombo.addSelectionListener(new SelectionListener() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                    matrix.setDirty();
                }
                @Override
                public void widgetDefaultSelected(SelectionEvent e) {
                }
            });
            return natCombo;
        }
        return null;
    }

}
