/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.categorical;

import java.util.Collection;

import org.eclipse.nebula.widgets.nattable.data.convert.DisplayConverter;

import eu.etaxonomy.cdm.model.description.CategoricalData;
import eu.etaxonomy.cdm.model.description.State;

/**
 * Converts CategoricalData and States of one cell of the character matrix to a
 * String.
 *
 * @author pplitzner
 * @since Dec 1, 2017
 *
 */
public class CategoricalDataDisplayConverter extends DisplayConverter {

    /**
     * {@inheritDoc}
     */
    @Override
    public Object canonicalToDisplayValue(Object canonicalValue) {
        if(canonicalValue instanceof CategoricalData){
            CategoricalData categoricalData = (CategoricalData)canonicalValue;
            String result = categoricalData.getStatesOnly().toString();
            result = result.substring(1, result.length() - 1);
            return result;
        }
        else if(canonicalValue instanceof State){
            return ((State) canonicalValue).getLabel();
        }
        else if (canonicalValue instanceof Collection) {
            // Collection.toString() will add [ and ] around
            // the values in the Collection
            // So by removing the leading and ending
            // character, we remove the brackets
            String result = canonicalValue.toString();
            result = result.substring(1, result.length() - 1);
            return result;
        }
        else if(canonicalValue!=null){
            return canonicalValue.toString();
        }
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Object displayToCanonicalValue(Object displayValue) {
        return null;
    }

}
