/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.handler;

import javax.inject.Named;

import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;

import eu.etaxonomy.cdm.api.service.IDescriptiveDataSetService;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.description.DescriptiveDataSet;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;
import eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.CharacterMatrixPart;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author pplitzner
 * @since Sep 7, 2018
 *
 */
public class CreateDefaultTaxonDescriptionHandler extends CreateTaxonDescriptionHandler{

    @Override
    protected boolean getMarkerFlag() {
        return true;
    }

    @Override
    protected MarkerType getMarkerType() {
        //FIXME #7957 Use actual marker type when implemented
        return MarkerType.TO_BE_CHECKED();
    }

    @Override
    public boolean canExecute(@Named(IServiceConstants.ACTIVE_PART)MPart activePart,
            MHandledMenuItem menuItem){
        boolean canExecute = super.canExecute(activePart, menuItem);
        if(canExecute){
            CharacterMatrixPart matrixPart = (CharacterMatrixPart) activePart.getObject();
            DescriptiveDataSet descriptiveDataSet = matrixPart.getDescriptiveDataSet();
            IStructuredSelection selection = ((CharacterMatrixPart)activePart.getObject()).getSelection();
            TaxonNodeDto taxonNodeDto = (TaxonNodeDto) selection.getFirstElement();

            TaxonDescription defaultTaxonDescription = CdmStore.getService(IDescriptiveDataSetService.class)
                    .findTaxonDescriptionByMarkerType(descriptiveDataSet.getUuid(), taxonNodeDto.getUuid(), getMarkerType());
            canExecute = defaultTaxonDescription==null;
            menuItem.setVisible(canExecute);
        }

        return canExecute;
    }
}
