/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.handler;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;

import eu.etaxonomy.cdm.api.service.IDescriptiveDataSetService;
import eu.etaxonomy.cdm.api.service.dto.TaxonRowWrapperDTO;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.description.DescriptiveDataSet;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;
import eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.CharacterMatrixPart;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author pplitzner
 * @since Sep 7, 2018
 *
 */
public abstract class CreateTaxonDescriptionHandler {

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART)MPart activePart, UISynchronize sync) {
        CharacterMatrixPart matrixPart = (CharacterMatrixPart) activePart.getObject();
        if(StoreUtil.promptCheckIsDirty(matrixPart)){
            return;
        }
        DescriptiveDataSet descriptiveDataSet = matrixPart.getDescriptiveDataSet();
        TaxonNodeDto node = (TaxonNodeDto) matrixPart.getSelection().getFirstElement();
        TaxonRowWrapperDTO taxonRowWrapperDTO = CdmStore.getService(IDescriptiveDataSetService.class)
                .createTaxonDescription(descriptiveDataSet.getUuid(), node.getUuid(), getMarkerType(), getMarkerFlag());
        matrixPart.getMatrix().getCdmEntitiySession().load(taxonRowWrapperDTO.getDescription(), true);
        matrixPart.getMatrix().getDescriptions().add(taxonRowWrapperDTO);
    }

    protected abstract boolean getMarkerFlag();


    protected abstract MarkerType getMarkerType();

    @CanExecute
    public boolean canExecute(@Named(IServiceConstants.ACTIVE_PART)MPart activePart,
            MHandledMenuItem menuItem){
        boolean canExecute = false;
        IStructuredSelection selection = ((CharacterMatrixPart)activePart.getObject()).getSelection();
        canExecute = selection.size()==1
                && selection.getFirstElement() instanceof TaxonNodeDto;
        menuItem.setVisible(canExecute);
        return canExecute;
    }

}
