/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.quantitative;

import java.util.List;
import java.util.Map;

import org.eclipse.jface.window.Window;
import org.eclipse.nebula.widgets.nattable.config.IConfigRegistry;
import org.eclipse.nebula.widgets.nattable.edit.editor.ICellEditor;
import org.eclipse.nebula.widgets.nattable.edit.gui.CellEditDialog;
import org.eclipse.nebula.widgets.nattable.layer.cell.ILayerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.description.QuantitativeData;
import eu.etaxonomy.cdm.model.description.StatisticalMeasure;
import eu.etaxonomy.cdm.model.description.StatisticalMeasurementValue;
import eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.CharacterMatrix;

/**
 * @author pplitzner
 * @since Jul 19, 2018
 *
 */
public class QuantitativeDataDialog extends CellEditDialog {

    private Character initialInput;

    private CharacterMatrix matrix;

    public QuantitativeDataDialog(Character initialInput, CharacterMatrix matrix, Shell parentShell, Object originalCanonicalValue, ILayerCell cell,
            ICellEditor cellEditor, IConfigRegistry configRegistry) {
        super(parentShell, originalCanonicalValue, cell, cellEditor, configRegistry);
        this.initialInput = initialInput;
        this.matrix = matrix;
    }

    private QuantitativeData editorValue;

    private QuantitativeDataDialogComposite composite;

    @Override
    protected Control createDialogArea(Composite parent) {
        composite = new QuantitativeDataDialogComposite(initialInput, editorValue, parent, SWT.NONE);
        return composite;
    }

    void setEditorValue(QuantitativeData editorValue) {
        this.editorValue = editorValue;
    }

    @Override
    public boolean close() {
        if(getReturnCode()==Window.OK){
            updateQuantitativeData(editorValue);
            matrix.setDirty();
        }
        return super.close();
    }

    void updateQuantitativeData(QuantitativeData quantitativeData){
        //clear values
        quantitativeData.getStatisticalValues().clear();
        //add back all values from text fields
        Map<StatisticalMeasure, List<Text>> textFields = composite.getTextFields();
        textFields.forEach((measure, texts)->{
            texts.forEach(text->{
                if(text.isEnabled()){
                    String string = text.getText();
                    try {
                        float exactValue = Float.parseFloat(string);
                        quantitativeData.addStatisticalValue(StatisticalMeasurementValue.NewInstance(measure, exactValue));
                    } catch (NumberFormatException e) {
                    }
                }
            });
        });
    }

}
