// $Id$
/**
 * Copyright (C) 2018 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.quantitative;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.description.QuantitativeData;
import eu.etaxonomy.cdm.model.description.StatisticalMeasure;
import eu.etaxonomy.taxeditor.model.ImageResources;

/**
 * @author pplitzner
 * @date 16.07.2018
 *
 */
public class QuantitativeDataDialogComposite extends Composite {

    private Map<StatisticalMeasure, List<Text>> textFieldMap = new HashMap<>();

    public QuantitativeDataDialogComposite(Character initialInput, QuantitativeData editorValue, Composite parent, int style) {
        super(parent, style);
        setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

        setLayout(new GridLayout(1, false));

        ScrolledComposite scrolledComposite_1 = new ScrolledComposite(this, SWT.BORDER | SWT.V_SCROLL);
        scrolledComposite_1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        scrolledComposite_1.setExpandHorizontal(true);
        scrolledComposite_1.setExpandVertical(true);

        Composite valuesComposite = new Composite(scrolledComposite_1, SWT.NONE);
        GridLayout gl_valuesComposite = new GridLayout();
        gl_valuesComposite.marginWidth = 0;
        gl_valuesComposite.marginHeight = 0;
        valuesComposite.setLayout(gl_valuesComposite);

        //add empty text field for exact value
        Text emptyTextField = addText(valuesComposite, StatisticalMeasure.EXACT_VALUE(), initialInput==null?null:initialInput.toString());
        emptyTextField.setFocus();
        emptyTextField.setSelection(emptyTextField.getText().length());

        //add existing exact values
        editorValue.getStatisticalValues().stream()
        .filter(measure->measure.getType().equals(StatisticalMeasure.EXACT_VALUE()))
        .forEach(exact->addText(valuesComposite, exact.getType(), Float.toString(exact.getValue())));

        //add aggregation values
        editorValue.getFeature().getRecommendedStatisticalMeasures()
        .stream()
        .filter(sm->!sm.equals(StatisticalMeasure.EXACT_VALUE()))
        .forEach(measure->{
            Float specificStatisticalValue = editorValue.getSpecificStatisticalValue(measure);
            addText(valuesComposite, measure, specificStatisticalValue!=null?Float.toString(specificStatisticalValue):null);
        });

        scrolledComposite_1.setContent(valuesComposite);
        scrolledComposite_1.setMinSize(valuesComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));

        if(initialInput!=null){
            enableAggregationFields(false);
        }
    }

    private Text addText(Composite valuesComposite, StatisticalMeasure type, String value){
        Composite composite_2 = new Composite(valuesComposite, SWT.NONE);
        composite_2.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
        GridLayout gl_composite_2 = new GridLayout(4, false);
        gl_composite_2.marginHeight = 0;
        gl_composite_2.marginWidth = 0;
        gl_composite_2.verticalSpacing = 0;
        gl_composite_2.horizontalSpacing = 2;
        composite_2.setLayout(gl_composite_2);

        Label lblNewLabel = new Label(composite_2, SWT.NONE);
        lblNewLabel.setText(type.getLabel().substring(0, 3));

        Text text = new Text(composite_2, SWT.BORDER);
        text.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        if(value!=null){
            text.setText(value);
        }

        List<Text> list = textFieldMap.get(type);
        if(list==null){
            list = new ArrayList<>();
        }
        textFieldMap.put(type, list);
        list.add(text);

        Button btnNewButton_2 = new Button(composite_2, SWT.NONE);
        btnNewButton_2.setImage(ImageResources.getImage(ImageResources.TRASH_ICON));
        new Label(composite_2, SWT.NONE);
        btnNewButton_2.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                text.setText("");
            }
        });
        text.addModifyListener(e->{
            boolean hasExactValues = textFieldMap.entrySet().stream()
                    .filter(entry->entry.getKey().equals(StatisticalMeasure.EXACT_VALUE()))
            .anyMatch(exactEntry->exactEntry.getValue().stream().anyMatch(exactText->CdmUtils.isNotBlank(exactText.getText()))
            );
            if(hasExactValues){
                enableAggregationFields(false);
            }
            else{
                enableAggregationFields(true);
            }
        });
        return text;
    }

    private void enableAggregationFields(boolean enabled){
        textFieldMap.entrySet().stream()
        .filter(entry->!entry.getKey().equals(StatisticalMeasure.EXACT_VALUE()))
        .forEach(aggEntry->aggEntry.getValue().forEach(aggText->aggText.setEnabled(enabled)));
    }

    public Map<StatisticalMeasure, List<Text>> getTextFields() {
        return textFieldMap;
    }

}
