/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.quantitative;

import java.util.Map;

import org.eclipse.jface.window.Window;
import org.eclipse.nebula.widgets.nattable.config.IConfigRegistry;
import org.eclipse.nebula.widgets.nattable.edit.ICellEditHandler;
import org.eclipse.nebula.widgets.nattable.edit.gui.AbstractDialogCellEditor;
import org.eclipse.nebula.widgets.nattable.layer.cell.ILayerCell;
import org.eclipse.nebula.widgets.nattable.widget.EditModeEnum;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.api.service.dto.RowWrapperDTO;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.QuantitativeData;
import eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.CharacterMatrix;

/**
 * @author pplitzner
 * @since Jul 19, 2018
 *
 */
public class QuantitativeDataDialogEditor extends AbstractDialogCellEditor {

    private QuantitativeData editorValue;
    private boolean closed;
    private CharacterMatrix matrix;
    private Character initialInput;

    public QuantitativeDataDialogEditor(CharacterMatrix matrix) {
        super();
        this.matrix = matrix;
    }

    @Override
    public int open() {
        Object rowObject = matrix.getBodyDataProvider().getRowObject(getRowIndex());
        if(rowObject instanceof RowWrapperDTO){
            this.closed = false;
            return getDialogInstance().open();
        }
        return Window.CANCEL;
    }

    @Override
    public QuantitativeDataDialog createDialogInstance() {
        return new QuantitativeDataDialog(this.initialInput, matrix, this.parent.getShell(), getCanonicalValue(), this.layerCell, this, this.configRegistry);
    }

    @Override
    public QuantitativeDataDialog getDialogInstance() {
        return (QuantitativeDataDialog) this.dialog;
    }

    @Override
    public Object getEditorValue() {
        return this.editorValue;
    }

    @Override
    public void setEditorValue(Object value) {
        ((QuantitativeDataDialog)this.dialog).setEditorValue((QuantitativeData) value);
        this.editorValue = (QuantitativeData) value;
    }

    @Override
    public void close() {
        this.closed = true;
        this.dialog = null;
        this.editorValue = null;
    }

    @Override
    public boolean isClosed() {
        return closed;
    }

    @Override
    public Control activateCell(Composite parent, Object originalCanonicalValue, EditModeEnum editMode,
            ICellEditHandler editHandler, ILayerCell cell, IConfigRegistry configRegistry) {
        this.initialInput = null;
        Object rowObject = matrix.getBodyDataProvider().getRowObject(cell.getRowIndex());
        Feature feature = matrix.getIndexToCharacterMap().get(cell.getColumnIndex());
        QuantitativeData quantitativeData = null;
        if(rowObject instanceof RowWrapperDTO){
            if(cell.getDataValue() instanceof QuantitativeData){
                quantitativeData = (QuantitativeData) cell.getDataValue();
            }
            //create new Quantitative Data
            if (cell.getDataValue() == null) {
                quantitativeData = ((RowWrapperDTO) rowObject).addQuantitativeData(feature);
            }
            /**
             * when opening the the dialog by a key stroke, store the key pressed
             * and pass it to the dialog. This is common behavior but by overriding
             * this method we always pass the QuantitativeData object to the dialog
             * so we need to buffer the key stroke
             */
            if(originalCanonicalValue instanceof Character){
                this.initialInput = (Character) originalCanonicalValue;
            }
        }
        return super.activateCell(parent, quantitativeData, editMode, editHandler, cell, configRegistry);
    }

    @Override
    public void setDialogSettings(Map<String, Object> editDialogSettings) {
        //pass through dialog setting to wrapped dialog
        ((QuantitativeDataDialog)this.dialog).setDialogSettings(editDialogSettings);
    }

}
