/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.supplementalInfo;

import org.eclipse.nebula.widgets.nattable.data.convert.DisplayConverter;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.cdm.model.occurrence.GatheringEvent;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;

/**
 * Converts the supplemental information (taxon and specimen data)
 * belonging to a specimen description to a String.
 *
 * @author pplitzner
 * @since Dec 14, 2017
 *
 */
public class SupplementalInfoDisplayConverter extends DisplayConverter{

    public SupplementalInfoDisplayConverter() {
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Object canonicalToDisplayValue(Object canonicalValue) {
        String displayValue = "";
        if(canonicalValue instanceof TaxonNodeDto){
            TaxonNodeDto node = (TaxonNodeDto)canonicalValue;
            displayValue = node.getTaxonTitleCache();
        }
        else if(HibernateProxyHelper.isInstanceOf(canonicalValue, FieldUnit.class)){
            FieldUnit fieldUnit = HibernateProxyHelper.deproxy(canonicalValue, FieldUnit.class);
            GatheringEvent gatheringEvent = fieldUnit.getGatheringEvent();
            if(gatheringEvent!=null){
                AgentBase collector = gatheringEvent.getCollector();
                if(collector!=null && collector.getTitleCache()!=null){
                    displayValue += collector.getTitleCache()+" ";
                }
                String fieldNumber = fieldUnit.getFieldNumber();
                if(fieldNumber!=null){
                    displayValue += fieldNumber;
                }
            }
        }
        else if(HibernateProxyHelper.isInstanceOf(canonicalValue, NamedArea.class)){
            displayValue = HibernateProxyHelper.deproxy(canonicalValue, NamedArea.class).getLabel();
        }

        if(displayValue.isEmpty() && canonicalValue!=null){
            return canonicalValue.toString();
        }
        return displayValue;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Object displayToCanonicalValue(Object displayValue) {
        return null;
    }

}
