/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.group.authority.e4;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.ui.MDirtyable;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.Group;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.editor.group.authority.CdmAuthorityEditorInput;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.IDirtyMarkable;
import eu.etaxonomy.taxeditor.ui.group.grantedauthority.CdmAuthorityCompositeViewer;

/**
 * Editor responsible for editing authorities related to a particular {@link Group} entity.
 *
 * @author cmathew
 * @created Mar 28, 2013
 *
 */
public class CdmAuthorityEditorE4 implements IConversationEnabled, IDirtyMarkable {

	public static final String ID = "eu.etaxonomy.taxeditor.editor.group.authority"; //$NON-NLS-1$

	@Inject
	private MDirtyable dirty;
	private ConversationHolder conversation;
	private CdmAuthorityCompositeViewer viewer;

	private CdmAuthorityEditorInput input;

	@Inject
	private MPart thisPart;

    private Composite container;

	@Inject
	public CdmAuthorityEditorE4() {

	}

	/**
	 * Create contents of the editor part.
	 * @param parent
	 */
	@PostConstruct
	public void createPartControl(Composite parent) {
		container = new Composite(parent, SWT.NONE);
		container.setLayout(new FillLayout(SWT.HORIZONTAL));

	}

	@Focus
	public void setFocus() {
	    if(input!=null){
	        input.bind();
	    }
	}

	@Persist
	public void doSave(IProgressMonitor monitor) {
		try {
			monitor.beginTask(Messages.CdmAuthorityEditor_SAVING_AUTHORITY_EDITOR, 1);
			getConversationHolder().commit(true);
			input.merge();
			dirty.setDirty(false);
			monitor.worked(1);
		} finally {
			monitor.done();
			viewer.save();
		}
	}

	public void init(CdmAuthorityEditorInput input) {
	    this.input = input;

        conversation = input.getConversationHolder();

        this.viewer = new CdmAuthorityCompositeViewer(container, this,input.getGroup());

        thisPart.setLabel(input.getName());
        container.layout();
	}

	public boolean isDirty() {
		return dirty.isDirty();
	}

    public CdmAuthorityEditorInput getInput() {
        return input;
    }

	@Override
	public void update(CdmDataChangeMap changeEvents) {
	}

	@Override
	public void changed(Object element) {
		//FIXME : should be optimised to split changes into adding / removing authorities
		//        vs updating authorites
		dirty.setDirty(viewer.isDirty());
		// if the change has happened on the group then refresh the table
		if(element instanceof Group) {
			//FIXME: activating (setting focus) on the editor happens on every change
			//       This should be changed to only when the drop is successful
		    //FIXME E4 migrate/delete
//			getSite().getPage().activate(this);
		}

	}

	@Override
	public void forceDirty() {
	    changed(null);
	}

	@Override
	public ConversationHolder getConversationHolder() {
		return conversation;
	}

	@PreDestroy
	public void dispose() {
	    if(conversation!=null){
	        conversation.unregisterForDataStoreChanges(this);
	        conversation.close();
	        conversation = null;
	    }
		if(input!=null){
		    input.getCdmEntitySession().dispose();
		}
		dirty.setDirty(false);
	}
}
