/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.group.authority.e4.handler;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.viewers.IStructuredSelection;

import eu.etaxonomy.cdm.model.common.Group;
import eu.etaxonomy.taxeditor.bulkeditor.e4.BulkEditorE4;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.group.authority.CdmAuthorityEditorInput;
import eu.etaxonomy.taxeditor.editor.group.authority.e4.CdmAuthorityEditorE4;

/**
 * Handler which opens an instance of the {@link CdmAuthorityEditorE4} for a2
 * particluar group.
 *
 * @author cmathew
 * @created Mar 28, 2013
 *
 */

public class EditCdmAuthoritiesHandlerE4 {

    @Execute
    public void execute(EPartService partService, EModelService modelService, MApplication application,
            @Named(IServiceConstants.ACTIVE_PART)MPart activePart) {
        BulkEditorE4 editor = (BulkEditorE4) activePart.getObject();
        Group group = (Group) editor.getSelection().getFirstElement();
        try {
            EditorUtil.openRightsEditor(CdmAuthorityEditorInput.NewInstance(group.getUuid()), modelService, partService, application);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @CanExecute
    public boolean execute(MHandledMenuItem menuItem,
            @Named(IServiceConstants.ACTIVE_PART)MPart activePart){
        boolean canExecute = false;
        BulkEditorE4 editor = (BulkEditorE4) activePart.getObject();
        IStructuredSelection selection = editor.getSelection();
        canExecute =
                selection.size()==1
                && selection.getFirstElement() instanceof Group;
        menuItem.setVisible(canExecute);
        return canExecute;

    }
}
