/**
 *
 */
package eu.etaxonomy.taxeditor.editor.key.e4;

import javax.inject.Inject;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.ui.MDirtyable;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.ESelectionService;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.zest.core.viewers.AbstractZoomableViewer;
import org.eclipse.zest.core.viewers.GraphViewer;
import org.eclipse.zest.core.viewers.IZoomableWorkbenchPart;
import org.eclipse.zest.core.viewers.ZoomContributionViewItem;
import org.eclipse.zest.layouts.LayoutAlgorithm;
import org.eclipse.zest.layouts.LayoutStyles;
import org.eclipse.zest.layouts.algorithms.TreeLayoutAlgorithm;

import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.description.IIdentificationKey;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.editor.key.polytomous.PolytomousKeyEditorInput;
import eu.etaxonomy.taxeditor.model.IDirtyMarkable;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.workbench.part.IE4SavablePart;

/**
 *
 * @author pplitzner
 * @since Sep 28, 2017
 *
 * @param <T>
 */
public abstract class AbstractGraphKeyEditorE4<T extends IIdentificationKey>
implements IConversationEnabled, IE4SavablePart,
		IZoomableWorkbenchPart, IPostOperationEnabled,
		IDirtyMarkable {

	protected CdmFormFactory formFactory;
	protected Composite container;
	protected GraphViewer viewer;

	private LayoutAlgorithm layoutAlgoritm;
	private ZoomContributionViewItem zoomContributionViewItem;

	private IToolBarManager toolBarManager;

    private PolytomousKeyEditorInput input;

    @Inject
    protected ESelectionService selService;

    @Inject
    private MDirtyable dirty;

    protected ISelectionChangedListener selectionChangedListener;

    @Inject
    private MPart thisPart;

	@Persist
	public void save(IProgressMonitor monitor) {
		getConversationHolder().commit(true);
		setDirty(false);
		viewer.refresh();
	}

	public void init(PolytomousKeyEditorInput input) {
		this.input = input;


        viewer.setInput(getKey());
        if(getKey()!=null){
            thisPart.setLabel(getKey().toString());
        }
	}

	public boolean isDirty() {
		return dirty.isDirty();
	}

    public PolytomousKeyEditorInput getEditorInput() {
        return input;
    }

	//FIXME E4 migrate
//	private void createToolbar() {
//		getToolBarManager().add(getZoomContributionViewItem());
//	}

	protected abstract IBaseLabelProvider getLabelProvider();

	protected abstract IContentProvider getContentProvider();

	/**
	 * Return the key that this editor
	 *
	 * @return
	 */
	public abstract T getKey();

	@Focus
	public void setFocus() {
	    if(viewer!=null && viewer.getControl()!=null && !viewer.getControl().isDisposed()) {
            viewer.getControl().setFocus();
        }
	}

	public void refresh() {
		viewer.refresh();
	}

	@Override
	public void update(CdmDataChangeMap changeEvents) {

	}

	@Override
	public AbstractZoomableViewer getZoomableViewer() {
		return viewer;
	}

	protected LayoutAlgorithm getLayoutAlgoritm() {
		if (layoutAlgoritm == null) {
			// layoutAlgoritm = new CompositeLayoutAlgorithm(
			// LayoutStyles.NO_LAYOUT_NODE_RESIZING,
			// new LayoutAlgorithm[] {
			// new TreeLayoutAlgorithm(
			// LayoutStyles.NO_LAYOUT_NODE_RESIZING),
			// new HorizontalShift(
			// LayoutStyles.NO_LAYOUT_NODE_RESIZING) });

			layoutAlgoritm = new TreeLayoutAlgorithm(
					LayoutStyles.NO_LAYOUT_NODE_RESIZING);
			layoutAlgoritm.setEntityAspectRatio(0.5);
		}
		return layoutAlgoritm;
	}

	private ZoomContributionViewItem getZoomContributionViewItem() {
		if (zoomContributionViewItem == null) {
			zoomContributionViewItem = new ZoomContributionViewItem(this);
		}
		return new ZoomContributionViewItem(this);
	}

	@Override
	public boolean postOperation(Object objectAffectedByOperation) {
		setDirty(true);
		refresh();

		if (objectAffectedByOperation instanceof PolytomousKeyNode) {
			viewer.setSelection(new StructuredSelection(
					objectAffectedByOperation), true);
		}

		return true;
	}

	//FIXME E4 migrate
//	private IToolBarManager getToolBarManager() {
//		if (toolBarManager == null) {
//			toolBarManager = getEditorSite().getActionBars()
//					.getToolBarManager();
//		}
//		return toolBarManager;
//	}

	public void applyLayout() {
		viewer.applyLayout();
	}

	@Override
	public boolean onComplete() {
		return true;
	}

	public void setDirty(boolean dirty) {
		this.dirty.setDirty(dirty);
	}

	@Override
	public void changed(Object element) {
		setDirty(true);
		viewer.update(element, null);
	}

	@Override
	public void forceDirty() {
	    changed(null);
	}

}
