/**
 *
 */
package eu.etaxonomy.taxeditor.editor.key.polytomous.e4;

import javax.annotation.PostConstruct;
import javax.inject.Inject;
import javax.inject.Named;

import org.eclipse.e4.ui.services.EMenuService;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.zest.core.viewers.GraphViewer;
import org.eclipse.zest.core.widgets.ZestStyles;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.taxeditor.editor.key.e4.AbstractGraphKeyEditorE4;
import eu.etaxonomy.taxeditor.editor.key.polytomous.IPolytomousKeyEditorPage;
import eu.etaxonomy.taxeditor.editor.key.polytomous.PolytomousKeyGraphContentProvider;
import eu.etaxonomy.taxeditor.editor.key.polytomous.PolytomousKeyLabelProvider;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;

/**
 * @author n.hoffmann
 *
 */
public class PolytomousKeyGraphEditorE4 extends
		AbstractGraphKeyEditorE4<PolytomousKey> implements
		IPolytomousKeyEditorPage, IPartContentHasDetails {

	public static final String ID = "eu.etaxonomy.taxeditor.editor.key.polytomous.graph"; //$NON-NLS-1$

	@Inject
	public PolytomousKeyGraphEditorE4() {
	}

	@PostConstruct
    public void createPartControl(Composite parent, EMenuService menuService,
            @Named(IServiceConstants.ACTIVE_SHELL)Shell shell) {
	    if(!CdmStore.isActive()){
	        return;
	    }

        formFactory = new CdmFormFactory(shell.getDisplay());
        container = formFactory.createComposite(parent);

        container.setLayout(new FillLayout());
        viewer = new GraphViewer(container, SWT.NONE);

        viewer.setContentProvider(getContentProvider());
        viewer.setLabelProvider(getLabelProvider());

        viewer.setConnectionStyle(ZestStyles.CONNECTIONS_DIRECTED);

        viewer.setLayoutAlgorithm(getLayoutAlgoritm(), false);

        //propagate selection
        selectionChangedListener = (event -> selService.setSelection(event.getSelection()));
        viewer.addSelectionChangedListener(selectionChangedListener);

        //create context menu
        menuService.registerContextMenu(viewer.getControl(), "eu.etaxonomy.taxeditor.editor.popupmenu.polytomouskeygrapheditor");

        //FIXME E4 migrate
//      createToolbar();

        container.addControlListener(new ControlListener() {

            @Override
            public void controlResized(ControlEvent e) {
                // applyLayout();
            }

            @Override
            public void controlMoved(ControlEvent e) {

            }
        });
    }

	@Override
	public ConversationHolder getConversationHolder() {
		return getEditorInput()
				.getConversationHolder();
	}

	@Override
	public PolytomousKey getKey() {
		return getEditorInput().getKey();
	}

	@Override
	protected IBaseLabelProvider getLabelProvider() {
		return new PolytomousKeyLabelProvider();
	}

	@Override
	protected IContentProvider getContentProvider() {
		return new PolytomousKeyGraphContentProvider();
	}
}
