/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.key.polytomous.e4;

import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.MDirtyable;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.services.EMenuService;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.ESelectionService;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.key.e4.KeyEditorDataChangeBehaviourE4;
import eu.etaxonomy.taxeditor.editor.key.polytomous.IPolytomousKeyEditorPage;
import eu.etaxonomy.taxeditor.editor.key.polytomous.PolytomousKeyEditorInput;
import eu.etaxonomy.taxeditor.editor.key.polytomous.PolytomousKeyListContentProvider;
import eu.etaxonomy.taxeditor.editor.key.polytomous.PolytomousKeyListLabelProvider;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.DataChangeBridge;
import eu.etaxonomy.taxeditor.model.IDirtyMarkable;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;
import eu.etaxonomy.taxeditor.workbench.part.IE4SavablePart;

/**
 *
 * @author pplitzner
 * @since Sep 28, 2017
 *
 */
public class PolytomousKeyListEditorE4 implements
		IConversationEnabled, IDirtyMarkable, IPartContentHasDetails, IPartContentHasSupplementalData,
		IPolytomousKeyEditorPage, IE4SavablePart{

	private class LinkListener extends MouseAdapter {

		@Override
		public void mouseUp(MouseEvent event) {

	    		if(event.button == 1 && event.count == 2) {
		        Table table = (Table) event.widget;
		        // Determine where the mouse was clicked
		        Point point = new Point(event.x, event.y);

		        int selectedColumn = getSelectedColumn(table, point);

		        if (table == null || point == null ){
		            return;
		        }

		       TableItem item = getTableItem(
                        table, point);
		       PolytomousKeyNode node ;
		       if (item != null){
		         node =(PolytomousKeyNode) item.getData();
		       } else{
		           return;
		       }
		        if (selectedColumn == 4) {
		            PolytomousKeyNode linkData = getItemLinkData(node);
		            if (linkData != null) {
		                viewer.setSelection(new StructuredSelection(linkData), true);
		            }
		        }
		        if (selectedColumn == 5) {
		            Taxon taxon = getItemTaxon(node);
		            if (taxon != null) {
		                EditorUtil.openTaxonBaseE4((taxon).getUuid(), modelService, partService, application);
		            }
		        }
		    }
		}

		private int getSelectedColumn(Table table, Point point) {
			TableItem item = getTableItem(table, point);
			if (item != null) {
				for (int i = 0, n = table.getColumnCount(); i < n; i++) {
					Rectangle rect = item.getBounds(i);
					if (rect.contains(point)) {
						// This is the selected column
						return i;
					}
				}
			}
			return -1;
		}

		private TableItem getTableItem(Table table, Point point) {
			return table.getItem(point);
		}

		private PolytomousKeyNode getItemLinkData(PolytomousKeyNode node) {
			return node.getChildren().isEmpty() ? null : node
					.getChildAt(0);
		}

        private Taxon getItemTaxon(PolytomousKeyNode node) {
            return node.getTaxon();
        }
	}

	public static final String ID = "eu.etaxonomy.taxeditor.editor.key.polytomous.list"; //$NON-NLS-1$

	private TableViewer viewer;

    private KeyEditorDataChangeBehaviourE4 dataChangeBehavior;

    private PolytomousKeyEditorInput input;

    @Inject
    private ESelectionService selService;

    @Inject
    private MDirtyable dirty;

    @Inject
    private MApplication application;

    @Inject
    private EModelService modelService;

    @Inject
    private EPartService partService;

    private ISelectionChangedListener selectionChangedListener;

    @Inject
    private MPart thisPart;

    @Inject
	public PolytomousKeyListEditorE4() {
	}

	@Override
	public void update(CdmDataChangeMap map) {
	    if(dataChangeBehavior == null){
            dataChangeBehavior = new KeyEditorDataChangeBehaviourE4(this);
        }
        DataChangeBridge.handleDataChange(map, dataChangeBehavior);
	}

	@Override
	public ConversationHolder getConversationHolder() {
		return input.getConversationHolder();
	}

	@Override
    @Persist
	public void save(IProgressMonitor monitor) {
        try {
            monitor.beginTask(Messages.KeyEditor_SAVING, 1);
            getConversationHolder().bind();
            getConversationHolder().commit(true);
            input.merge();
            dirty.setDirty(false);
            monitor.worked(1);
        } finally {
            monitor.done();
        }
    }

	public void init(PolytomousKeyEditorInput input) {
		this.input = input;

        PolytomousKey key = input.getKey();
        key = HibernateProxyHelper.deproxy(key, PolytomousKey.class);
        key.setRoot(HibernateProxyHelper.deproxy(key.getRoot(), PolytomousKeyNode.class));
        thisPart.setLabel(key.getTitleCache());

        viewer.setInput(input);
	}

    public PolytomousKeyEditorInput getEditorInput() {
        return input;
    }

	public boolean isDirty() {
		return dirty.isDirty();
	}

	@PostConstruct
	public void createPartControl(Composite parent, EMenuService menuService) {

		FillLayout fillLayout = new FillLayout();
		fillLayout.marginWidth = 0;
		fillLayout.marginHeight = 0;
		fillLayout.type = SWT.VERTICAL;
		parent.setLayout(fillLayout);

		viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.FULL_SELECTION);

		createColumns(viewer);
		viewer.getControl().addMouseListener(new LinkListener());
		viewer.setContentProvider(new PolytomousKeyListContentProvider());
		viewer.setLabelProvider(new PolytomousKeyListLabelProvider());

		//propagate selection
        selectionChangedListener = (event -> selService.setSelection(event.getSelection()));
        viewer.addSelectionChangedListener(selectionChangedListener);

        //create context menu
        menuService.registerContextMenu(viewer.getControl(), "eu.etaxonomy.taxeditor.editor.popupmenu.polytomouskeylisteditor");
	}


    @PreDestroy
    public void dispose() {
        if(input!=null){
            input.dispose();
            input.getConversationHolder().close();
        }
    }

	public int getTableItemCount() {
	    if (viewer != null && viewer.getTable() != null) {
	        return viewer.getTable().getItemCount();
	    }
	    return 0;
	}

	public PolytomousKey getViewerInputKey() {
	    return ((PolytomousKeyEditorInput) viewer.getInput()).getKey();
	}

	// This will create the columns for the table
	private void createColumns(TableViewer viewer) {
		Table table = viewer.getTable();
		String[] titles = { Messages.PolytomousKeyListEditor_NODE, Messages.PolytomousKeyListEditor_QUESTION, Messages.PolytomousKeyListEditor_EDGE,  Messages.PolytomousKeyListEditor_STATEMENT, Messages.PolytomousKeyListEditor_LINK, Messages.PolytomousKeyListEditor_TAXON };
		int[] bounds = { 50, 200, 50, 200, 100, 200 };

		for (int i = 0; i < titles.length; i++) {
			TableViewerColumn column = new TableViewerColumn(viewer, SWT.NONE);
			column.getColumn().setText(titles[i]);
			column.getColumn().setWidth(bounds[i]);
			column.getColumn().setResizable(true);
			column.getColumn().setMoveable(true);
		}
		table.setHeaderVisible(true);
		table.setLinesVisible(false);

	}

	@Focus
	public void setFocus() {
	    if(viewer!=null && viewer.getControl()!=null && !viewer.getControl().isDisposed()) {
	        viewer.getControl().setFocus();
	    }
	    if(input!=null){
	        input.bind();
	    }
	}

	@Override
	public void changed(Object element) {
        if(element != null) {
            viewer.update(element, null);

        }

        if (element instanceof PolytomousKeyNode) {
            List<PolytomousKeyNode> children = ((PolytomousKeyNode) element)
                    .getParent().getChildren();
            for (PolytomousKeyNode child : children) {
                viewer.update(child, null);
            }
        }
        dirty.setDirty(true);
        viewer.refresh();

	}

    @Override
    public void forceDirty() {
        changed(null);
    }

	@Override
	public boolean postOperation(Object objectAffectedByOperation) {
		viewer.refresh();

		if (objectAffectedByOperation != null) {
			viewer.setSelection(new StructuredSelection(
					objectAffectedByOperation), true);
		}
		return true;
	}

	@Override
	public boolean onComplete() {
		return true;
	}

    public void setPartName() {
        PolytomousKey key = input.getKey();
        thisPart.setLabel(key.getTitleCache());
    }

}
