/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.key.polytomous.e4.handler;

import javax.inject.Named;

import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.key.polytomous.PolytomousKeyEditorLabels;
import eu.etaxonomy.taxeditor.editor.key.polytomous.e4.PolytomousKeyListEditorE4;
import eu.etaxonomy.taxeditor.editor.key.polytomous.operation.DeleteNodeOperation;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.operation.e4.RemotingCdmHandlerE4;

/**
 * @author n.hoffmann
 * @created Dec 6, 2010
 * @version 1.0
 */
public class DeleteNodeHandlerE4 extends RemotingCdmHandlerE4 {


    private static final String DO_YOU_REALLY_WANT_TO_DELETE_THE_NODE_THIS_OPERATION_IS_NOT_REVERSABLE = Messages.DeleteNodeHandler_REALLY_DELETE;
    private static final String CONFIRM_DELETION_OF_CHILDREN = Messages.DeleteNodeHandler_CONFIRM_DELETE;
    private static final String NO = Messages.DeleteNodeHandler_NO;
    private static final String CANCEL = Messages.DeleteNodeHandler_CANCEL;
    private static final String YES = Messages.DeleteNodeHandler_YES;
    PolytomousKeyNode nodeToBeDeleted;
    boolean deleteChildren;

    public DeleteNodeHandlerE4(String label) {
        super(label);
    }

    public DeleteNodeHandlerE4() {
        super(PolytomousKeyEditorLabels.DELETE_NODE_POLYTOMOUS_KEY_NODE_LABEL);
    }

    @Override
    public IStatus allowOperations(@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            @Named(IServiceConstants.ACTIVE_SHELL)Shell shell,
            @Named(IServiceConstants.ACTIVE_PART)MPart activePart,
            MHandledMenuItem menuItem) {

        if (selection.getFirstElement() instanceof PolytomousKeyNode) {
            label = menuItem.getLocalizedLabel();
            PolytomousKeyNode node = (PolytomousKeyNode)selection.getFirstElement();
            nodeToBeDeleted = node;
            MessageDialog dialog;
            if (node.getChildren().size()>0){
                String[] buttonLables = {YES, NO,CANCEL};
                dialog = new MessageDialog(null, CONFIRM_DELETION_OF_CHILDREN, null, DO_YOU_REALLY_WANT_TO_DELETE_THE_NODE_THIS_OPERATION_IS_NOT_REVERSABLE+Messages.DeleteNodeHandler_NODE_HAS_CHILDREN, MessageDialog.QUESTION_WITH_CANCEL, buttonLables, 0);
                int returnCode = dialog.open();

                if (returnCode == 0){
                    deleteChildren = false;
                } else if (returnCode == 1){
                    deleteChildren = true;
                } else{
                    return new Status(IStatus.CANCEL, "unknown", //$NON-NLS-1$
                            null);
                }


            }else{
                String[] buttonLables = {YES, CANCEL};
                dialog = new MessageDialog(null, CONFIRM_DELETION_OF_CHILDREN, null, DO_YOU_REALLY_WANT_TO_DELETE_THE_NODE_THIS_OPERATION_IS_NOT_REVERSABLE, MessageDialog.QUESTION_WITH_CANCEL, buttonLables, 1);
                int returnCode = dialog.open();

                if (returnCode == 0){
                    deleteChildren = false;
                } else if (returnCode == 1){
                   return new Status(IStatus.CANCEL, "unknown", //$NON-NLS-1$
                            null);
                }
            }
        } else {
                MessageDialog.openInformation(
                        shell,
                        PolytomousKeyEditorLabels.NO_KEY_NODE_SELECTED,
                        PolytomousKeyEditorLabels.NO_KEY_NODE_FOR_INSERT_NODE_SELECTED_MESSAGE
                        );
        }

        return Status.OK_STATUS;
    }

    @Override
    public AbstractOperation prepareOperation(@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            @Named(IServiceConstants.ACTIVE_SHELL)Shell shell,
            @Named(IServiceConstants.ACTIVE_PART)MPart activePart,
            MHandledMenuItem menuItem) {
        IUndoContext undoContext = EditorUtil.getUndoContext();
        PolytomousKeyListEditorE4 editor = (PolytomousKeyListEditorE4) activePart.getObject();
        if (editor.isDirty()){
            boolean proceed = MessageDialog.openQuestion(null,
                    Messages.DeleteNodeHandler_SAVE_CHANGES_TITLE, Messages.DeleteNodeHandler_SAVE_CHANGES_MESSAGE);
            if (!proceed) {
                return null;
            }else{
                editor.save(AbstractUtility.getMonitor());
            }
        }

        label = menuItem.getLocalizedLabel();
        DeleteNodeOperation operation ;
        if (deleteChildren){
            operation = new DeleteNodeOperation(label, undoContext, nodeToBeDeleted, editor, true);
        }else{
            operation =  new DeleteNodeOperation(label, undoContext, nodeToBeDeleted, editor, false);
        }
        return operation;
    }

    @Override
    public void onComplete() {
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected Object getTrigger() {
        return this;
    }

    @CanExecute
    public boolean canExecute(@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            MHandledMenuItem menuItem){
        boolean canExecute = false;
        canExecute = selection!=null && selection.size()==1;
        menuItem.setVisible(canExecute);
        return canExecute;
    }

}