/**
 * Copyright (C) 2016 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.key.polytomous.e4.handler;

import javax.inject.Named;

import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.taxeditor.editor.key.polytomous.PolytomousKeyEditorLabels;
import eu.etaxonomy.taxeditor.editor.key.polytomous.e4.PolytomousKeyListEditorE4;
import eu.etaxonomy.taxeditor.editor.key.polytomous.operation.InsertPolytomousKeyNodeOperation;
import eu.etaxonomy.taxeditor.operation.e4.RemotingCdmHandlerE4;

/**
 * @author k.luther
 * @date 17.11.2016
 *
 */
public class InsertNewNodeHandlerE4 extends RemotingCdmHandlerE4{

    private PolytomousKeyNode parentNode ;


    public InsertNewNodeHandlerE4() {
        super(PolytomousKeyEditorLabels.INSERT_NODE_POLYTOMOUS_KEY_NODE_LABEL);
    }

    @Override
    public IStatus allowOperations(@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            @Named(IServiceConstants.ACTIVE_SHELL)Shell shell,
            @Named(IServiceConstants.ACTIVE_PART)MPart activePart,
            MHandledMenuItem menuItem) {

        PolytomousKeyListEditorE4 editor = (PolytomousKeyListEditorE4) activePart.getObject();

        if(editor.getTableItemCount() == 0) {
            PolytomousKey pk = editor.getViewerInputKey();
            parentNode = pk.getRoot();

        } else {
            if (selection.getFirstElement() instanceof PolytomousKeyNode) {
                parentNode = ((PolytomousKeyNode) selection.getFirstElement()).getParent();
            } else {
                return new Status(IStatus.ERROR,
                        "unknown", //$NON-NLS-1$
                        PolytomousKeyEditorLabels.NO_KEY_NODE_FOR_INSERT_NODE_SELECTED_MESSAGE);
            }
        }
        return Status.OK_STATUS;
    }

    @Override
    public AbstractOperation prepareOperation(@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            @Named(IServiceConstants.ACTIVE_SHELL)Shell shell,
            @Named(IServiceConstants.ACTIVE_PART)MPart activePart,
            MHandledMenuItem menuItem) {
        PolytomousKeyListEditorE4 editor = (PolytomousKeyListEditorE4) activePart.getObject();
        editor.changed(null);
        return new InsertPolytomousKeyNodeOperation(getTrigger(), false, parentNode, editor);
    }

    @Override
    public void onComplete() {
    }

    @CanExecute
    public boolean canExecute(@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            MHandledMenuItem menuItem){
        return true;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected Object getTrigger() {
        return this;
    }

}
