package eu.etaxonomy.taxeditor.editor.key.polytomous.e4.handler;

import javax.inject.Named;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;

import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.key.polytomous.e4.PolytomousKeyListEditorE4;
import eu.etaxonomy.taxeditor.editor.key.polytomous.operation.RefreshNodeNumberingOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;

public class RefreshNodeNumberingHandlerE4 {

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART)MPart activePart,
            @Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            MHandledMenuItem menuItem,
            UISynchronize sync){

        PolytomousKeyListEditorE4 editor = (PolytomousKeyListEditorE4) activePart.getObject();

        if (selection.getFirstElement() instanceof PolytomousKeyNode) {
            String label = menuItem.getLocalizedLabel();
            IUndoContext undoContext = EditorUtil.getUndoContext();

            PolytomousKeyNode keyNode = (PolytomousKeyNode) selection
                    .getFirstElement();

            AbstractPostOperation operation = new RefreshNodeNumberingOperation(
                    label, undoContext, keyNode, editor);
            AbstractUtility.executeOperation(operation, sync);
            editor.forceDirty();
        }
    }

    @CanExecute
    public boolean canExecute(@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            MHandledMenuItem menuItem){
        boolean canExecute = false;
        canExecute = selection!=null && selection.size()==1;
        menuItem.setVisible(canExecute);
        return canExecute;
    }
}
