/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.e4.container;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.taxeditor.editor.CdmDataTransfer;
import eu.etaxonomy.taxeditor.editor.name.e4.IDropTargetableE4;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.name.e4.dnd.NameEditorDropTargetEffect;
import eu.etaxonomy.taxeditor.editor.name.e4.dnd.NameEditorDropTargetListenerE4;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 *
 * @author pplitzner
 * @date Aug 24, 2017
 *
 */
public abstract class AbstractGroupE4 implements IDropTargetableE4{

	private Composite control;

	private IEclipseContext context;

	private final List<AbstractGroupedContainerE4> groupedContainers = new ArrayList<>();

	protected TaxonNameEditorE4 editor;

	private DropTarget target;
	private DropTargetListener dropListener;

	public AbstractGroupE4(TaxonNameEditorE4 editor){
		this.editor = editor;
		this.context = editor.getContext();
	}

	protected void createContent(){
		createControl();

		createContainers();

		// Drop functionality
		this.setDroppable(true);

		editor.getManagedForm().reflow(true);
	}

	protected void createControl() {
		control = editor.getToolkit().createComposite(editor.getControl());

		control.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB));
		TableWrapLayout layout = new TableWrapLayout();

		layout.topMargin = 0;
		layout.rightMargin = 0;
		layout.bottomMargin = 1;
		layout.leftMargin = 0;

		layout.verticalSpacing = 0;
		layout.horizontalSpacing = 0;
		control.setLayout(layout);

		control.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_GRAY));
	}

	public void add(AbstractGroupedContainerE4 groupedContainer){
		groupedContainers.add(groupedContainer);
	}

	public void remove(AbstractGroupedContainerE4 groupedContainer){
		groupedContainer.dispose();
		groupedContainers.remove(groupedContainer);

		if(!(this instanceof AcceptedGroupE4) && groupedContainers.isEmpty()){
			getEditor().removeGroup(this);
		}
	}

	public List<AbstractGroupedContainerE4> getGroupedContainers(){
		return groupedContainers;
	}


	public void setDroppable(boolean droppable) {
		if (droppable) {
			Transfer[] types = new Transfer[] {CdmDataTransfer.getInstance()};
			int operations = DND.DROP_MOVE;
			target = new DropTarget(control, operations);
			target.setTransfer(types);
			dropListener = new NameEditorDropTargetListenerE4(this);
			target.addDropListener(dropListener);

			target.setDropTargetEffect(new NameEditorDropTargetEffect(control));

		} else {
			if (dropListener != null) {
				target.removeDropListener(dropListener);
			}
		}
	}

	@Override
	public IEclipseContext getContext() {
	    return context;
	}

	@Override
    public TaxonNameEditorE4 getEditor() {
		return editor;
	}

	@Override
    public void dragEntered() {
		Color color = AbstractUtility.getColor(Resources.COLOR_DRAG_ENTER);

		for(AbstractGroupedContainerE4 container : groupedContainers){
			container.setBackground(color);
		}
	}

	@Override
    public void dragLeft() {
		for(AbstractGroupedContainerE4 container : groupedContainers){
			container.restoreColor();
		}
	}

	/** {@inheritDoc} */
	@Override
    public boolean postOperation(Object objectAffectedByOperation) {
		editor.changed(objectAffectedByOperation);
		redraw();
		return true;
	}

	public void redraw(){
	    emptyGroup();

		createContainers();
	}

	protected abstract void createContainers();

	@Override
	public Composite getControl() {
		return control;
	}

	@Override
	public boolean onComplete() {
		return true;
	}

	public void dispose(){
		if(getControl() != null){
		    getControl().setMenu(null);
			for(AbstractGroupedContainerE4 container : getGroupedContainers()){
				container.dispose();
			}
			getControl().dispose();
		}
	}

	protected void emptyGroup(){
		for(AbstractGroupedContainerE4 container : groupedContainers){
		    container.getControl().setMenu(null);
			container.dispose();
		}
		groupedContainers.clear();
	}

	protected void setMenuToGroup(){
        for(AbstractGroupedContainerE4 container : groupedContainers){
            container.setMenu();

        }

    }
}
