/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.e4.container;

import java.util.ArrayList;
import java.util.List;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;

/**
 *
 * @author pplitzner
 * @date Aug 24, 2017
 *
 */
public abstract class AbstractHomotypicalGroupContainerE4 extends AbstractGroupE4 {

	private HomotypicalGroup group;

	public AbstractHomotypicalGroupContainerE4(TaxonNameEditorE4 editor, HomotypicalGroup group) {
		super(editor);
		this.group = group;

		createContent();
	}

	protected abstract void createSynonymContainer();

	protected List<Synonym> getSynonyms(){
		Taxon taxon = getEditor().getTaxon();
		return taxon.getSynonymsInGroup(getGroup());
	}

	public void setGroup(HomotypicalGroup group) {
		this.group = group;
	}

	public void redraw(HomotypicalGroup homotypicalGroup){
		if(redrawNeeded(homotypicalGroup)){
			setGroup(homotypicalGroup);
			redraw();
		}else{
		    setMenuToGroup();
		}
	}

	protected boolean redrawNeeded(HomotypicalGroup homotypicalGroup) {
		Taxon taxon = getEditor().getTaxon();
		homotypicalGroup = HibernateProxyHelper.deproxy(homotypicalGroup, HomotypicalGroup.class);
		List<Synonym> synonyms = taxon.getSynonymsInGroup(homotypicalGroup);

		List<Synonym> presentSynonyms = new ArrayList<Synonym>();
		List<AbstractGroupedContainerE4> containers = getGroupedContainers();
		if (containers == null || containers.isEmpty() || (containers.get(0) == null && containers.size() == 1)){
		    return true;
		}
		for(AbstractGroupedContainerE4 container : getGroupedContainers()){
			if(container.getData() instanceof Synonym){
				presentSynonyms.add((Synonym) container.getData());
			}
		}

		boolean result = ! (presentSynonyms.containsAll(synonyms) && synonyms.containsAll(presentSynonyms));

		return result;
	}

	public HomotypicalGroup getGroup() {
		return group;
	}
}
