/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.e4.container;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.graphics.Color;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 *
 * @author pplitzner
 * @date Aug 24, 2017
 *
 */
public class AcceptedGroupE4 extends AbstractHomotypicalGroupContainerE4 {

	private AcceptedNameContainerE4 acceptedContainer;

	public AcceptedGroupE4(TaxonNameEditorE4 editor, HomotypicalGroup group) {
		super(editor, group);
	}

	@Override
	public void dragEntered() {
		Color color = AbstractUtility.getColor(Resources.COLOR_DRAG_ENTER);
		acceptedContainer.setBackground(color);
		super.dragEntered();
	}

	@Override
	public void dragLeft() {
		acceptedContainer.restoreColor();
		super.dragLeft();
	}

	@Override
	public List<AbstractGroupedContainerE4> getGroupedContainers() {
		List<AbstractGroupedContainerE4> containers = new ArrayList<>();
		containers.add(getAcceptedNameContainer());
		containers.addAll(super.getGroupedContainers());
		return containers;
	}

	@Override
    protected void createContainers() {
		this.acceptedContainer = new AcceptedNameContainerE4(this, getEditor().getTaxon());
		acceptedContainer.createContent();
		createSynonymContainer();
	}

	@Override
    protected void createSynonymContainer(){
		for(Synonym synonym : getSynonyms()){
		    SynonymContainerE4 container = new SynonymContainerE4(this, synonym);
			this.add(container);
			container.createContent();
		}
	}

	public AcceptedNameContainerE4 getAcceptedNameContainer(){
		return acceptedContainer;
	}

	@Override
	protected void emptyGroup() {
	    if (acceptedContainer != null){
	        acceptedContainer.dispose();
	        acceptedContainer = null;
	    }

		super.emptyGroup();
	}

	@Override
	protected boolean redrawNeeded(HomotypicalGroup homotypicalGroup) {
		if(acceptedContainer != null && ! acceptedContainer.getData().equals(getEditor().getTaxon())){
			return true;
		}
		return super.redrawNeeded(homotypicalGroup);
	}

	@Override
	protected void setMenuToGroup(){
        super.setMenuToGroup();
        acceptedContainer.setMenu();
    }
}
