/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.e4.container;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.graphics.Font;

import eu.etaxonomy.cdm.format.taxon.TaxonRelationshipFormatter;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.strategy.cache.TaggedCacheHelper;
import eu.etaxonomy.cdm.strategy.cache.TaggedText;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.name.container.EditorAnnotation;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.name.e4.container.MisappliedGroupE4.ConceptType;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.NameHelper;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 *
 * @author pplitzner
 * @date Aug 24, 2017
 *
 */
public class ConceptContainerE4 extends AbstractGroupedContainerE4<Taxon> {
	TaxonRelationship relationship;
	TaxonRelationshipFormatter misappliedFormatter;
	ConceptType conceptType;

	public ConceptContainerE4(AbstractGroupE4 group, Taxon misappliedName,  ConceptType conceptType) {
		super(group, misappliedName);
		this.conceptType = conceptType;
		misappliedFormatter = new TaxonRelationshipFormatter();

	}

	/** {@inheritDoc} */
	@Override
	protected void initializeComposite() {
		setIsDraggable(true);
		setFont(getViewerFont());
		TaxonNameEditorE4 editor = getEditor();
		Taxon taxon = editor.getTaxon();
		for (TaxonRelationship rel: taxon.getTaxonRelations(getMisappliedName())){
			if (rel.getType().isAnyMisappliedName() && this.conceptType.equals(ConceptType.Misapplication)){
			    relationship = rel;
				break;
			}else if (rel.getType().isAnySynonym() && this.conceptType.equals(ConceptType.Synonym)){
			    relationship = rel;
                break;
			} else if (rel.getType().isInvalidDesignation() && this.conceptType.equals(ConceptType.InvalidDesignation)){
                relationship = rel;
                break;
            }
		}
		showSec();
		initTextViewer();
	}

	@Override
	protected void updateIcon() {
	    if (relationship.getType().isAnyMisappliedName() ){
	        setIcon(MISAPPLIEDNAME_ICON);
	    }else if (relationship.getType().isInvalidDesignation()){
            setIcon(INVALID_TYPE_DESIGNATION_ICON);
        }else if (relationship.getType().isProParte()){
	        setIcon(PRO_PARTE_SYNONYM_ICON);
	    }else if (relationship.getType().isAnySynonym()){
            setIcon(PARTIAL_SYNONYM_ICON);
        }
	}

	private void showSec() {
		if (getMisappliedName() == null) {
			return;
		}
		List<Language> languages = new ArrayList<>();
		languages.add(CdmStore.getDefaultLanguage());
		List<TaggedText> taggedText = misappliedFormatter.getTaggedText(relationship, true, languages, true);

		if (taggedText != null){
    		String title = TaggedCacheHelper.createString(taggedText);
    		title = title.replace("&", "&&");
    		setNonEditableInfo(title, false);
		}

	}

	public Taxon getMisappliedName() {
		return getTaxonBase();
	}

	/** {@inheritDoc} */
	@Override
	protected Font getViewerFont() {
		return AbstractUtility.getFont(Resources.MISAPPLIEDNAME_FONT);
	}

	@Override
	public void showAnnotations() {
		if(getData().getSec() == null){
			getNameViewer().addAnnotation(
					new EditorAnnotation(0, Messages.MisapplicationContainer_SEC_REF_REQUIRED));
		}
		super.showAnnotations();
	}


	/** {@inheritDoc} */
	@Override
	protected void updateNonEditableInfo() {
		showSec();
	}

	@Override
	protected void updateIndent() {
		setIndent(MISAPPLIEDNAME_INDENT);
	}

	@Override
	protected void initTextViewer() {
		updateIndent();

		updateIcon();
		String text;
		if (!conceptType.equals(ConceptType.Synonym) ){
		    text = NameHelper.getDisplayNameCache(getData());
		}else{
		    text = NameHelper.getDisplayName(getData());
		}

		if (text.length() == 0) {
			initEmptyText();
		} else {
			getNameViewer().setText(text);
			placeCursor();
		}
		calculateAnnotations();
	}

	@Override
    public void refresh() {
		// showNameRelations();
	    if (relationship.getType().isAnyMisappliedName()){
            conceptType = ConceptType.Misapplication;
        }else if (relationship.getType().isAnySynonym()){
            conceptType = ConceptType.Synonym;
        }else{
            conceptType = ConceptType.InvalidDesignation;
        }

		String text;
		if (!conceptType.equals(ConceptType.Synonym) ){
            text = NameHelper.getDisplayNameCache(getData());
        }else{
            text = NameHelper.getDisplayName(getData());
        }


		if (getNameViewer().getTextWidget() == null) {
			// we might get here via dnd. Look slike it can be ignored
			return;
		}

		if (text.length() == 0) {
			initEmptyText();
		} else if (!getNameViewer().getTextWidget().getText().equals(text)) {
			removeListener();
			getNameViewer().getTextWidget().setText(text);
			addListener();
		}

		updateNonEditableInfo();

		updateIcon();
		// placeCursor();
		updateIndent();
		setDelayedSelection();

		enableFreeText();
	}

}
