/**
 *
 */
package eu.etaxonomy.taxeditor.editor.name.e4.container;

import java.util.ArrayList;
import java.util.List;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;

/**
 *
 * @author pplitzner
 * @date Aug 24, 2017
 *
 */
public class ContainerFactoryE4 {

	public static void createOrUpdateAcceptedTaxonsHomotypicGroup(
	        TaxonNameEditorE4 taxonNameEditor) {

		if(taxonNameEditor.getAcceptedGroup() == null){
			taxonNameEditor.setAcceptedGroup(new AcceptedGroupE4(taxonNameEditor, taxonNameEditor.getTaxon().getHomotypicGroup()));
		}
		else{
			taxonNameEditor.getAcceptedGroup().redraw(taxonNameEditor.getTaxon().getHomotypicGroup());
		}

	}

	public static void createOrUpdateHeterotypicSynonymyGroups(
			TaxonNameEditorE4 taxonNameEditor) {
		List<HomotypicalSynonymGroupE4> retainedGroups = new ArrayList<>();

		List<HomotypicalSynonymGroupE4> heterotypicSynonymGroups = taxonNameEditor.getHeterotypicSynonymGroups();

		if (heterotypicSynonymGroups != null) {

			for(HomotypicalSynonymGroupE4 group : heterotypicSynonymGroups){
				retainedGroups.add(group);
			}
		}

		for(HomotypicalGroup homotypicalGroup : taxonNameEditor.getTaxon().getHeterotypicSynonymyGroups()){
		    homotypicalGroup = HibernateProxyHelper.deproxy(homotypicalGroup, HomotypicalGroup.class);
		    HomotypicalSynonymGroupE4 group = createOrUpdateHeterotypicSynonymyGroup(taxonNameEditor, homotypicalGroup);

			retainedGroups.remove(group);
		}

		for(HomotypicalSynonymGroupE4 retainedGroup : retainedGroups){
			taxonNameEditor.removeGroup(retainedGroup);
		}
	}


	public static HomotypicalSynonymGroupE4 createOrUpdateHeterotypicSynonymyGroup(
	        TaxonNameEditorE4 taxonNameEditor, HomotypicalGroup homotypicalGroup){
	    HomotypicalSynonymGroupE4 group = null;
		if(taxonNameEditor.getHomotypicalGroupContainer(homotypicalGroup) == null){
			group = new HomotypicalSynonymGroupE4(taxonNameEditor, homotypicalGroup);
			taxonNameEditor.addHeterotypicSynonymGroup(group);
			if(taxonNameEditor.getMisappliedGroup() != null){
				group.getControl().moveAbove(taxonNameEditor.getMisappliedGroup().getControl());
			}
		}
		else{
			group = taxonNameEditor.getHomotypicalGroupContainer(homotypicalGroup);
			group.redraw(homotypicalGroup);
		}

		return group;
	}

	public static void createOrUpdateMisapplicationsGroup(
	        TaxonNameEditorE4 taxonNameEditor) {
		MisappliedGroupE4 group = taxonNameEditor.getMisappliedGroup();
		Taxon taxon = HibernateProxyHelper.deproxy(taxonNameEditor.getTaxon(), Taxon.class);
		if(taxon.getMisappliedNames(true).isEmpty() && taxon.getProParteAndPartialSynonyms().isEmpty() && taxon.getInvalidDesignations().isEmpty()){
			taxonNameEditor.removeGroup(group);
			taxonNameEditor.setMisapplicationsGroup(null);
		}else{
			if(group == null){
				taxonNameEditor.setMisapplicationsGroup(new MisappliedGroupE4(taxonNameEditor));
			}else{
				group.redraw();
			}
		}

	}

	public static void setMenuToAllContainers(TaxonNameEditorE4 taxonNameEditor){
	    if (taxonNameEditor == null) {
            return;
        }
	    if (taxonNameEditor.getAcceptedGroup() != null){
	        taxonNameEditor.getAcceptedGroup().setMenuToGroup();
	    }
	    if (taxonNameEditor.getTaxon() != null){
    	    for(HomotypicalGroup homotypicalGroup : taxonNameEditor.getTaxon().getHeterotypicSynonymyGroups()){
                homotypicalGroup = HibernateProxyHelper.deproxy(homotypicalGroup, HomotypicalGroup.class);
                HomotypicalSynonymGroupE4 group = taxonNameEditor.getHomotypicalGroupContainer(homotypicalGroup);
                if (group != null){
                    group.setMenuToGroup();
                }
            }
    	    if (taxonNameEditor.getMisappliedGroup() != null){
    	        taxonNameEditor.getMisappliedGroup().setMenuToGroup();
    	    }
	    }
	}

}
