/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.e4.container;

import java.util.HashSet;
import java.util.Set;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;

/**
 *
 * @author pplitzner
 * @date Aug 24, 2017
 *
 */
public class MisappliedGroupE4 extends AbstractGroupE4{

    protected enum ConceptType {
        Misapplication,
        Synonym,
        InvalidDesignation
    }


	public MisappliedGroupE4(TaxonNameEditorE4 editor) {
		super(editor);

		createContent();
	}

	@Override
	protected void createContainers() {

		for(Taxon proParteSynonym : getEditor().getTaxon().getAllProParteSynonyms()){
            ConceptContainerE4 container = new ConceptContainerE4(this, proParteSynonym, ConceptType.Synonym);
            this.add(container);
            container.createContent();
        }
		for(Taxon invalidDes : getEditor().getTaxon().getInvalidDesignations()){

            ConceptContainerE4 container = new ConceptContainerE4(this, invalidDes, ConceptType.InvalidDesignation);
            this.add(container);
            container.createContent();
        }

		for(Taxon misapplication : getEditor().getTaxon().getAllMisappliedNames()){

            ConceptContainerE4 container = new ConceptContainerE4(this, misapplication, ConceptType.Misapplication);
            this.add(container);
            container.createContent();
        }
	}

	@Override
	public void redraw() {
		if(redrawNeeded()){
			super.redraw();
		}else{
		    super.setMenuToGroup();
		}
	}

	private boolean redrawNeeded() {
		Set<Taxon> concepts = getEditor().getTaxon().getMisappliedNames(true);
		concepts.addAll(getEditor().getTaxon().getProParteAndPartialSynonyms());
		concepts.addAll(getEditor().getTaxon().getInvalidDesignations());
		Set<Taxon> presentConcepts = new HashSet<>();

		for (AbstractGroupedContainerE4<Taxon> container  : getGroupedContainers()){
		    presentConcepts.add(container.getData());
		}
		return ! (presentConcepts.containsAll(concepts) && concepts.containsAll(presentConcepts));
	}
}
