/**
 *
 */
package eu.etaxonomy.taxeditor.editor.name.e4.dnd;

import org.eclipse.core.runtime.Assert;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;

import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.name.e4.IDropTargetableE4;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.name.e4.container.AbstractHomotypicalGroupContainerE4;
import eu.etaxonomy.taxeditor.editor.name.e4.container.MisappliedGroupE4;
import eu.etaxonomy.taxeditor.editor.name.e4.operation.MoveSynonymToAnotherAcceptedTaxonOperationE4;
import eu.etaxonomy.taxeditor.editor.name.operation.ChangeConceptRelationshipTypeOperation;
import eu.etaxonomy.taxeditor.editor.name.operation.ChangeConceptToSynonymOperation;
import eu.etaxonomy.taxeditor.editor.name.operation.ChangeHomotypicGroupOperation;
import eu.etaxonomy.taxeditor.editor.name.operation.ChangeSynonymToMisapplicationOperation;
import eu.etaxonomy.taxeditor.event.EventUtility;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;

/**
 *
 * @author pplitzner
 * @date Aug 24, 2017
 *
 */
public class NameEditorDropTargetListenerE4 extends DropTargetAdapter {

	private IDropTargetableE4 target;

	public NameEditorDropTargetListenerE4(IDropTargetableE4 target){
		this.target = target;
	}

	/** {@inheritDoc} */
	@Override
    public void dragEnter(DropTargetEvent dropTargetEvent) {
		target.dragEntered();
	}

	/** {@inheritDoc} */
	@Override
    public void dragLeave(DropTargetEvent dropTargetEvent) {
		target.dragLeft();
	}


	/** {@inheritDoc} */
	@Override
    public void drop(DropTargetEvent dropTargetEvent) {

		TaxonBase taxonBase = getDroppedTaxonBase(dropTargetEvent);

		Assert.isNotNull(taxonBase);

		if(taxonBase.getHomotypicGroup().equals(getHomotypicalGroup())){
			// do nothing when dragged to the same group
			dropTargetEvent.detail = DND.DROP_NONE;
			return;
		}

		if (target instanceof MisappliedGroupE4){
		   dropTargetEvent.detail = DND.DROP_NONE;
	       return;
		}

		// Execute operations at end of drag event
		AbstractPostOperation operation = createOperation(taxonBase);

		// Execute operation if it has been initiated
		if (operation == null) {
			MessagingUtils.warn(getClass(), String.format(Messages.NameEditorDropTargetListener_UNSUCCESSFULL_DROP, taxonBase.getClass(), target));
			dropTargetEvent.detail = DND.DROP_NONE;
		} else {
			AbstractUtility.executeOperation(operation, target.getContext().get(UISynchronize.class));
		}
		target.getEditor().getConversationHolder().commit();
		if (!target.equals(EventUtility.getTaxonEditor())){
			((TaxonNameEditorE4)EventUtility.getTaxonEditor()).redraw();
			((TaxonNameEditorE4)EventUtility.getTaxonEditor()).getConversationHolder().commit();

		}
		target.getEditor().redraw();

	}

	private AbstractPostOperation createOperation(TaxonBase taxonBase) {

		if(taxonBase instanceof Synonym){
			Synonym synonym = (Synonym) taxonBase;

			if(target instanceof AbstractHomotypicalGroupContainerE4){
				HomotypicalGroup homotypicalGroup = ((AbstractHomotypicalGroupContainerE4) target).getGroup();

				return new ChangeHomotypicGroupOperation(Messages.NameEditorDropTargetListener_CHANGE_HOMOTYPICAL_GROUP, getEditor().getUndoContext(),
						getEditor().getTaxon(), synonym, homotypicalGroup, target);
			}
			else if(target instanceof MisappliedGroupE4){
				return new ChangeSynonymToMisapplicationOperation(Messages.NameEditorDropTargetListener_CHANGE_SYNONYM_TO_MISAPP, getEditor().getUndoContext(),
						getEditor().getTaxon(), synonym, target);
			}
			else if(target instanceof TaxonNameEditorE4){
				if (target.equals(getEditor())){
					return new ChangeHomotypicGroupOperation(Messages.NameEditorDropTargetListener_CHANGE_HOMOTYPICAL_GROUP, getEditor().getUndoContext(),
						getEditor().getTaxon(), synonym, null, target);
				} else{
					getEditor().getConversationHolder().commit();
					((TaxonNameEditorE4) target).getConversationHolder().commit();

					return new MoveSynonymToAnotherAcceptedTaxonOperationE4(Messages.NameEditorDropTargetListener_CHANGE_ACC_TAXON, getEditor().getUndoContext(), synonym.getUuid(), ((TaxonNameEditorE4) target).getTaxon(), (TaxonNameEditorE4) target, (TaxonNameEditorE4) target);

				}
			}

		}else if(taxonBase instanceof Taxon){
			Taxon concept = (Taxon) taxonBase;

			if(target instanceof AbstractHomotypicalGroupContainerE4){
				HomotypicalGroup homotypicalGroup = ((AbstractHomotypicalGroupContainerE4) target).getGroup();

				return new ChangeConceptToSynonymOperation(Messages.NameEditorDropTargetListener_CHANGE_SYNONYM, getEditor().getUndoContext(),
						getEditor().getTaxon(), concept, homotypicalGroup, target);
			}
			else if(target instanceof MisappliedGroupE4){
				return new ChangeConceptRelationshipTypeOperation(Messages.NameEditorDropTargetListener_CHANGE, getEditor().getUndoContext(),
						getEditor().getTaxon(), concept, TaxonRelationshipType.MISAPPLIED_NAME_FOR(), target);
			}
		}


		return null;
	}

	/** {@inheritDoc} */
	@Override
	public void dropAccept(DropTargetEvent event) {
		// pass
	}


	private HomotypicalGroup getHomotypicalGroup() {
		if(target instanceof AbstractHomotypicalGroupContainerE4){
			return ((AbstractHomotypicalGroupContainerE4) target).getGroup();
		}
		return null;
	}

	private TaxonBase getDroppedTaxonBase(DropTargetEvent dropTargetEvent){
		if(dropTargetEvent.data instanceof ICdmBase[]){

			ICdmBase[] cdmBaseObjects = (ICdmBase[]) dropTargetEvent.data;

			if(cdmBaseObjects[0] instanceof TaxonBase){
				return (TaxonBase) cdmBaseObjects[0];
			}
		}
		return null;
	}

	private TaxonNameEditorE4 getEditor() {
		return (TaxonNameEditorE4) EventUtility.getTaxonEditor();
	}
}
