/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.e4.handler;

import java.util.Set;

import javax.inject.Named;

import org.apache.log4j.Logger;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.e4.TaxonEditorInputE4;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.name.handler.NameEditorMenuPropertyTester;
import eu.etaxonomy.taxeditor.editor.name.operation.ChangeSynonymToAcceptedTaxonOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.ui.dialog.selection.TaxonNodeSelectionDialog;

/**
 *
 * @author pplitzner
 * @since Aug 28, 2017
 *
 */
public class ChangeSynonymToAcceptedTaxonHandlerE4 implements IPostOperationEnabled {
	private static final Logger logger = Logger
			.getLogger(ChangeSynonymToAcceptedTaxonHandlerE4.class);

	private TaxonNameEditorE4 editor;
    private EPartService partService;
    private MApplication application;
    private EModelService modelService;

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART) MPart activePart,
            @Named(IServiceConstants.ACTIVE_SELECTION) IStructuredSelection selection, EPartService partService,
            EModelService modelService, MApplication application, @Named(IServiceConstants.ACTIVE_SHELL) Shell shell,
            UISynchronize sync) {

        this.modelService = modelService;
        this.application = application;
        this.partService = partService;

        editor = (TaxonNameEditorE4) activePart.getObject();

        TaxonEditorInputE4 input = editor.getEditorInput();

		// Get synonym from selection
		if (!(selection.getFirstElement() instanceof Synonym)) {
			logger.error("Selection does not contain a Synonym"); //$NON-NLS-1$
			return;
		}

		Synonym synonym = (Synonym) selection.getFirstElement();

		// Force user to save taxon - not really necessary though, is it?
		if (!EditorUtil.forceUserSaveE4Editor(editor, shell)) {
			return;
		}

		// Get taxon
		Taxon taxon = input.getTaxon();

		TaxonNode newParentNode = TaxonNodeSelectionDialog.select(shell,// editor.getConversationHolder(),
		        Messages.ChangeSynonymToAcceptedTaxonHandler_SELECT_PARENT, null, null, input.getTaxonNode().getClassification().getUuid(), true);


		if(newParentNode != null){

			// TODO get synonyms from homotypical group and add them as homotypic synonyms to new accepted taxon
			// apply confirmation dialog
			HomotypicalGroup group = synonym.getHomotypicGroup();
			Set<TaxonName> namesInGroup = group.getTypifiedNames();
			// FIXME with this implementation we can not create a taxonNode that is a direct child of the classification node
			ChangeSynonymToAcceptedTaxonOperation operation = new ChangeSynonymToAcceptedTaxonOperation(Messages.ChangeSynonymToAcceptedTaxonHandler_CHANGE_SYN_TO_ACC_TAXON, EditorUtil.getUndoContext(),
					taxon, newParentNode, synonym, namesInGroup, this, editor, editor.getEditorInput()); //$NON-NLS-1$

			AbstractUtility.executeOperation(operation, sync);
		}

	}

	/** {@inheritDoc} */
	@Override
    public boolean postOperation(Object objectAffectedByOperation) {

		// Redraw existing editor
		((IPostOperationEnabled) editor).postOperation(null);

		editor.save(AbstractUtility.getMonitor());

		if (objectAffectedByOperation instanceof TaxonNode) {

			// Open new unsaved editor with existing taxon's parent as temporary parent
			TaxonNode newNode = (TaxonNode) objectAffectedByOperation;

			EditorUtil.openTaxonNodeE4(newNode.getUuid(), modelService, partService, application);
		}
		return true;
	}

	@Override
    public boolean onComplete() {
		return false;
	}


    @CanExecute
    public boolean canExecute(@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            MHandledMenuItem menuItem){
        boolean canExecute = false;
        if(selection.size()==1){
            Object selectedElement = selection.getFirstElement();
            canExecute =
                    NameEditorMenuPropertyTester.isNotHomotypicSynonymOfAcceptedTaxon(selectedElement)
                    && !NameEditorMenuPropertyTester.isAccepted(selectedElement)
                    && NameEditorMenuPropertyTester.isNotMisapplication(selectedElement)
                    && NameEditorMenuPropertyTester.isNotProparteSynonym(selectedElement)
                    && NameEditorMenuPropertyTester.isNotInvalidDesignation(selectedElement);
            menuItem.setVisible(canExecute);
        }
        return canExecute;
    }

}
