/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.e4.handler;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.name.handler.NameEditorMenuPropertyTester;
import eu.etaxonomy.taxeditor.editor.name.operation.ChangeConceptRelationshipTypeOperation;
import eu.etaxonomy.taxeditor.editor.name.operation.ChangeSynonymToMisapplicationOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;

/**
 *
 * @author pplitzner
 * @since Aug 28, 2017
 *
 */
public class ChangeToMisapplicationHandlerE4 {

    private TaxonNameEditorE4 editor;

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART)MPart activePart,
            @Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            @Named(IServiceConstants.ACTIVE_SHELL)Shell shell,
            MHandledMenuItem menuItem,
            UISynchronize sync) {

        editor = (TaxonNameEditorE4) activePart.getObject();

        if (!EditorUtil.forceUserSaveE4Editor(editor, shell)){
            return ;
        }
		Object selectedElement = selection.getFirstElement();

		AbstractPostTaxonOperation operation = null;
		if(selectedElement instanceof Taxon){
        	operation = new ChangeConceptRelationshipTypeOperation(menuItem.getLocalizedLabel(),
        							editor.getUndoContext(), editor.getTaxon(), (Taxon) selectedElement, TaxonRelationshipType.MISAPPLIED_NAME_FOR(), editor);
        }
        if(selectedElement instanceof Synonym){

                operation = new ChangeSynonymToMisapplicationOperation(menuItem.getLocalizedLabel(),
                        editor.getUndoContext(), editor.getTaxon(), (Synonym) selectedElement, editor);


        }



        AbstractUtility.executeOperation(operation, sync);
	}

    @CanExecute
    public boolean canExecute(@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            MHandledMenuItem menuItem){
        boolean canExecute = false;
        if(selection.size()==1){
            Object selectedElement = selection.getFirstElement();
            canExecute = NameEditorMenuPropertyTester.isNotMisapplication(selectedElement);
        }
        menuItem.setVisible(canExecute);
        return canExecute;
    }

}
