/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.e4.handler;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.name.handler.NameEditorMenuPropertyTester;
import eu.etaxonomy.taxeditor.editor.name.operation.CreateSynonymInExistingHomotypicalGroupOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.parser.ParseHandler;

/**
 *
 * @author pplitzner
 * @since Aug 28, 2017
 *
 */
public class CreateSynonymInHomotypicalGroupHandlerE4 {

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART)MPart activePart,
            @Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            MHandledMenuItem menuItem,
            UISynchronize sync) {

        TaxonNameEditorE4 editor = (TaxonNameEditorE4) activePart.getObject();

		Object selecteObject = selection.getFirstElement();

		if(! (selecteObject instanceof Synonym)){
			return ;
		}

		Synonym synonym = (Synonym) selecteObject;

		HomotypicalGroup group = synonym.getHomotypicGroup();

		TaxonName newSynonymName = ParseHandler.createEmptyName();

		AbstractPostOperation<?> operation;

		operation = new CreateSynonymInExistingHomotypicalGroupOperation(menuItem.getLocalizedLabel(), editor.getUndoContext(),
        					editor.getTaxon(), group, newSynonymName, editor);
        AbstractUtility.executeOperation(operation, sync);

	}


    @CanExecute
    public boolean canExecute(@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            MHandledMenuItem menuItem){
        boolean canExecute = false;
        Object selectedElement = selection.getFirstElement();
        canExecute = NameEditorMenuPropertyTester.isSynonym(selectedElement);
        menuItem.setVisible(canExecute);
        return canExecute;
    }
}
