/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.e4.handler;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;

import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.name.handler.NameEditorMenuPropertyTester;
import eu.etaxonomy.taxeditor.editor.name.operation.ChangeHomotypicalGroupBasionymOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;

/**
 *
 * @author pplitzner
 * @since Aug 28, 2017
 *
 */
public class SetBasionymHandlerE4 {

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART)MPart activePart,
            @Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            MHandledMenuItem menuItem,
            UISynchronize sync) {

        TaxonNameEditorE4 editor = (TaxonNameEditorE4) activePart.getObject();

		TaxonBase selectedTaxonBase = (TaxonBase) selection.getFirstElement();

		AbstractPostOperation operation;

		operation = new ChangeHomotypicalGroupBasionymOperation(menuItem.getLocalizedLabel(), editor.getUndoContext(),
        					editor.getTaxon(), selectedTaxonBase, editor);
        AbstractUtility.executeOperation(operation, sync);
	}

    @CanExecute
    public boolean canExecute(@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            MHandledMenuItem menuItem){
        boolean canExecute = false;
        if(selection.size()==1){
            Object selectedElement = selection.getFirstElement();
            canExecute = NameEditorMenuPropertyTester.isSynonymInHomotypicalGroupWithMoreSynonyms(selectedElement);
        }
        menuItem.setVisible(canExecute);
        return canExecute;
    }
}
