/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.e4.handler;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.name.handler.NameEditorMenuPropertyTester;
import eu.etaxonomy.taxeditor.editor.name.operation.SwapSynonymAndAcceptedOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 *
 * @author pplitzner
 * @since Aug 28, 2017
 *
 */
public class SwapSynonymAndAcceptedHandlerE4 implements IPostOperationEnabled {

    private TaxonNameEditorE4 editor;
    private Taxon taxon;
    private EPartService partService;
    private MPart activePart;
    private MApplication application;
    private EModelService modelService;

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART) MPart activePart,
            @Named(IServiceConstants.ACTIVE_SELECTION) IStructuredSelection selection,
            @Named(IServiceConstants.ACTIVE_SHELL) Shell shell, MHandledMenuItem menuItem, EPartService partService,
            EModelService modelService, MApplication application,
            UISynchronize sync) {

        this.activePart = activePart;
        this.modelService = modelService;
        this.application = application;
        this.partService = partService;

        editor = (TaxonNameEditorE4) activePart.getObject();

        Synonym synonym = (Synonym) selection.getFirstElement();

        // Force user to save taxon - not really necessary though, is it?
        if (!EditorUtil.forceUserSaveE4Editor(editor, shell)) {
            return;
        }

        SwapSynonymAndAcceptedOperation operation = new SwapSynonymAndAcceptedOperation(menuItem.getLocalizedLabel(),
                editor.getUndoContext(), editor.getTaxon(), synonym, this, editor.getEditorInput());

        AbstractUtility.executeOperation(operation, sync);

    }

    @CanExecute
    public boolean canExecute(@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            MHandledMenuItem menuItem){
        boolean canExecute = false;
        if(selection.size()==1){
            Object selectedElement = selection.getFirstElement();
            canExecute = NameEditorMenuPropertyTester.isSynonym(selectedElement);
        }
        menuItem.setVisible(canExecute);
        return canExecute;
    }

    @Override
    public boolean postOperation(Object objectAffectedByOperation) {
        // Redraw existing editor
        // ((IPostOperationEnabled) editor).postOperation(null);

        editor.save(AbstractUtility.getMonitor());
        partService.hidePart(activePart);

        if (objectAffectedByOperation instanceof Taxon) {
            taxon = (Taxon) objectAffectedByOperation;
        }
        return true;
    }

    @Override
    public boolean onComplete() {
        Display display = Display.getCurrent();
        display.asyncExec(new Runnable() {
            @Override
            public void run() {
                try {
                    EditorUtil.openTaxonBaseE4(taxon.getUuid(), modelService, partService, application);
                } catch (Exception e) {
                    MessagingUtils.warningDialog(Messages.SwapSynonymAndAcceptedHandler_COULD_NOT_OPEN, this,
                            e.getMessage());
                }

            }
        });
        return true;
    }
}
