/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.operation;

import java.util.Set;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.SynonymType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>ChangeSynonymToMisapplicationOperation class.</p>
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 14.01.2009
 * @version 1.0
 */
public class ChangeSynonymToMisapplicationOperation extends AbstractPostTaxonOperation {

	private final Synonym synonym;
	private Taxon misapplication;
	private Set<SynonymType> synonymTypes;

	/**
	 * <p>Constructor for ChangeSynonymToMisapplicationOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param synonym a {@link eu.etaxonomy.cdm.model.taxon.Synonym} object.
	 * @param editor a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public ChangeSynonymToMisapplicationOperation(String label,
			IUndoContext undoContext, Taxon taxon, Synonym synonym, IPostOperationEnabled editor) {
		super(label, undoContext, taxon, editor);

		this.synonym = synonym;
	}

	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

    	/*// get name from synonym
		TaxonNameBase<?, ?> synonymName = synonym.getName();

		// make misapplied name with synonym name
		misapplication = Taxon.NewInstance(synonymName, null);
		CdmStore.getService(ITaxonService.class).saveOrUpdate(misapplication);
		monitor.worked(20);

		// store SynonymType for later undo operations
		synonymTypes = synonym.getRelationType(element);

		// remove synonym from taxon
		element.removeSynonym(synonym);
		monitor.worked(40);

		// add misapplied name to taxon
		// TODO add microcitation for misapplied name to property sheet (if microcitation is indeed needed?!)
		element.addMisappliedName(misapplication, null, null);*/


		misapplication =
		        (Taxon) CdmStore.getService(ITaxonService.class).changeSynonymToRelatedTaxon(synonym.getUuid(),
		                element.getUuid(),
		                TaxonRelationshipType.MISAPPLIED_NAME_FOR(),
		                null,
		                null).getCdmEntity();
		// redraw editor if exists
		return postExecute(misapplication);
	}

	/** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// add misapplied name to taxon
		// TODO add citation for misapplied name to property sheet
		element.addMisappliedName(misapplication, null, null);

		// remove synonym from taxon
		element.removeSynonym(synonym);

		// redraw editor if exists
		return postExecute(misapplication);
	}

	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		// remove misapplied name from taxon
		element.removeTaxon(misapplication, TaxonRelationshipType.MISAPPLIED_NAME_FOR());

		// add synonym to taxon
		for (SynonymType synonymType : synonymTypes){
			element.addSynonym(synonym, synonymType);
		}

		// redraw editor if exists
		return postExecute(null);
	}
}
