/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.name.operation;

import java.util.UUID;

import org.eclipse.core.commands.operations.IUndoContext;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 * @author k.luther
 * @since 02.05.2018
 *
 */
public abstract class CreateNewTaxonBaseOperation extends AbstractPostTaxonOperation {

    /**
     * @param label
     * @param undoContext
     * @param taxonUuid
     */
    public CreateNewTaxonBaseOperation(String label, IUndoContext undoContext, UUID taxonUuid) {
        super(label, undoContext, taxonUuid);

    }

    /**
     * @param label
     * @param undoContext
     * @param taxon
     * @param postOperationEnabled
     */
    public CreateNewTaxonBaseOperation(String label, IUndoContext undoContext, Taxon taxon,
            IPostOperationEnabled postOperationEnabled) {
        super(label, undoContext, taxon, postOperationEnabled);
    }

    /**
     * @param newTaxonBase
     */
    protected void addToSaveList(CdmBase newCdmBase) {
        TaxonNameEditorE4 editor = null;
        Object activePart = EditorUtil.getActivePart();
        if (activePart instanceof TaxonNameEditorE4){
           editor = (TaxonNameEditorE4) activePart;
           if (newCdmBase instanceof Synonym){
               editor.getEditorInput().addToSaveNewSynonym((Synonym)newCdmBase);
           }else if (newCdmBase instanceof Taxon ){
               editor.getEditorInput().addToSaveNewConcept((Taxon)newCdmBase);
           }else if (newCdmBase instanceof TaxonName){
               editor.getEditorInput().addToSaveNewName((TaxonName)newCdmBase);
           }
           editor.setDirty();
        }
    }

}
