/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.HybridRelationship;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 * <p>CreateSynonymInExistingHomotypicalGroupOperation class.</p>
 *
 * @author n.hoffmann
 * @created 02.02.2009
 */
public class CreateSynonymInExistingHomotypicalGroupOperation extends
		CreateNewTaxonBaseOperation {

	private final HomotypicalGroup group;
	private final TaxonName newSynonymName;

	private Synonym synonym;

	/**
	 * <p>Constructor for CreateSynonymInExistingHomotypicalGroupOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param group a {@link eu.etaxonomy.cdm.model.name.HomotypicalGroup} object.
	 * @param newSynonymName a {@link eu.etaxonomy.cdm.model.name.TaxonNameBase} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public CreateSynonymInExistingHomotypicalGroupOperation(String label,
			IUndoContext undoContext, Taxon taxon, HomotypicalGroup group,
			TaxonName newSynonymName, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, taxon, postOperationEnabled);
		this.group = group;
		this.newSynonymName = newSynonymName;
	}

	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		// Add name to given homotypic group

//	    HomotypicalSynonymGroupE4 groupContainer = ((TaxonNameEditorE4)postOperationEnabled).getHomotypicalGroupContainer(group);
		group.addTypifiedName(newSynonymName);
//	    groupContainer.add(groupedContainer);
		monitor.worked(20);

		// Create a new synonym for the taxon
		// TODO add citations
		if(group.equals(element.getHomotypicGroup())){
			synonym = element.addHomotypicSynonymName(newSynonymName);
		}else{
			synonym = element.addHeterotypicSynonymName(newSynonymName);
		}
		for (HybridRelationship rel : newSynonymName.getHybridChildRelations()){
            if (!rel.getHybridName().isPersited()) {
                addToSaveList(rel.getHybridName());
            }
            if (!rel.getParentName().isPersited()) {
                addToSaveList(rel.getParentName());
            }
        }
		addToSaveList(synonym);
		monitor.worked(40);

//		Synonym affectedSynonym = null;
//
//		for(Synonym synonym : group.getSynonymsInGroup(taxon.getSec())){
//			if(synonym.getName() == synonymName){
//				affectedSynonym = synonym;
//				break;
//			}
//		}

		return postExecute(synonym);
	}

	/** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		element.removeSynonym(synonym);

		return postExecute(null);
	}
}
