/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.HybridRelationship;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 * <ol>
 * 	<li>Create synonym name using newSynonymText</li>
 * 	<li>Add name to new homotypic group</li>
 * 	<li>Create a new synonym for the taxon</li>
 * </ol>
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 16.01.2009
 */
public class CreateSynonymInNewGroupOperation extends CreateNewTaxonBaseOperation {

	private final TaxonName newSynonymName;
	private Synonym newSynonym;

	/**
	 * <p>Constructor for CreateSynonymInNewGroupOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param newSynonymName a {@link eu.etaxonomy.cdm.model.name.TaxonNameBase} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public CreateSynonymInNewGroupOperation(String label,
			IUndoContext undoContext, Taxon taxon, TaxonName newSynonymName, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, taxon, postOperationEnabled);

		this.newSynonymName = newSynonymName;
	}

	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		// Add name to new homotypic group
		HomotypicalGroup.NewInstance().addTypifiedName(newSynonymName);
		monitor.worked(20);
		HibernateProxyHelper.deproxy(element);
		newSynonym = element.addHeterotypicSynonymName(newSynonymName);
		for (HybridRelationship rel : newSynonymName.getHybridChildRelations()){
            if (!rel.getHybridName().isPersited()) {
                addToSaveList(rel.getHybridName());
            }
            if (!rel.getParentName().isPersited()) {
                addToSaveList(rel.getParentName());
            }
        }
		addToSaveList(newSynonym);
		monitor.worked(40);

		return postExecute(newSynonym);
	}



    /** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		// Remove the synonym
		element.removeSynonym(newSynonym);
		newSynonym = null;

		return postExecute(null);
	}
}
