/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import eu.etaxonomy.cdm.api.application.ICdmRepository;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.DeleteResult;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.api.service.config.TaxonBaseDeletionConfigurator;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.SynonymType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>DeleteSynonymOperation class.</p>
 *
 * @author p.ciardelli
 * @created 14.01.2009
 */
public class DeleteSynonymOperation extends DeleteTaxonBaseOperation {

	private final Synonym synonym;
	private SynonymType synonymType;


	/**
	 * <p>Constructor for DeleteSynonymOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param synonym a {@link eu.etaxonomy.cdm.model.taxon.Synonym} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public DeleteSynonymOperation(String label, IUndoContext undoContext, TaxonBaseDeletionConfigurator configurator,
			Taxon taxon, Synonym synonym, IPostOperationEnabled postOperationEnabled, IConversationEnabled conversationEnabled,ICdmEntitySessionEnabled cdmEntitySessionEnabled) {
		super(label, undoContext, configurator, postOperationEnabled, conversationEnabled, cdmEntitySessionEnabled);
		this.synonym = synonym;
		this.element = taxon;
	}

	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		// Store SynonymType for later undo operations
		synonymType = synonym.getType();
		monitor.worked(20);

		// Remove synonym from taxon
		ICdmRepository controller;

		controller = CdmStore.getCurrentApplicationConfiguration();

		ITaxonService service = controller.getTaxonService();


		result = service.isDeletable(synonym.getUuid(), configurator);



		monitor.worked(40);

		// Redraw editor if exists
		return Status.OK_STATUS;
//		return postExecute(element);
	}

	/** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		// Add synonym to taxon
		element.addSynonym(synonym, synonymType);

		// Redraw editor if exists
		return postExecute(synonym);
	}

    /**
     * @return the result
     */
    @Override
    public DeleteResult getResult() {
        return result;
    }

    /**
     * @param result the result to set
     */
    public void setResult(DeleteResult result) {
        this.result = result;
    }
}
