package eu.etaxonomy.taxeditor.editor.validation;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ui.IMemento;

import eu.etaxonomy.taxeditor.editor.internal.TaxeditorEditorPlugin;
import eu.etaxonomy.taxeditor.model.IContextListener;

/**
 * An {@code IContextListener} responsible for starting the
 * {@link ValidationDaemon}.
 * 
 * @see TaxeditorEditorPlugin#start(org.osgi.framework.BundleContext)
 * 
 * @author ayco_holleman
 * 
 */
public class ValidationContextListener implements IContextListener {

	@SuppressWarnings("unused")
	private static final Logger logger = Logger.getLogger(ValidationContextListener.class);

	private ValidationDaemon daemon;


	public ValidationContextListener(){
	}


	@Override
	public void contextAboutToStop(IMemento memento, IProgressMonitor monitor){
		if (daemon != null) {
			daemon.cancel();
			daemon.setCancelRequested();
		}
	}


	@Override
	public void contextStop(IMemento memento, IProgressMonitor monitor){
		if (daemon != null) {
			daemon.cancel();
			daemon.setCancelRequested();
		}
	}


	@Override
	public void contextStart(IMemento memento, IProgressMonitor monitor){
		if (daemon == null) {
			daemon = new ValidationDaemon();
		}
		daemon.schedule();
	}


	@Override
	public void contextRefresh(IProgressMonitor monitor){
		if (daemon != null) {
			daemon.cancel();
			daemon.setCancelRequested();
		}
	}


	@Override
	public void workbenchShutdown(IMemento memento, IProgressMonitor monitor){
		if (daemon != null) {
			daemon.cancel();
			daemon.setCancelRequested();
			daemon = null;
		}
	}

}
