package eu.etaxonomy.taxeditor.editor.validation;

import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

import eu.etaxonomy.cdm.api.service.IEntityValidationService;
import eu.etaxonomy.cdm.model.validation.EntityValidation;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * A job that repeatedly checks the error tables and refreshes the problem
 * markers created from them.
 *
 * @author ayco_holleman
 *
 */
public class ValidationDaemon extends Job {
	@SuppressWarnings("unused")
	private static final Logger logger = Logger.getLogger(ValidationDaemon.class);

	private final IEntityValidationService entityValidationService;

	// Might want to make this configurable:
	private int SLEEP_TIME = 5000;
	
	private boolean cancelRequested = false;



	public ValidationDaemon(){
		super(Messages.ValidationDaemon_RUNNING_DAEMON);
		entityValidationService = CdmStore.getService(IEntityValidationService.class);
	}


	@Override
	protected void canceling(){
		cancelRequested = true;
	}


	/**
	 * This method is called by {@link ValidationContextListener} rather than
	 * {@link Job#cancel()}, because that method does not have the desired
	 * effect.
	 */
	public void setCancelRequested(){
		cancelRequested = true;
	}


	@Override
	protected IStatus run(IProgressMonitor monitor){
		MarkerManager markerManager;
		List<EntityValidation> results;
		try {
			while (!cancelRequested) {
				results = entityValidationService.getValidationResults();
				markerManager = new MarkerManager(results);
				markerManager.deleteObsoleteMarkers();
				markerManager.createMarkers();
				Thread.sleep(SLEEP_TIME);
			}
			MessagingUtils.info(Messages.ValidationDaemon_VALIDATION_STOPPED);
			return Status.OK_STATUS;
		}
		catch (Throwable t) {
			MessagingUtils.info(Messages.ValidationDaemon_VALIDATION_EXCEPTION + t.getMessage());
			return Status.CANCEL_STATUS;
		}
	}

}
