/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.checklist;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;

import eu.etaxonomy.cdm.model.taxon.Taxon;

/**
 * @author a.oppermann
 * @created 30.04.2014
 * @version 1.0
 */
public class ChecklistEditorComparator extends ViewerComparator {

	private int propertyIndex;
	private static final int ASCENDING = 0;
	private static final int DESCENDING = 1;
	private int direction = ASCENDING;

	public ChecklistEditorComparator() {
		this.propertyIndex = 1;
		direction = ASCENDING;
	}

	public void setColumn(int column) {
		if (column == this.propertyIndex) {
			// Same column as last sort; toggle the direction
			direction = 1 - direction;
		} else {
			// New column; do an ascending sort
			this.propertyIndex = column;
			direction = DESCENDING;
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ViewerComparator#compare(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	@Override
	public int compare(Viewer viewer, Object e1, Object e2) {
	  Taxon d1 = (Taxon) e1;
	  Taxon d2 = (Taxon) e2;

		int result = 0;

		switch (propertyIndex) {

			default:
				result = d1.getTitleCache().compareTo(d2.getTitleCache());
		}

		// If descending order, flip the direction
		if (direction == DESCENDING) {
			result = -result;
		}
		return result;
	}
}
