/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.checklist.e4;

import java.util.Comparator;
import java.util.List;

import org.eclipse.nebula.widgets.nattable.data.IColumnPropertyAccessor;
import org.eclipse.nebula.widgets.nattable.data.ListDataProvider;
import org.eclipse.nebula.widgets.nattable.layer.AbstractLayerTransform;
import org.eclipse.nebula.widgets.nattable.layer.DataLayer;
import org.eclipse.nebula.widgets.nattable.layer.cell.IConfigLabelAccumulator;
import org.eclipse.nebula.widgets.nattable.selection.SelectionLayer;
import org.eclipse.nebula.widgets.nattable.viewport.ViewportLayer;

import ca.odell.glazedlists.EventList;
import ca.odell.glazedlists.FilterList;
import ca.odell.glazedlists.GlazedLists;
import ca.odell.glazedlists.SortedList;
import ca.odell.glazedlists.TransformedList;

/**
 * @author k.luther
 * @since 13.12.2018
 *
 */
public class BodyLayerStack<T> extends AbstractLayerTransform {
    private final FilterList<T> filterList;

    private final ListDataProvider<T> bodyDataProvider;

    private final SelectionLayer selectionLayer;
    private final ViewportLayer viewportLayer ;

    public BodyLayerStack(List<T> values, IColumnPropertyAccessor<T> columnPropertyAccessor, Comparator<T> comparator, IConfigLabelAccumulator labelAccumulator) {
        // wrapping of the list to show into GlazedLists
        // see http://publicobject.com/glazedlists/ for further information
        EventList<T> eventList = GlazedLists.eventList(values);
        TransformedList<T, T> rowObjectsGlazedList = GlazedLists.threadSafeList(eventList);
        SortedList<T> sortedList = new SortedList<>(rowObjectsGlazedList, comparator);
        // wrap the SortedList with the FilterList
        this.filterList = new FilterList<>(sortedList);

        this.bodyDataProvider =
                new ListDataProvider<>(this.filterList, columnPropertyAccessor);

        DataLayer bodyDataLayer = new DataLayer(bodyDataProvider);
        bodyDataLayer.setConfigLabelAccumulator(labelAccumulator);

//        // layer for event handling of GlazedLists and PropertyChanges
//        GlazedListsEventLayer<T> glazedListsEventLayer =
//                new GlazedListsEventLayer<>(bodyDataLayer, this.filterList);

        this.selectionLayer = new SelectionLayer(bodyDataLayer);
        viewportLayer = new ViewportLayer(getSelectionLayer());

        setUnderlyingLayer(viewportLayer);
    }

    public SelectionLayer getSelectionLayer() {
        return this.selectionLayer;
    }

    public ViewportLayer getViewPortLayer() {
        return this.viewportLayer;
    }

    public FilterList<T> getFilterList() {
        return this.filterList;
    }

    public ListDataProvider<T> getBodyDataProvider() {
        return this.bodyDataProvider;
}
}
