/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.checklist.e4;

import org.eclipse.nebula.widgets.nattable.config.AbstractRegistryConfiguration;
import org.eclipse.nebula.widgets.nattable.config.CellConfigAttributes;
import org.eclipse.nebula.widgets.nattable.config.IConfigRegistry;
import org.eclipse.nebula.widgets.nattable.config.IEditableRule;
import org.eclipse.nebula.widgets.nattable.edit.EditConfigAttributes;
import org.eclipse.nebula.widgets.nattable.filterrow.FilterRowTextCellEditor;
import org.eclipse.nebula.widgets.nattable.style.DisplayMode;

import eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.supplementalInfo.SupplementalInfoDisplayConverter;

/**
 * @author k.luther
 * @since 05.12.2018
 *
 */
public class DistributionCellEditorDataConversionConfiguration extends AbstractRegistryConfiguration {
    private DistributionEditor editor;

    public DistributionCellEditorDataConversionConfiguration(DistributionEditor editor) {
        super();
        this.editor = editor;
    }

    @Override
    public void configureRegistry(IConfigRegistry configRegistry) {

        //add display converter for string representation
        configRegistry.registerConfigAttribute(
                CellConfigAttributes.DISPLAY_CONVERTER,
                new SupplementalInfoDisplayConverter(),
                DisplayMode.NORMAL,
                DistributionEditor.TAXON_COLUMN);

        configRegistry.registerConfigAttribute(
                CellConfigAttributes.DISPLAY_CONVERTER,
                new SupplementalInfoDisplayConverter(),
                DisplayMode.NORMAL,
                DistributionEditor.RANK_COLUMN);

        configRegistry.registerConfigAttribute(
                CellConfigAttributes.DISPLAY_CONVERTER,
                new StatusDataDisplayConverter(),
                DisplayMode.NORMAL,
                DistributionEditorConfigLabelAccumulator.STATUS
                );

        //---register cell editors---


        //make cell editable
        configRegistry.registerConfigAttribute(
                EditConfigAttributes.CELL_EDITABLE_RULE,
                IEditableRule.ALWAYS_EDITABLE,
                DisplayMode.EDIT,
                DistributionEditorConfigLabelAccumulator.STATUS_EDITABLE
                );
        //add display converter for string representation
        configRegistry.registerConfigAttribute(
                CellConfigAttributes.DISPLAY_CONVERTER,
                new StatusDisplayConverter(),
                DisplayMode.NORMAL,
                DistributionEditorConfigLabelAccumulator.STATUS
                );
        //register status editor

        StatusDataCellEditor cellEditor = new StatusDataCellEditor(editor, new StatusComboBoxDataProvider(editor));
        configRegistry.registerConfigAttribute(EditConfigAttributes.CELL_EDITOR,
                cellEditor,
                DisplayMode.EDIT,
                DistributionEditorConfigLabelAccumulator.STATUS_EDITABLE
                );



        FilterRowTextCellEditor filterCellEditor = new FilterRowTextCellEditor();
        configRegistry.registerConfigAttribute(EditConfigAttributes.CELL_EDITOR,
                filterCellEditor,
                DisplayMode.EDIT,
                DistributionEditorConfigLabelAccumulator.FILTER_EDITABLE
                );







    }

}
