/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.checklist.e4;

import java.util.Collection;

import org.eclipse.e4.ui.workbench.modeling.ESelectionService;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.nebula.widgets.nattable.data.IRowDataProvider;
import org.eclipse.nebula.widgets.nattable.extension.e4.selection.E4SelectionListener;
import org.eclipse.nebula.widgets.nattable.layer.cell.ILayerCell;
import org.eclipse.nebula.widgets.nattable.layer.event.CellVisualChangeEvent;
import org.eclipse.nebula.widgets.nattable.layer.event.ILayerEvent;
import org.eclipse.nebula.widgets.nattable.selection.SelectionLayer;
import org.eclipse.nebula.widgets.nattable.selection.event.CellSelectionEvent;
import org.eclipse.nebula.widgets.nattable.selection.event.RowSelectionEvent;

import eu.etaxonomy.cdm.api.service.dto.RowWrapperDTO;
import eu.etaxonomy.cdm.api.service.dto.TaxonDistributionDTO;

/**
 * @author k.luther
 * @since 28.11.2018
 *
 */
public class DistributionCellSelectionListener extends E4SelectionListener<TaxonDistributionDTO>{
    private DistributionEditorPart part;

    public DistributionCellSelectionListener(
            ESelectionService service,
            SelectionLayer selectionLayer,
            IRowDataProvider<TaxonDistributionDTO> rowDataProvider,
            DistributionEditorPart part) {
        super(service, selectionLayer, rowDataProvider);
        this.part = part;
    }

    @Override
    public void handleLayerEvent(ILayerEvent event) {
        if(event instanceof CellSelectionEvent){
            CellSelectionEvent cellSelectionEvent = (CellSelectionEvent)event;
            int columnPosition = cellSelectionEvent.getColumnPosition();
            if(columnPosition>=part.getEditor().getFirstDataColumnIndex()){
                Collection<ILayerCell> selectedCells = cellSelectionEvent.getSelectionLayer().getSelectedCells();
                if(selectedCells.size()==1){
                    ILayerCell cell = selectedCells.iterator().next();
                    Object dataValue = cell.getDataValue();
                    if(dataValue!=null){
                        part.getSelectionService().setSelection(new StructuredSelection(dataValue));
                        return;
                    }else{
                        part.getSelectionService().setSelection(new StructuredSelection());
                        return;
                    }
                }
            }
        }
        else if(event instanceof RowSelectionEvent){
            RowSelectionEvent rowSelectionEvent = (RowSelectionEvent) event;
            int[] fullySelectedRowPositions = rowSelectionEvent.getSelectionLayer().getFullySelectedRowPositions();
            if(fullySelectedRowPositions.length==1){
                Object rowObject = part.getEditor().getBodyDataProvider().getRowObject(fullySelectedRowPositions[0]);
                if(rowObject instanceof RowWrapperDTO){
                    part.getSelectionService().setSelection(new StructuredSelection(((RowWrapperDTO) rowObject).getDescription()));
                    return;
                }
            }
        }
        else if (event instanceof CellVisualChangeEvent){
            CellVisualChangeEvent cellSelectionEvent = (CellVisualChangeEvent)event;
            int columnPosition = cellSelectionEvent.getColumnPosition();
            int rowPosition = cellSelectionEvent.getRowPosition();
            if(columnPosition>=part.getEditor().getFirstDataColumnIndex()){
                Object dataValue = cellSelectionEvent.getLayer().getDataValueByPosition(columnPosition, rowPosition);
                if(dataValue!=null){
                    part.getSelectionService().setSelection(new StructuredSelection(dataValue));
                    return;
                }else{
                    part.getSelectionService().setSelection(new StructuredSelection());
                    return;
                }

            }
        }
      //  part.getSelectionService().setSelection(new StructuredSelection());
    }
}
