/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.checklist.e4;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.inject.Inject;

import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.nebula.widgets.nattable.data.IColumnPropertyAccessor;

import eu.etaxonomy.cdm.api.service.dto.TaxonDistributionDTO;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementSource;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTerm;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.taxeditor.event.EventUtility;
import eu.etaxonomy.taxeditor.event.WorkbenchEventConstants;


/**
 * @author k.luther
 * @since 28.11.2018
 *
 */
public class DistributionColumnAccessor implements IColumnPropertyAccessor<TaxonDistributionDTO> {
    private DistributionEditor editor;
    public static final String DEFAULT_ENTRY = "";
    @Inject
    private IEventBroker eventBroker;

    public DistributionColumnAccessor(DistributionEditor editor) {
        this.editor = editor;
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public Object getDataValue(TaxonDistributionDTO rowObject, int columnIndex) {
     switch (columnIndex) {
            case 0:
                return rowObject.getNameCache();
            case 1:
                if (editor.isShowRank()){
                    return rowObject.getRankString();
                }else{
                    break;
                }
            default:
                break;
            }
        NamedArea area = editor.getAreaToColumnIndexMap().get(columnIndex);

        //TODO: do not get the distribution objcts, but the label of the status.
        Map<NamedArea, Set<DescriptionElementBase>> distributionMap = editor.taxonDistributionMap.get(rowObject.getTaxonUuid());
        if (distributionMap != null){
            Set<DescriptionElementBase> distributionsForArea = editor.taxonDistributionMap.get(rowObject.getTaxonUuid()).get(area);
            if (distributionsForArea == null){
                return null;
            }
            if (distributionsForArea.size() == 1){
                return distributionsForArea.iterator().next();
            }
            if (distributionsForArea.size() > 1){
                List<String> labels = new ArrayList();
                distributionsForArea.forEach(desc -> labels.add(((Distribution)desc).getStatus().getLabel()));
                return labels;
            }
        }
        return null;

    }


    /**
     * {@inheritDoc}
     */
    @Override
    public int getColumnCount() {
        return editor.getPropertyToLabelMap().size();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getColumnProperty(int columnIndex) {
        return editor.getPropertyToLabelMap().get(columnIndex);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int getColumnIndex(String propertyName){
        return editor.getPropertyToLabelMap().indexOf(propertyName);
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public void setDataValue(TaxonDistributionDTO taxonWrapper, int columnIndex, Object newValue) {
        if (newValue instanceof PresenceAbsenceTerm){
                NamedArea area =editor.getAreaToColumnIndexMap().get(columnIndex);
                Map<NamedArea, Set<DescriptionElementBase>> distributionMap = editor.taxonDistributionMap.get(taxonWrapper.getTaxonUuid());
                if (distributionMap == null){
                    distributionMap = new HashMap();
                    editor.taxonDistributionMap.put(taxonWrapper.getTaxonUuid(),distributionMap);
                }
                Distribution dist = null;
                Set<DescriptionElementBase> distributions = distributionMap.get(area);
                if (distributions != null && !distributions.isEmpty()){
                    DescriptionElementBase desc = distributions.iterator().next();
                    if (desc instanceof Distribution){
                        if (((PresenceAbsenceTerm)newValue).getId() == 0){
                            desc.getInDescription().removeElement(desc);
                            distributions.remove(desc);
                        }else {
                            ((Distribution)desc).setStatus((PresenceAbsenceTerm)newValue);
                        }
                    }
                }else{
                    if (((PresenceAbsenceTerm)newValue).getId() == 0){
                        return;
                    }
                    if (distributions == null){
                        distributions = new HashSet();
                        distributionMap.put(area, distributions);
                    }
                    dist = Distribution.NewInstance(area, (PresenceAbsenceTerm)newValue);
                    Set<TaxonDescription> descs = taxonWrapper.getDescriptionsWrapper().getDescriptions();
                    TaxonDescription desc;
                    if (descs.size() >= 1){
                        desc = descs.iterator().next();
                    }else {
                        desc = TaxonDescription.NewInstance();
                        taxonWrapper.getDescriptionsWrapper().getDescriptions().add(desc);
                    }
                    if (editor.getDefaultSource() != null){
                      dist.addSource(DescriptionElementSource.NewPrimarySourceInstance(editor.getDefaultSource(), null));
                    }
                    desc.addElement(dist);
                    if(desc.isPersited()){
                        editor.part.getCdmEntitySession().load(desc, true);
                    }
                    distributions.add(dist);


                }

                editor.descriptionsToSave.add(taxonWrapper);
            }
            EventUtility.postEvent(WorkbenchEventConstants.REFRESH_DETAILS, true);
    }

}
