// $Id$
/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.checklist.e4;

import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerEditor;
import org.eclipse.jface.viewers.ViewerCell;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.taxon.Taxon;

/**
 * @author k.luther
 * @date 28.09.2017
 *
 */
public class DistributionEditorHelper {

    public static Distribution getDistributionForColumn(SelectionChangedEvent event, Map<UUID, Integer> areaPosition) {
        //      List<TaxonDescription> listTaxonDescriptions = descriptionService.listTaxonDescriptions(taxon, null, null,
        //      null, null, null, DESC_INIT_STRATEGY);
        TableViewer viewer = null;
        TableViewerEditor editor = null;
        Taxon taxon = null;
        int columnIndex;
        if (event.getSource() instanceof TableViewer){
            viewer = (TableViewer)event.getSource();
        } else {
            return null;
        }
        if (viewer.getColumnViewerEditor() instanceof TableViewerEditor){
            editor = (TableViewerEditor) viewer.getColumnViewerEditor();
        } else {
            return null;
        }
        ViewerCell cell = editor.getFocusCell();
        if (cell == null){
            return null;
        }
        if (cell.getElement() instanceof Taxon){
            taxon =(Taxon)cell.getElement();
        } else {
            return null;
        }
        columnIndex = cell.getColumnIndex();
        UUID areaUuid = null;
        for (Map.Entry<UUID,Integer> entry: areaPosition.entrySet()){
            if (entry.getValue().intValue()== columnIndex){
                areaUuid = entry.getKey();
                break;
            }
        }
        if (areaUuid == null){
            return null;
        }
        Set<TaxonDescription> listTaxonDescriptions = taxon.getDescriptions();

        for (TaxonDescription td : listTaxonDescriptions) {
          for (DescriptionElementBase deb : td.getElements()) {
              if (deb instanceof Distribution) {
                  Distribution distribution = (Distribution) deb;
                  if (distribution != null && distribution.getArea() != null && distribution.getArea().getUuid().equals(areaUuid)){
                      return distribution;
                  }
              }
          }
        }
        return null;
    }

}
