/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.checklist.e4;

import org.eclipse.nebula.widgets.nattable.config.AbstractRegistryConfiguration;
import org.eclipse.nebula.widgets.nattable.config.CellConfigAttributes;
import org.eclipse.nebula.widgets.nattable.config.IConfigRegistry;
import org.eclipse.nebula.widgets.nattable.edit.EditConfigAttributes;
import org.eclipse.nebula.widgets.nattable.edit.editor.ComboBoxCellEditor;
import org.eclipse.nebula.widgets.nattable.filterrow.FilterRowDataLayer;
import org.eclipse.nebula.widgets.nattable.filterrow.FilterRowTextCellEditor;
import org.eclipse.nebula.widgets.nattable.filterrow.TextMatchingMode;
import org.eclipse.nebula.widgets.nattable.filterrow.config.FilterRowConfigAttributes;
import org.eclipse.nebula.widgets.nattable.style.DisplayMode;

import eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix.supplementalInfo.SupplementalInfoDisplayConverter;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author k.luther
 * @since 13.12.2018
 *
 */
public class FilterRowConfiguration extends AbstractRegistryConfiguration {

    DistributionEditor editor;

        /**
     * @param distributionEditor
     */
    public FilterRowConfiguration(DistributionEditor distributionEditor) {
        editor = distributionEditor;
    }

        @Override
        public void configureRegistry(IConfigRegistry configRegistry) {

            // register the FilterRowTextCellEditor in the first column which
            // immediately commits on key press
            configRegistry.registerConfigAttribute(
                    EditConfigAttributes.CELL_EDITOR,
                    new FilterRowTextCellEditor(),
                    DisplayMode.NORMAL,
                    FilterRowDataLayer.FILTER_ROW_COLUMN_LABEL_PREFIX + 1);

            configRegistry.registerConfigAttribute(
                    FilterRowConfigAttributes.TEXT_MATCHING_MODE,
                    TextMatchingMode.CONTAINS,
                    DisplayMode.NORMAL,
                    FilterRowDataLayer.FILTER_ROW_COLUMN_LABEL_PREFIX + 1);

            configRegistry.registerConfigAttribute(
                    CellConfigAttributes.DISPLAY_CONVERTER,
                    new SupplementalInfoDisplayConverter(),
                    DisplayMode.NORMAL,
                    FilterRowDataLayer.FILTER_ROW_COLUMN_LABEL_PREFIX + 1);

            if (PreferencesUtil.isShowRankInChecklistEditor()){
                configRegistry.registerConfigAttribute(
                        EditConfigAttributes.CELL_EDITOR,
                        new FilterRowTextCellEditor(),
                        DisplayMode.NORMAL,
                        FilterRowDataLayer.FILTER_ROW_COLUMN_LABEL_PREFIX + 2);

                configRegistry.registerConfigAttribute(
                        FilterRowConfigAttributes.TEXT_MATCHING_MODE,
                        TextMatchingMode.CONTAINS,
                        DisplayMode.NORMAL,
                        FilterRowDataLayer.FILTER_ROW_COLUMN_LABEL_PREFIX + 2);

                configRegistry.registerConfigAttribute(
                        CellConfigAttributes.DISPLAY_CONVERTER,
                        new SupplementalInfoDisplayConverter(),
                        DisplayMode.NORMAL,
                        FilterRowDataLayer.FILTER_ROW_COLUMN_LABEL_PREFIX + 2);
            }

            // Register a combo box editor to be displayed in the filter row
            // cell when a value is selected from the combo, the object is
            // converted to a string using the converter (registered below)
            StatusComboBoxDataProvider dataProvider;
            int start = 1;
            if (PreferencesUtil.isShowRankInChecklistEditor()){
                start = 2;
            }

            for (int i = start; i < editor.areas.size()+start; ++i) {
                dataProvider = new StatusComboBoxDataProvider(editor);
                ComboBoxCellEditor cellEditor = new ComboBoxCellEditor(dataProvider, dataProvider.getMaxVisibleItems());
                cellEditor.setFreeEdit(true);
                configRegistry.registerConfigAttribute(
                        EditConfigAttributes.CELL_EDITOR,
                        cellEditor,
                        DisplayMode.NORMAL,
                        FilterRowDataLayer.FILTER_ROW_COLUMN_LABEL_PREFIX + i);



            configRegistry.registerConfigAttribute(
                    FilterRowConfigAttributes.FILTER_DISPLAY_CONVERTER,
                    new FilterStatusDisplayConverter(),
                    DisplayMode.NORMAL,
                    FilterRowDataLayer.FILTER_ROW_COLUMN_LABEL_PREFIX+i);

            configRegistry.registerConfigAttribute(
                    CellConfigAttributes.DISPLAY_CONVERTER,
                    new StatusDisplayConverter(),
                    DisplayMode.NORMAL,
                    DistributionEditorConfigLabelAccumulator.STATUS);

            }


//            // register the FilterRowTextCellEditor in the first column which
//            // immediately commits on key press
//            configRegistry.registerConfigAttribute(
//                    EditConfigAttributes.CELL_EDITOR,
//                    new FilterRowTextCellEditor(),
//                    DisplayMode.NORMAL,
//                    FilterRowDataLayer.FILTER_ROW_COLUMN_LABEL_PREFIX
//                            + DistributionEditorConfigLabelAccumulator.STATUS);
//
//            configRegistry.registerConfigAttribute(
//                    FilterRowConfigAttributes.TEXT_MATCHING_MODE,
//                    TextMatchingMode.CONTAINS,
//                    DisplayMode.NORMAL,
//                    FilterRowDataLayer.FILTER_ROW_COLUMN_LABEL_PREFIX
//                    + DistributionEditorConfigLabelAccumulator.STATUS);
//
//            configRegistry.registerConfigAttribute(
//                    CellConfigAttributes.DISPLAY_CONVERTER,
//                    new StatusDisplayConverter(),
//                    DisplayMode.NORMAL,
//                    FilterRowDataLayer.FILTER_ROW_COLUMN_LABEL_PREFIX
//                    + DistributionEditorConfigLabelAccumulator.STATUS);





            configRegistry.registerConfigAttribute(
                    FilterRowConfigAttributes.TEXT_DELIMITER, "&"); //$NON-NLS-1$

        }

}
