/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.checklist.e4;

import org.apache.commons.lang.StringUtils;
import org.eclipse.nebula.widgets.nattable.data.convert.DisplayConverter;

import eu.etaxonomy.cdm.model.term.Representation;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTerm;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.TermDisplayEnum;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 05.12.2018
 *
 */
public class FilterStatusDisplayConverter extends DisplayConverter{
    public static final String DEFAULT_ENTRY = " ";
    /**
     * {@inheritDoc}
     */
    @Override
    public Object canonicalToDisplayValue(Object canonicalValue) {
        if(canonicalValue instanceof Distribution){
            Distribution distribution = (Distribution)canonicalValue;
            if (distribution.getStatus() != null){

                Representation rep = distribution.getStatus().getPreferredRepresentation(CdmStore.getDefaultLanguage());
                String displayStatus = PreferencesUtil.getStringValue(PreferencePredicate.DisplayOfStatus.getKey());
                String label = rep.getLabel();
                if(PreferencesUtil.displayStatusInChecklistEditor().equals(TermDisplayEnum.IdInVocabulary.getKey())){
                    return (StringUtils.isNotBlank(distribution.getStatus().getIdInVocabulary()))?distribution.getStatus().getIdInVocabulary():label;
                }else if(PreferencesUtil.displayStatusInChecklistEditor().equals(TermDisplayEnum.Symbol1.getKey())){

                    return (StringUtils.isNotBlank(distribution.getStatus().getSymbol() ))?distribution.getStatus().getSymbol():label;
                }else if(PreferencesUtil.displayStatusInChecklistEditor().equals(TermDisplayEnum.Symbol2.getKey())){
                    return (StringUtils.isNotBlank(distribution.getStatus().getSymbol2() ))?distribution.getStatus().getSymbol2():label;
                }else{
                    return (label != null)?label:DEFAULT_ENTRY;
                }



            }

            return "";
        }else if(canonicalValue instanceof PresenceAbsenceTerm){
            PresenceAbsenceTerm status = (PresenceAbsenceTerm)canonicalValue;
            Representation rep = status.getPreferredRepresentation(CdmStore.getDefaultLanguage());
            String displayStatus = PreferencesUtil.getStringValue(PreferencePredicate.DisplayOfStatus.getKey());
            String label = rep.getLabel();
            if (status.getId() == 0){
                return DEFAULT_ENTRY;
            }
            if(PreferencesUtil.displayStatusInChecklistEditor().equals(TermDisplayEnum.IdInVocabulary.getKey())){
                return (StringUtils.isNotBlank(status.getIdInVocabulary()))?status.getIdInVocabulary():label;
            }else if(PreferencesUtil.displayStatusInChecklistEditor().equals(TermDisplayEnum.Symbol1.getKey())){

                return (StringUtils.isNotBlank(status.getSymbol() ))?status.getSymbol():label;
            }else if(PreferencesUtil.displayStatusInChecklistEditor().equals(TermDisplayEnum.Symbol2.getKey())){
                return (StringUtils.isNotBlank(status.getSymbol2() ))?status.getSymbol2():label;
            }else{
                return (label != null)?label:DEFAULT_ENTRY;
            }


        } else if(canonicalValue!=null){
            return canonicalValue.toString();
        }
        return DEFAULT_ENTRY;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Object displayToCanonicalValue(Object displayValue) {
        return null;
    }

}
