/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.checklist.e4;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.UUID;

import org.apache.commons.lang.StringUtils;
import org.eclipse.nebula.widgets.nattable.edit.editor.IComboBoxDataProvider;

import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.OrderedTermComparator;
import eu.etaxonomy.cdm.model.term.Representation;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTerm;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 05.12.2018
 *
 */
public class StatusComboBoxDataProvider implements IComboBoxDataProvider {

    private DistributionEditor editor;
    private int maxVisibleItems;

    public StatusComboBoxDataProvider(DistributionEditor editor) {
        super();
        this.editor = editor;

    }

    @Override
    public List<?> getValues(int columnIndex, int rowIndex) {
        return getValues();
    }

    public static List<?> getValues(){
        List<DefinedTermBase> inputAll = new ArrayList<>();
        PresenceAbsenceTerm noneTerm = PresenceAbsenceTerm.NewInstance();
        noneTerm.setTitleCache(" - ", true);
        noneTerm.addRepresentation(Representation.NewInstance(" - ", " ", " ", CdmStore.getDefaultLanguage()));
        inputAll.add(noneTerm);
        String statusString = PreferencesUtil.getStringValue(PreferencePredicate.AvailableDistributionStatus.getKey());
        List<PresenceAbsenceTerm> inputList;
        if (!StringUtils.isBlank(statusString)){

            String [] statusArray = statusString.split(";");
            List<UUID> uuidList = new ArrayList();
            for (String status: statusArray){
                uuidList.add(UUID.fromString(status));
            }

            inputList = CdmStore.getTermManager().getTerms(uuidList, PresenceAbsenceTerm.class);
        }else{

            inputList = CdmStore.getTermManager().getPreferredTerms(PresenceAbsenceTerm.class);
        }
        Comparator<DefinedTermBase> comp = new OrderedTermComparator<>();
        Collections.sort(inputList, comp);
        Collections.sort(inputList, Collections.reverseOrder());

        inputAll.addAll(1, inputList);
        return inputAll;
    }

    public int getMaxVisibleItems(){
        return maxVisibleItems;
    }

}
