/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.checklist.e4;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.eclipse.nebula.widgets.nattable.edit.editor.ComboBoxCellEditor;
import org.eclipse.nebula.widgets.nattable.widget.NatCombo;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.api.service.dto.TaxonDistributionDTO;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.location.NamedArea;

/**
 * @author k.luther
 * @since 05.12.2018
 *
 */
public class StatusDataCellEditor extends ComboBoxCellEditor {
    private DistributionEditor editor;



    public StatusDataCellEditor(DistributionEditor editor, StatusComboBoxDataProvider statusComboBoxDataProvider) {
        super(statusComboBoxDataProvider, statusComboBoxDataProvider.getMaxVisibleItems());
        this.editor = editor;
        setUseCheckbox(false);
        setMultiselect(false);
        maxVisibleItems = 10;

    }

    @Override
    public void setCanonicalValue(Object canonicalValue) {
        if(canonicalValue instanceof Distribution){
            Object rowWrapper = editor.getBodyDataProvider().getRowObject(this.getRowIndex());
            if(rowWrapper instanceof TaxonDistributionDTO){
                NamedArea area = editor.getAreaToColumnIndexMap().get(getColumnIndex());
                Map<NamedArea, Set<DescriptionElementBase>> distributionMap = editor.taxonDistributionMap.get(((TaxonDistributionDTO) rowWrapper).getTaxonUuid());
                if (distributionMap == null){
                    distributionMap = new HashMap();
                    editor.taxonDistributionMap.put(((TaxonDistributionDTO) rowWrapper).getTaxonUuid(),distributionMap);
                }
                Set<DescriptionElementBase> elements = distributionMap.get(area);
                if (elements == null){

                }else if (elements.size() == 1){
                    DescriptionElementBase base = elements.iterator().next();
                    Distribution distr = (Distribution) base;
                    //editor.descriptionsToSave.add( (TaxonDistributionDTO) rowWrapper);
                    //TODO set value corresponding to preference
                    if (distr.getStatus() != null){
                        if (distr.getStatus().getLabel() == " - "){
                            distr.setStatus(null);
                            super.setCanonicalValue("");

                        }else{
                            super.setCanonicalValue(distr.getStatus().getLabel());
                        }
                    }
                }
            }
        }else if (canonicalValue != null){
            super.setCanonicalValue(canonicalValue);
        }
    }

    @Override
    protected Control activateCell(Composite parent, Object originalCanonicalValue) {
            if (originalCanonicalValue instanceof Distribution || originalCanonicalValue == null){
                NatCombo natCombo = (NatCombo) super.activateCell(parent, originalCanonicalValue);
                natCombo.addSelectionListener(new SelectionListener() {
                    @Override
                    public void widgetSelected(SelectionEvent e) {
                        editor.setDirty();
                    }
                    @Override
                    public void widgetDefaultSelected(SelectionEvent e) {
                    }
                });
                return natCombo;
            }else{
                return null;
            }


    }
}
