/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.concept;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.Viewer;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;

/**
 * @author n.hoffmann
 * @created Jan 24, 2011
 * @version 1.0
 */
public class ConceptContentProvider implements IStructuredContentProvider {

	@Override
	public void dispose() {

	}

	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
	}

	@Override
	public Object[] getElements(Object inputElement) {
		Taxon taxon = null;
		if(inputElement instanceof Taxon){
			taxon = (Taxon) inputElement;
		}
		if(taxon!=null){
			Map<TaxonRelationship, Taxon> taxonToTaxonRelationsMap = new HashMap<>();

			for (TaxonRelationship relationship : taxon.getTaxonRelations()) {
				if (! relationship.getType().equals(TaxonRelationshipType.MISAPPLIED_NAME_FOR()) ||
						 relationship.getType().equals(TaxonRelationshipType.TAXONOMICALLY_INCLUDED_IN())) {
					taxonToTaxonRelationsMap.put(relationship, taxon);
				}
			}
			return taxonToTaxonRelationsMap.entrySet().toArray();
		}
		return new Object[0];
	}

}
