/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.concept.e4;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;

import javax.annotation.PostConstruct;

import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.services.EMenuService;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.taxeditor.editor.key.e4.AbstractGraphKeyEditorE4;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.view.concept.ConceptContentProvider;
import eu.etaxonomy.taxeditor.editor.view.concept.ConceptLabelProvider;
import eu.etaxonomy.taxeditor.editor.view.descriptive.e4.FactualDataPartE4;
import eu.etaxonomy.taxeditor.editor.view.media.e4.MediaViewPartE4;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;
import eu.etaxonomy.taxeditor.view.e4.AbstractCdmEditorPartE4;
import eu.etaxonomy.taxeditor.view.e4.details.DetailsPartE4;
import eu.etaxonomy.taxeditor.view.e4.supplementaldata.SupplementalDataPartE4;

/**
 *
 * @author pplitzner
 * @date 21.08.2017
 *
 */
public class ConceptViewPartE4 extends AbstractCdmEditorPartE4
implements IPartContentHasDetails, IPartContentHasSupplementalData{

    @Override
    public void selectionChanged_internal(Object selection, MPart activePart, MPart thisPart) {
        if(activePart==thisPart){
            return;
        }
        if (!viewer.getControl().isDisposed()){
            viewer.getControl().setEnabled(true);
        }
        Object partObject = createPartObject(activePart);
        IStructuredSelection structuredSelection = createSelection(selection);

        if (partObject instanceof DetailsPartE4 || partObject instanceof SupplementalDataPartE4
                || partObject instanceof FactualDataPartE4|| partObject instanceof MediaViewPartE4) {
            // do not show empty page as these views are also used to edit the
            // description selected in this view
            return;
        }
		else if(partObject instanceof AbstractGraphKeyEditorE4){
			showEmptyPage();
		}
		else if(selection!=null){
		    showViewer(structuredSelection, activePart, viewer);
		}
		else {
		    showEmptyPage();
		}

	}

	@PostConstruct
    public void create(Composite parent, EMenuService menuService, MPart thisPart) {

	    this.thisPart = thisPart;

	    ConceptRelationViewer conceptViewer = new ConceptRelationViewer(parent);

	    conceptViewer.setContentProvider(new ConceptContentProvider());
	    conceptViewer.setLabelProvider(new ConceptLabelProvider());


	    conceptViewer.setSorter(new ViewerSorter());

		viewer = conceptViewer;

		// Propagate selection from viewer
        selectionChangedListener = (event -> selService.setSelection(viewer.getSelection()));
        viewer.addSelectionChangedListener(selectionChangedListener);

        //create context menu
        menuService.registerContextMenu(viewer.getControl(), "eu.etaxonomy.taxeditor.editor.popupmenu.conceptview");

	}

	@Override
	public void changed(Object object) {
		getViewer().refresh();
		super.changed(object);
	}

	private class ConceptRelationViewer extends ListViewer {

		public ConceptRelationViewer(Composite parent) {
			super(parent);
		}

		@Override
		public ISelection getSelection() {
			ISelection selection = super.getSelection();
			List<TaxonRelationship> relationships = new ArrayList<>();
			if(selection instanceof IStructuredSelection && !selection.isEmpty()){
			    for(Iterator iter = ((IStructuredSelection) selection).iterator();iter.hasNext();){
			        Object object = iter.next();
			        relationships.add(((Entry<TaxonRelationship, Taxon>)object).getKey());
			    }
			    return new StructuredSelection(relationships);
			}
			return selection;
		}

		@Override
		public void setSelection(ISelection selection) {
			super.setSelection(selection);
		}

	}

	@Override
	protected String getViewName() {
		return Messages.ConceptViewPart_VIEWER_NAME;
	}

}
