/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.concept.e4;

import java.util.List;

import org.eclipse.e4.ui.di.AboutToShow;
import org.eclipse.e4.ui.model.application.commands.MCommand;
import org.eclipse.e4.ui.model.application.commands.MCommandsFactory;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.model.application.ui.menu.MMenu;
import org.eclipse.e4.ui.model.application.ui.menu.MMenuElement;
import org.eclipse.e4.ui.model.application.ui.menu.MMenuFactory;

import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 *
 * @author pplitzner
 * @date 21.08.2017
 *
 */
public class CreateConceptRelationMenuE4 {

    public String commandName = "eu.etaxonomy.taxeditor.editor.name.createConceptRelation"; //$NON-NLS-1$

    /** {@inheritDoc} */
    @AboutToShow
    public void aboutToShow(List<MMenuElement> items) {

        MMenu menu = MMenuFactory.INSTANCE.createMenu();
        menu.setLabel("New");
        items.add(menu);

        for (final TaxonRelationshipType taxonRelationshipType : CdmStore.getTermManager().getPreferredTerms(TaxonRelationshipType.class)) {
            if (!taxonRelationshipType.equals(TaxonRelationshipType.TAXONOMICALLY_INCLUDED_IN())) {
                MHandledMenuItem menuItem = MMenuFactory.INSTANCE.createHandledMenuItem();
                menuItem.setLabel(taxonRelationshipType.getTitleCache());
                MCommand mCommand = MCommandsFactory.INSTANCE.createCommand();
                mCommand.setElementId(commandName);
                mCommand.setCommandName("Create concept relation");
                // set params
                menuItem.getTransientData().put(commandName + ".taxonRelationshipType.uuid",
                        taxonRelationshipType.getUuid());

                menuItem.setCommand(mCommand);
                menu.getChildren().add(menuItem);
            }
        }
    }

}
