/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.view.concept.e4.handler;

import java.util.UUID;

import javax.inject.Named;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.view.concept.e4.ConceptViewPartE4;
import eu.etaxonomy.taxeditor.editor.view.concept.operation.CreateConceptRelationOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.TaxonRelationshipTypeInverseContainer;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.dialog.selection.TaxonSelectionDialog;
import eu.etaxonomy.taxeditor.workbench.WorkbenchUtility;

/**
 *
 * @author pplitzner
 * @date 21.08.2017
 *
 */
public class CreateConceptRelationHandlerE4 {

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART) MPart activePart,
            @Optional@Named(IServiceConstants.ACTIVE_SHELL) Shell shell,
            MHandledMenuItem menuItem,
            UISynchronize sync) {

        String commandId = menuItem.getCommand().getElementId();
        UUID uuid = (UUID) menuItem.getTransientData().get(commandId+".taxonRelationshipType.uuid");
        TaxonRelationshipType taxonRelationshipType = HibernateProxyHelper.deproxy(CdmStore.getService(ITermService.class).load(uuid), TaxonRelationshipType.class);

        TaxonNameEditorE4 editor = null;

        ConceptViewPartE4 conceptView = (ConceptViewPartE4) activePart.getObject();

        Object e4WrappedPart = WorkbenchUtility.getE4WrappedPart(conceptView.getSelectionProvidingPart());
        if(e4WrappedPart instanceof TaxonNameEditorE4){
            editor = (TaxonNameEditorE4) e4WrappedPart;

            IUndoContext undoContext = null;
            ConversationHolder conversationHolder = null;
            Taxon taxon = null;
            taxon = editor.getTaxon();
            conversationHolder = editor.getConversationHolder();
            undoContext = editor.getUndoContext();

            //FIXME should this also be enabled for bulk editor
            //		else if(activeEditor instanceof BulkEditor){
            //		    BulkEditor bulkEditor = (BulkEditor)activeEditor;
            //            ISelection selection = bulkEditor.getSelectionProvider().getSelection();
            //		    taxon = (Taxon) ((IStructuredSelection)selection).getFirstElement();
            //		    conversationHolder = bulkEditor.getConversationHolder();
            //		    undoContext = bulkEditor.getUndoContext();
            //		}

            Taxon relatedConcept = TaxonSelectionDialog.selectTaxon(shell, //conversationHolder,
                    taxon);


            if(relatedConcept == null){
                return;
            }
            boolean isInverse = taxonRelationshipType.getInverseLabel().isEmpty()?false:true;
            TaxonRelationshipTypeInverseContainer typeInverseContainer =new TaxonRelationshipTypeInverseContainer(taxonRelationshipType, isInverse) ;


            AbstractPostOperation operation = new CreateConceptRelationOperation(menuItem.getLocalizedLabel(),
                    undoContext, taxon, relatedConcept, typeInverseContainer, conceptView);
            AbstractUtility.executeOperation(operation, sync);
        }
    }

    @CanExecute
    public boolean canExecute(@Optional @Named(IServiceConstants.ACTIVE_PART) MPart activePart,
            MHandledMenuItem menuItem){
        boolean canExecute = false;
        ConceptViewPartE4 conceptView = (ConceptViewPartE4) activePart.getObject();

        Object e4WrappedPart = WorkbenchUtility.getE4WrappedPart(conceptView.getSelectionProvidingPart());
        canExecute = e4WrappedPart instanceof TaxonNameEditorE4;
        menuItem.setVisible(canExecute);
        return canExecute;
    }
}
