/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.concept.e4.handler;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.model.application.ui.menu.MMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;

import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.view.concept.e4.ConceptViewPartE4;
import eu.etaxonomy.taxeditor.editor.view.concept.operation.DeleteConceptRelationOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.workbench.WorkbenchUtility;

/**
 *
 * @author pplitzner
 * @date 21.08.2017
 *
 */
public class DeleteConceptRelationHandlerE4 {

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART) MPart activePart,
            @Optional@Named(IServiceConstants.ACTIVE_SELECTION) IStructuredSelection selection,
            MMenuItem menuItem,
            UISynchronize sync) {

        TaxonNameEditorE4 editor = null;

        ConceptViewPartE4 conceptView = (ConceptViewPartE4) activePart.getObject();

        Object e4WrappedPart = WorkbenchUtility.getE4WrappedPart(conceptView.getSelectionProvidingPart());
        if(e4WrappedPart instanceof TaxonNameEditorE4){
            editor = (TaxonNameEditorE4) e4WrappedPart;

            Set<TaxonRelationship> relations = new HashSet<TaxonRelationship>();


            for (Iterator iterator = selection.iterator(); iterator.hasNext();){
                Object object = iterator.next();
                if(object instanceof TaxonRelationship){
                    relations.add((TaxonRelationship) object);
                }
            }

            //editor should not be null because this is already checked in canExecute
            AbstractPostOperation operation = new DeleteConceptRelationOperation(menuItem.getLocalizedLabel(),
                    editor.getUndoContext(), editor.getTaxon(), relations, conceptView);
            AbstractUtility.executeOperation(operation, sync);
        }
    }

	@CanExecute
    public boolean canExecute(@Named(IServiceConstants.ACTIVE_SELECTION) IStructuredSelection selection,
            MHandledMenuItem menuItem) {
        boolean canExecute = false;
        canExecute = selection != null && !selection.isEmpty();
        menuItem.setVisible(canExecute);
        return canExecute;
    }
}
