/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.concept.e4.handler;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.viewers.IStructuredSelection;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.taxeditor.bulkeditor.e4.BulkEditorE4;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.view.concept.e4.ConceptViewPartE4;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.workbench.WorkbenchUtility;

/**
 *
 * @author pplitzner
 * @date 21.08.2017
 *
 */
public class OpenRelatedConceptHandlerE4  {

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART) MPart activePart,
            @Optional@Named(IServiceConstants.ACTIVE_SELECTION) IStructuredSelection selection,
            EModelService modelService, EPartService partService, MApplication application) {

        TaxonNameEditorE4 editor = null;
        BulkEditorE4 bulkEditor = null;
        TaxonRelationship taxonRelationship = (TaxonRelationship) selection.getFirstElement();

        ConceptViewPartE4 conceptView = (ConceptViewPartE4) activePart.getObject();

        Object e4WrappedPart = WorkbenchUtility.getE4WrappedPart(conceptView.getSelectionProvidingPart());
        if(e4WrappedPart instanceof TaxonNameEditorE4){
            editor = (TaxonNameEditorE4) e4WrappedPart;
        }
        else if(e4WrappedPart instanceof BulkEditorE4){
            bulkEditor = (BulkEditorE4) e4WrappedPart;
        }
        TaxonBase<?> relatedTaxon = null;
        if(editor!=null){
            relatedTaxon = getRelatedTaxon(taxonRelationship, editor.getTaxon());
        }
        else if(bulkEditor!=null){
            IStructuredSelection bulkSelection = bulkEditor.getSelection();
            Object firstElement = bulkSelection.getFirstElement();
            if(firstElement instanceof Taxon){
                relatedTaxon = getRelatedTaxon(taxonRelationship, (Taxon) firstElement);
            }
        }

		if(relatedTaxon==null){
		    MessagingUtils.messageDialog(Messages.OpenRelatedConceptHandler_COULD_NOT_OPEN, getClass(),
		            String.format(Messages.OpenRelatedConceptHandler_COULD_NOT_OPEN_MESSAGE ,relatedTaxon), null);
		}
		else{
		    openConcept(relatedTaxon, modelService, partService, application);
		}
	}

    protected void openConcept(TaxonBase<?> relatedTaxon, EModelService modelService, EPartService partService, MApplication application) {
        EditorUtil.openTaxonBaseE4(relatedTaxon.getUuid(), modelService, partService, application);
    }

	private Taxon getRelatedTaxon(TaxonRelationship relationship, Taxon taxon){
		if (relationship.getFromTaxon().equals(taxon)){
			return relationship.getToTaxon();
		}
		else if(relationship.getToTaxon().equals(taxon)){
			return relationship.getFromTaxon();
		}
		else{
			return null;
		}
	}

    @CanExecute
    public boolean canExecute(@Named(IServiceConstants.ACTIVE_SELECTION) IStructuredSelection selection,
            MHandledMenuItem menuItem) {
        boolean canExecute = false;
        canExecute = selection != null
                && selection.size()==1
                && selection.getFirstElement() instanceof TaxonRelationship;
        menuItem.setVisible(canExecute);
        return canExecute;
    }
}
