/**
 *
 */
package eu.etaxonomy.taxeditor.editor.view.concept.graph.e4;

import javax.annotation.PostConstruct;

import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.services.EMenuService;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.zest.core.viewers.GraphViewer;
import org.eclipse.zest.core.widgets.ZestStyles;
import org.eclipse.zest.layouts.LayoutAlgorithm;
import org.eclipse.zest.layouts.LayoutStyles;
import org.eclipse.zest.layouts.algorithms.TreeLayoutAlgorithm;

import eu.etaxonomy.taxeditor.bulkeditor.e4.BulkEditorE4;
import eu.etaxonomy.taxeditor.editor.e4.TaxonEditorInputE4;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.view.concept.graph.ConceptGraphContentProvider;
import eu.etaxonomy.taxeditor.editor.view.concept.graph.ConceptGraphLabelProvider;
import eu.etaxonomy.taxeditor.editor.view.descriptive.e4.FactualDataPartE4;
import eu.etaxonomy.taxeditor.editor.view.media.e4.MediaViewPartE4;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.view.e4.AbstractCdmEditorPartE4;
import eu.etaxonomy.taxeditor.view.e4.details.DetailsPartE4;
import eu.etaxonomy.taxeditor.view.e4.supplementaldata.SupplementalDataPartE4;

/**
 *
 * @author pplitzner
 * @since Sep 27, 2017
 *
 */
public class ConceptGraphViewE4 extends AbstractCdmEditorPartE4 {

	public static final String ID = "eu.etaxonomy.taxeditor.editor.view.concept.graph"; //$NON-NLS-1$

	private LayoutAlgorithm layoutAlgoritm;

    /**
     * {@inheritDoc}
     */
	@Override
	public void selectionChanged_internal(Object selection, MPart activePart, MPart thisPart) {
	    if(activePart==thisPart){
	        return;
	    }

	    Object partObject = createPartObject(activePart);

	    if (partObject instanceof DetailsPartE4 || partObject instanceof SupplementalDataPartE4
	            || partObject instanceof FactualDataPartE4|| partObject instanceof MediaViewPartE4) {
	        // do not show empty page as these views are also used to edit the
	        // description selected in this view
	        return;
	    }
		if(AbstractUtility.getActiveEditor() == null){
			showEmptyPage();
			return;
		}

		if(partObject instanceof BulkEditorE4){
			showEmptyPage();
			return;
		}


		if(partObject instanceof TaxonNameEditorE4){
		    if (viewer.getControl() != null && !viewer.getControl().isDisposed()) {
                viewer.getControl().setEnabled(true);
            }
		    TaxonEditorInputE4 input = ((TaxonNameEditorE4) partObject).getEditorInput();
		    showViewer(new StructuredSelection(input), activePart, viewer);
		}

	}

	@Override
	public boolean onComplete() {
		return false;
	}

	@Override
	public Viewer getViewer() {
		return viewer;
	}

	@PostConstruct
    public void create(Composite parent, EMenuService menuService, MPart thisPart) {

        this.thisPart = thisPart;

        GraphViewer viewer = new GraphViewer(parent, SWT.NONE);

		viewer.setContentProvider(new ConceptGraphContentProvider());
		viewer.setLabelProvider(new ConceptGraphLabelProvider());

		viewer.setConnectionStyle(ZestStyles.CONNECTIONS_DIRECTED);

		viewer.setLayoutAlgorithm(getLayoutAlgoritm(), false);

		this.viewer = viewer;

		// Propagate selection from viewer
        selectionChangedListener = (event -> selService.setSelection(AbstractUtility.getElementsFromSelectionChangedEvent(event)));
        viewer.addSelectionChangedListener(selectionChangedListener);

//		createMenu();
//
//		createToolbar();


	}

	private LayoutAlgorithm getLayoutAlgoritm() {

		if (layoutAlgoritm == null) {
			// layoutAlgoritm = new CompositeLayoutAlgorithm(
			// LayoutStyles.NO_LAYOUT_NODE_RESIZING,
			// new LayoutAlgorithm[] {
			// new TreeLayoutAlgorithm(
			// LayoutStyles.NO_LAYOUT_NODE_RESIZING),
			// new HorizontalShift(
			// LayoutStyles.NO_LAYOUT_NODE_RESIZING) });

			layoutAlgoritm = new TreeLayoutAlgorithm(
					LayoutStyles.NO_LAYOUT_NODE_RESIZING);
			layoutAlgoritm.setEntityAspectRatio(0.5);
		}
		return layoutAlgoritm;
	}

	@Override
	protected String getViewName() {
		return Messages.ConceptGraphView_VIEWER_NAME;
	}

}
