/**
 * Copyright (C) 2014 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.view.derivate;

import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;

import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.model.MessagingUtils;

/**
 * @author pplitzner
 * @date 18.03.2014
 *
 */
public class DerivateDragListener extends DragSourceAdapter{

    private final DerivateView derivateView;

    /**
     * @param derivateView
     */
    public DerivateDragListener(DerivateView derivateView) {
        this.derivateView = derivateView;
    }

    @Override
    public void dragSetData(DragSourceEvent event) {
        if (LocalSelectionTransfer.getTransfer().isSupportedType(event.dataType)) {
            DerivateViewSelection derivateViewSelection = new DerivateViewSelection(derivateView.getViewer().getSelection(), derivateView);
            LocalSelectionTransfer.getTransfer().setSelection(derivateViewSelection);
        }
    }

    @Override
    public void dragFinished(DragSourceEvent event) {
        super.dragFinished(event);
        LocalSelectionTransfer.getTransfer().setSelection(null);
    }

    @Override
    public void dragStart(DragSourceEvent event) {
        if(derivateView.isDirty()){
            event.doit = false;
            MessagingUtils.warningDialog(DerivateView.VIEW_HAS_UNSAVED_CHANGES, this, DerivateView.YOU_NEED_TO_SAVE_BEFORE_PERFORMING_THIS_ACTION);
            return;
        }
        ISelection selection = derivateView.getViewer().getSelection();
        if(EditorUtil.getTreeNodeOfSelection(selection)!=null){
            event.doit = true;
        }
        else{
            event.doit = false;
        }
    }
}
