/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.derivate;

import java.util.Collection;
import java.util.Collections;
import java.util.UUID;

import eu.etaxonomy.taxeditor.editor.l10n.Messages;

/**
 * Editor input for the {@link DerivateView}
 *
 * @author pplitzner
 * @date Nov 11, 2015
 *
 */
public class DerivateViewEditorInput {

    private Collection<UUID> derivativeUuids;
    private UUID taxonUuid;

    public DerivateViewEditorInput() {
    }

    public DerivateViewEditorInput(UUID derivativeUuid, UUID taxonUuid) {
    	this(Collections.singleton(derivativeUuid), taxonUuid);
    }

    public DerivateViewEditorInput(Collection<UUID> derivativeUuids, UUID taxonUuid) {
        this.derivativeUuids = derivativeUuids;
        this.taxonUuid = taxonUuid;
    }

    public DerivateViewEditorInput(UUID derivativeUuid) {
    	this(derivativeUuid, null);
    }

    public DerivateViewEditorInput(Collection<UUID> derivativeUuids) {
        this(derivativeUuids, null);
    }

    public Collection<UUID> getDerivativeUuids() {
        return derivativeUuids;
    }

    public UUID getTaxonUuid() {
        return taxonUuid;
    }

    private String getEditorName() {
        return Messages.DerivateViewEditorInput_SPECIMEN_EDITOR;
    }

}
