package eu.etaxonomy.taxeditor.editor.view.derivate.handler;

import java.util.Collections;
import java.util.UUID;

import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.IDescriptionService;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.IndividualsAssociation;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.editor.AppModelId;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateViewEditorInput;
import eu.etaxonomy.taxeditor.handler.defaultHandler.e4.DefaultOpenHandlerBaseE4;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

public class OpenDerivativeEditorForDescriptionElement extends DefaultOpenHandlerBaseE4<IndividualsAssociation> {

    @Override
    protected IndividualsAssociation getEntity(UUID uuid) {
        DescriptionElementBase descriptionElement = CdmStore.getService(IDescriptionService.class).loadDescriptionElement(uuid, null);
        if(descriptionElement!=null && descriptionElement.isInstanceOf(IndividualsAssociation.class)){
            return (IndividualsAssociation) descriptionElement;
        }
        return null;
    }

    @Override
    protected void open(IndividualsAssociation entity, Shell shell, EPartService partService) {
    	if(entity==null || entity.getAssociatedSpecimenOrObservation()==null){
    		MessagingUtils.messageDialog(Messages.OpenDerivativeEditorForDescriptionElement_NO_SPECIMENS, this, Messages.OpenDerivativeEditorForDescriptionElement_NO_SPECIMENS_MESSAGE);
    		return;
    	}
        SpecimenOrObservationBase associatedSpecimenOrObservation = entity.getAssociatedSpecimenOrObservation();
        if(associatedSpecimenOrObservation!=null){
            UUID specimenUuid = associatedSpecimenOrObservation.getUuid();
            EditorUtil.openSpecimenEditor(new DerivateViewEditorInput(Collections.singletonList(specimenUuid)), modelService, partService, application);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected boolean canExecute(IndividualsAssociation entity) {
        return true;
    }

    @Override
    protected String getPartId() {
        return AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_EDITOR_VIEW_DERIVATE_DERIVATEVIEW;
    }
}
