package eu.etaxonomy.taxeditor.editor.view.derivate.handler;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.UUID;

import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.IEventBaseService;
import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.cdm.model.occurrence.GatheringEvent;
import eu.etaxonomy.taxeditor.editor.AppModelId;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateViewEditorInput;
import eu.etaxonomy.taxeditor.handler.defaultHandler.e4.DefaultOpenHandlerBaseE4;
import eu.etaxonomy.taxeditor.store.CdmStore;

public class OpenDerivativeEditorForGatheringEvent extends DefaultOpenHandlerBaseE4<GatheringEvent> {

    protected static final String OPEN_DERIVATIVE_EDITOR_FOR_TAXON_NODE_COULD_NOT_OPEN = Messages.OpenDerivativeEditorForTaxonNode_COULD_NOT_OPEN;

    @Override
    protected GatheringEvent getEntity(UUID uuid) {
        ICdmBase load = CdmStore.getService(IEventBaseService.class).load(uuid);
        return HibernateProxyHelper.deproxy(load, GatheringEvent.class );
    }

    @Override
    protected void open(GatheringEvent entity, Shell shell, EPartService partService) {
        Collection<UUID> specimenUuids = new ArrayList<UUID>();
        List<FieldUnit> fieldUnits = CdmStore.getService(IOccurrenceService.class).findFieldUnitsForGatheringEvent(entity.getUuid());
        fieldUnits.forEach(fieldUnit->specimenUuids.add(fieldUnit.getUuid()));
        DerivateViewEditorInput input = new DerivateViewEditorInput(specimenUuids);
        EditorUtil.openSpecimenEditor(input, modelService, partService, application);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected boolean canExecute(GatheringEvent entity) {
        return true;
    }

    @Override
    protected String getPartId() {
        return AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_EDITOR_VIEW_DERIVATE_DERIVATEVIEW;
    }

}
