package eu.etaxonomy.taxeditor.editor.view.derivate.handler;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.UUID;

import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.api.service.ITaxonNodeService;
import eu.etaxonomy.cdm.api.service.config.FindOccurrencesConfigurator;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.AppModelId;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateViewEditorInput;
import eu.etaxonomy.taxeditor.handler.defaultHandler.e4.DefaultOpenHandlerBaseE4;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

public class OpenDerivativeEditorForTaxonNode extends DefaultOpenHandlerBaseE4<TaxonNode> {

    protected static final String OPEN_DERIVATIVE_EDITOR_FOR_TAXON_NODE_COULD_NOT_OPEN = Messages.OpenDerivativeEditorForTaxonNode_COULD_NOT_OPEN;

    @Override
    protected TaxonNode getEntity(UUID uuid) {
        return CdmStore.getService(ITaxonNodeService.class).load(uuid);
    }

    @Override
    protected void open(TaxonNode entity, Shell shell, EPartService partService) {
        FindOccurrencesConfigurator config = new FindOccurrencesConfigurator();
        UUID taxonUuid = entity.getTaxon().getUuid();
        config.setAssociatedTaxonUuid(taxonUuid);
        List<SpecimenOrObservationBase> specimens = CdmStore.getService(IOccurrenceService.class).findByTitle(config).getRecords();
        Collection<UUID> specimenUuids = new ArrayList<UUID>();
        for (SpecimenOrObservationBase specimenOrObservationBase : specimens) {
            specimenUuids.add(specimenOrObservationBase.getUuid());
        }
        DerivateViewEditorInput input = new DerivateViewEditorInput(specimenUuids, taxonUuid);
        EditorUtil.openSpecimenEditor(input, modelService, partService, application);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected boolean canExecute(TaxonNode entity) {
        return (entity.getTaxon()!=null && PreferencesUtil.getBooleanValue(PreferencePredicate.ShowSpecimen.getKey()));
    }

    @Override
    protected String getPartId() {
        return AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_EDITOR_VIEW_DERIVATE_DERIVATEVIEW;
    }

}
