package eu.etaxonomy.taxeditor.editor.view.derivate.handler;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeNode;

import eu.etaxonomy.cdm.api.service.molecular.ISequenceService;
import eu.etaxonomy.cdm.model.molecular.Sequence;
import eu.etaxonomy.cdm.model.molecular.SingleRead;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

public class SingleReadReuseHandler {

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART) MPart part,
            @Named(IServiceConstants.ACTIVE_SELECTION) IStructuredSelection selection){
        ISelection clipBoardSelection = LocalSelectionTransfer.getTransfer().getSelection();
        TreeNode clipBoardTreeNode = EditorUtil.getTreeNodeOfSelection(clipBoardSelection);
        DerivateView derivateView = (DerivateView)part.getObject();
        if(derivateView.isDirty()){
            MessagingUtils.warningDialog(DerivateView.VIEW_HAS_UNSAVED_CHANGES, this, DerivateView.YOU_NEED_TO_SAVE_BEFORE_PERFORMING_THIS_ACTION);
            return;
        }
        if(clipBoardTreeNode!=null && clipBoardTreeNode.getValue() instanceof SingleRead){
            SingleRead singleRead = (SingleRead)clipBoardTreeNode.getValue();
            Sequence sequence = (Sequence)((TreeNode) selection.getFirstElement()).getValue();
            sequence.addSingleRead(singleRead);
            CdmStore.getService(ISequenceService.class).merge(sequence);

            derivateView.getConversationHolder().commit();
            derivateView.refreshTree();
            LocalSelectionTransfer.getTransfer().setSelection(null);
        }
        derivateView.updateLabelCache();
        derivateView.refreshTree();
    }

    @CanExecute
    public boolean canExecute(@Optional @Named(IServiceConstants.ACTIVE_SELECTION) IStructuredSelection selection,
            MHandledMenuItem menuItem){
        boolean canExecute = false;
        canExecute = selection!=null
                && selection.size()==1
                && selection.getFirstElement() instanceof TreeNode
                && ((TreeNode) selection.getFirstElement()).getValue() instanceof Sequence;
        Sequence sequence = (Sequence) ((TreeNode) selection.getFirstElement()).getValue();
        TreeNode clipboardNode = EditorUtil.getTreeNodeOfSelection(LocalSelectionTransfer.getTransfer().getSelection());
        canExecute &= clipboardNode!=null
                && clipboardNode.getValue() instanceof SingleRead
                && !(sequence.getSingleReads().contains(clipboardNode.getValue()));
        menuItem.setVisible(canExecute);
        return canExecute;
    }

}
