/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.view.descriptive.e4.handler;

import java.util.UUID;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;

import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.view.descriptive.e4.FactualDataPartE4;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.CreateDescriptionElementOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 *
 * @author pplitzner
 * @date 15.08.2017
 *
 */
public class CreateDescriptionElementHandlerE4 {

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART) MPart activePart,
            @Optional@Named(IServiceConstants.ACTIVE_SELECTION) IStructuredSelection selection,
            MHandledMenuItem menuItem,
            UISynchronize sync) {

        String commandId = menuItem.getCommand().getElementId();
        UUID uuid = (UUID) menuItem.getTransientData().get(commandId+".feature.uuid");
        Feature feature = HibernateProxyHelper.deproxy(CdmStore.getService(ITermService.class).load(uuid), Feature.class);

        FactualDataPartE4 factualDataPart = (FactualDataPartE4) activePart.getObject();

        DescriptionBase description = null;
        Object object = selection.getFirstElement();
        if(object instanceof FeatureNodeContainer){
            description = ((FeatureNodeContainer) object).getDescription();
        }
        else if(object instanceof DescriptionElementBase){
            DescriptionElementBase elementBase = HibernateProxyHelper.deproxy(object, DescriptionElementBase.class);
            description = elementBase.getInDescription();
        }
        else if (object instanceof DescriptionBase) {
            description = HibernateProxyHelper.deproxy(object, DescriptionBase.class);
        } else {
            MessagingUtils.error(getClass(), new IllegalArgumentException("Could not determine the taxon description")); //$NON-NLS-1$
            return;
        }
        AbstractPostOperation operation = operationCreationInstance(menuItem.getLocalizedLabel(), feature, description,
                factualDataPart);
        // TODO use undo context specific to editor
        AbstractUtility.executeOperation(operation, sync);
    }

    @CanExecute
    public boolean canExecute(MHandledMenuItem menuItem,
            @Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection){
        boolean canExecute = false;
        canExecute = selection.size()==1
                &&
                (selection.getFirstElement() instanceof DescriptionBase
                || selection.getFirstElement() instanceof DescriptionElementBase
                || selection.getFirstElement() instanceof FeatureNodeContainer);
        menuItem.setVisible(canExecute);
        return canExecute;

    }

    /**
	 * Added to make the Class more generic and limit the amount of code changes required
	 * @param label
	 * @param event
	 * @param taxon
	 * @param description
	 * @param postOperationEnabled
	 * @return
	 */
    protected AbstractPostOperation operationCreationInstance(String label, Feature feature,
            DescriptionBase<?> description, IPostOperationEnabled postOperationEnabled) {
        return new CreateDescriptionElementOperation(label, EditorUtil.getUndoContext(), description, feature,
                postOperationEnabled);
    }

}
