/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.descriptive.e4.handler;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.inject.Named;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.model.application.ui.menu.MMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.SpecimenDescription;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.description.TaxonNameDescription;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.view.descriptive.e4.FactualDataPartE4;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.DeleteDescriptionElementOperation;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.DeleteSpecimenDescriptionOperation;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.DeleteTaxonDescriptionOperation;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.DeleteTaxonNameDescriptionOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;

/**
 *
 * @author pplitzner
 * @date 15.08.2017
 *
 */
public class DeleteHandlerE4 {

	@Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART) MPart activePart,
            @Named(IServiceConstants.ACTIVE_SHELL)Shell shell, MMenuItem menuItem,
            UISynchronize sync) {

        FactualDataPartE4 factualDataPart = (FactualDataPartE4) activePart.getObject();
        IStructuredSelection selection = (IStructuredSelection) factualDataPart.getViewer().getSelection();

        Object selectionProvidingPart = factualDataPart.getSelectionProvidingPart();
        ICdmEntitySessionEnabled cdmEntitySessionEnabled = null;
        if(selectionProvidingPart instanceof ICdmEntitySessionEnabled){
            cdmEntitySessionEnabled = (ICdmEntitySessionEnabled) selectionProvidingPart;
        }

		String label = menuItem.getLocalizedLabel();

        IUndoContext undoContext = EditorUtil.getUndoContext();

        List<AbstractPostOperation> operations = new ArrayList();

        for(Iterator iter = selection.iterator();iter.hasNext();){
            Object object = iter.next();

            // TaxonDescription
            if(object instanceof TaxonDescription ){
                DeleteTaxonDescriptionOperation operation = new DeleteTaxonDescriptionOperation(label, undoContext, (TaxonDescription) object, factualDataPart, cdmEntitySessionEnabled);

                operations.add(operation);
                ((TaxonDescription) object).getTaxon().removeDescription((TaxonDescription) object);
                operation.getPostOperationEnabled().postOperation(object);
            }
            if(object instanceof TaxonNameDescription ){
                DeleteTaxonNameDescriptionOperation operation = new DeleteTaxonNameDescriptionOperation(label, undoContext, (TaxonNameDescription) object, factualDataPart, cdmEntitySessionEnabled);
                operations.add(operation);
                ((TaxonNameDescription) object).getTaxonName().removeDescription((TaxonNameDescription) object);
                operation.getPostOperationEnabled().postOperation(object);
            }
            else if(object instanceof SpecimenDescription){
                DeleteSpecimenDescriptionOperation operation = new DeleteSpecimenDescriptionOperation(label, undoContext, (SpecimenDescription) object, factualDataPart, cdmEntitySessionEnabled);
                operations.add(operation) ;
                ((SpecimenDescription) object).getDescribedSpecimenOrObservation().removeDescription((SpecimenDescription) object);

            }
            // DescriptionElementBase
            else if(object instanceof DescriptionElementBase){
                DeleteDescriptionElementOperation operation = new DeleteDescriptionElementOperation(label, undoContext, (DescriptionElementBase) object, factualDataPart, cdmEntitySessionEnabled);
                AbstractUtility.executeOperation(operation, sync);
            }
            else if(object instanceof FeatureNodeContainer){
                List<DescriptionElementBase> descriptions = ((FeatureNodeContainer) object).getDescriptionElementsForEntireBranch();

                for(DescriptionElementBase description : descriptions){
                    DeleteDescriptionElementOperation operation = new DeleteDescriptionElementOperation(label, undoContext, description, factualDataPart, cdmEntitySessionEnabled);

                    AbstractUtility.executeOperation(operation, sync);
                }
            }

            else{
                MessagingUtils.error(getClass(), Messages.DeleteHandler_INVALID_SELECTION, null);
            }
        }

        // execute all cumulated operations
        for(AbstractPostOperation operation : operations){
            if (factualDataPart.getSelectionProvidingPart().getObject() instanceof TaxonNameEditorE4){
                ((TaxonNameEditorE4)factualDataPart.getSelectionProvidingPart().getObject()).getEditorInput().addOperation(operation);
                ((TaxonNameEditorE4)factualDataPart.getSelectionProvidingPart().getObject()).setDirty();
                ((TaxonNameEditorE4)factualDataPart.getSelectionProvidingPart().getObject()).redraw();
            }

           // AbstractUtility.executeOperation(operation, sync);
        }

	}


    @CanExecute
    public boolean canExecute(@Optional @Named(IServiceConstants.ACTIVE_PART) MPart activePart,
            MHandledMenuItem menuItem){
        FactualDataPartE4 factualDataPart = (FactualDataPartE4) activePart.getObject();
        ISelection selection = factualDataPart.getViewer().getSelection();
        boolean canExecute = selection instanceof IStructuredSelection && !selection.isEmpty();
        menuItem.setVisible(canExecute);
        return canExecute;
    }
}
