/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.descriptive.operation;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.e4.ui.di.UISynchronize;

import eu.etaxonomy.cdm.api.service.IDescriptionService;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Feb 8, 2011
 * @version 1.0
 */
public class MoveDescriptionElementsOperation extends AbstractPostTaxonOperation{

	private Collection<DescriptionElementBase> descriptionElements;
	private DescriptionBase targetDescription;
	private DescriptionBase sourceDescription;
	private IDescriptionService service;
	private UUID targetTaxonUuid;

	private String moveMessage;
	private boolean isCopy;
	private UISynchronize sync;

	/**
	 *
	 * @param label
	 * @param undoContext
	 * @param targetDescription
	 * @param descriptionElements
	 * @param isCopy
	 * @param postOperationEnabled
	 */
	public MoveDescriptionElementsOperation(String label,
			IUndoContext undoContext, DescriptionBase targetDescription, DescriptionBase sourceDescription,
			Collection<DescriptionElementBase> descriptionElements, boolean isCopy,
			IPostOperationEnabled postOperationEnabled, UISynchronize sync) {
		super(label, undoContext, postOperationEnabled);
		this.targetDescription = targetDescription;
		this.sourceDescription = sourceDescription;
		this.descriptionElements = descriptionElements;
		this.isCopy = isCopy;
		this.sync = sync;
		service = CdmStore.getService(IDescriptionService.class);
	}

	public MoveDescriptionElementsOperation(String label,
	        IUndoContext undoContext, UUID targetTaxonUuid, String moveMessage,
	        Collection<DescriptionElementBase> descriptionElements, boolean isCopy,
	        IPostOperationEnabled postOperationEnabled, UISynchronize sync) {
	    super(label, undoContext, postOperationEnabled);
	    this.targetTaxonUuid = targetTaxonUuid;
	    this.descriptionElements = descriptionElements;
	    this.moveMessage = moveMessage;
	    this.isCopy = isCopy;
	    this.sync = sync;
	    service = CdmStore.getService(IDescriptionService.class);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		Set<UUID> descriptionElementsUuid = new HashSet<UUID>();
		for(DescriptionElementBase deBase : descriptionElements) {
		    descriptionElementsUuid.add(deBase.getUuid());
		}
		UpdateResult result;
		if(targetDescription == null){
			result = service.moveDescriptionElementsToDescription(descriptionElementsUuid, targetTaxonUuid, moveMessage, isCopy);

		} else {
		    UUID targetDescriptionUuid = targetDescription.getUuid();
		    UUID sourceDescriptionUuid = sourceDescription.getUuid();
		    if (!targetDescription.isPersited()){
		        result = service.moveDescriptionElementsToDescription(descriptionElementsUuid, targetDescription, isCopy);
		    }else{
		        result = service.moveDescriptionElementsToDescription(descriptionElementsUuid, targetDescriptionUuid, isCopy);
		    }


		}
		return postExecute(null);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}



}
