// $Id$
/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.media;

import java.io.InputStream;
import java.net.URI;
import java.util.List;
import java.util.Set;

import org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider.IStyledLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.common.UriUtils;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.media.MediaRepresentation;
import eu.etaxonomy.cdm.model.media.MediaRepresentationPart;
import eu.etaxonomy.cdm.model.media.MediaUtils;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author k.luther
 * @date 11.10.2017
 *
 */
public class MediaViewLabelProvider extends LabelProvider implements IStyledLabelProvider {

    /**
     * {@inheritDoc}
     */
    @Override
    public String getText(Object element) {
        return getStyledText(element).getString();
    }

    @Override
    public StyledString getStyledText(Object element) {
        if (element instanceof IdentifiableEntity) {
            IdentifiableEntity<?> identifiableEntity= (IdentifiableEntity<?>) element;
            if(identifiableEntity.getId()==0){
                return new StyledString(identifiableEntity.generateTitle());
            }
            else{
                return new StyledString(identifiableEntity.getTitleCache());
            }
        }
        return null;
    }

    @Override
    public Image getImage(Object element) {
        Image image = null;
        int scale = 50;
        if (element instanceof Media && PreferencesUtil.isShowMediaPreview()){
            Set<MediaRepresentation> representations = ((Media)element).getRepresentations();
            if(representations.iterator().hasNext()){
                MediaRepresentation rep = null;

                MediaRepresentationPart smallestPart = null;
                rep = MediaUtils.findBestMatchingRepresentation((Media)element, null, null, scale, scale, null);
                List<MediaRepresentationPart> parts = rep.getParts();
                for (MediaRepresentationPart part: parts){
                    if (smallestPart != null){
                        if (smallestPart.getSize() != null && part.getSize() != null) {
                            if (smallestPart.getSize() > part.getSize()){
                                smallestPart = part;
                            }
                        }
                    }else{
                        smallestPart = part;
                    }

                }
                if (smallestPart != null){
                    URI uri = smallestPart.getUri();
                    if (uri == null){
                        return null;
                    }
                    InputStream imageStream;
                    try {
                        imageStream = UriUtils.getInputStream(uri);
                        image = new Image(Display.getCurrent(), imageStream);
                        int height = image.getBounds().height;
                        int width = image.getBounds().width;

                        if (height>width){
                            width = width/(height/scale);
                            height = scale;
                        }else{
                            height = height/(width/scale);
                            width = scale;
                        }
                        Image scaled = new Image(Display.getDefault(), scale, scale);

                        GC gc = new GC(scaled);
                        gc.setAntialias(SWT.ON);
                        gc.setInterpolation(SWT.HIGH);


                        gc.drawImage(image, 0, 0,
                        image.getBounds().width, image.getBounds().height,
                        0, 0, width, height);

                        gc.dispose();
                        image.dispose(); // don't forget about me!
                        return scaled;

                    } catch (Exception e) {
                        // TODO Auto-generated catch block
                       return null;
                    }

                }else{
                    return null;
                }

            }

        }
        return image;
    }

    @Override
    public void dispose() {
        // garbage collection system resources

    }



}
