/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.media.e4;

import javax.annotation.PostConstruct;

import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.services.EMenuService;
import org.eclipse.jface.viewers.AbstractTreeViewer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeNode;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;

import eu.etaxonomy.cdm.model.description.IDescribable;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.view.concept.e4.ConceptViewPartE4;
import eu.etaxonomy.taxeditor.editor.view.descriptive.e4.FactualDataPartE4;
import eu.etaxonomy.taxeditor.editor.view.media.MediaContentProvider;
import eu.etaxonomy.taxeditor.editor.view.media.MediaViewLabelProvider;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.model.IPartContentHasMedia;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;
import eu.etaxonomy.taxeditor.view.e4.AbstractCdmEditorPartE4;
import eu.etaxonomy.taxeditor.view.e4.details.DetailsPartE4;
import eu.etaxonomy.taxeditor.view.e4.supplementaldata.SupplementalDataPartE4;

/**
 *
 * @author pplitzner
 * @since Aug 14, 2017
 *
 */
public class MediaViewPartE4 extends AbstractCdmEditorPartE4
        implements IPartContentHasDetails, IPartContentHasSupplementalData {

    @PostConstruct
	public void create(Composite parent, EMenuService menuService, MPart thisPart) {

	    this.thisPart = thisPart;
		TreeViewer treeViewer = new TreeViewer(new Tree(parent, SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.MULTI));

		treeViewer.setContentProvider(new MediaContentProvider());
		treeViewer.setLabelProvider(new MediaViewLabelProvider());

//		TreeViewerColumn imagePreview = new TreeViewerColumn(treeViewer, SWT.NONE);
//		imagePreview.setLabelProvider(new DelegatingStyledCellLabelProvider(
//                new MediaViewLabelProvider(createImageDescriptor())));

		treeViewer.setAutoExpandLevel(2);
		viewer = treeViewer;

		// Propagate selection from viewer
        selectionChangedListener = (event -> selService.setSelection(event.getSelection()));
        viewer.addSelectionChangedListener(selectionChangedListener);

        //create context menu
        menuService.registerContextMenu(viewer.getControl(), "eu.etaxonomy.taxeditor.editor.popupmenu.mediaview");

	}

    @Override
    public void selectionChanged_internal(Object selection, MPart activePart, MPart thisPart) {
	    if(activePart==thisPart){
	        return;
	    }

	    Object partObject = createPartObject(activePart);

	    if (partObject instanceof DetailsPartE4 || partObject instanceof SupplementalDataPartE4
	            || partObject instanceof FactualDataPartE4 || partObject instanceof ConceptViewPartE4) {
	        // do not show empty page as these views are also used to edit the
	        // description selected in this view
	        return;
	    }

	    if(partObject instanceof IPartContentHasMedia && ((IPartContentHasMedia) partObject).canAttachMedia()){
	        if (viewer.getControl() != null && !viewer.getControl().isDisposed()) {
                viewer.getControl().setEnabled(true);
            }
	        IStructuredSelection structuredSelection = createSelection(selection);
	        if(structuredSelection==null || structuredSelection.isEmpty()){
	            showEmptyPage();
	            return;
	        }

	        Object firstElement = structuredSelection.getFirstElement();
	        if (partObject instanceof TaxonNameEditorE4){
	            if (firstElement != ((TaxonNameEditorE4)partObject).getTaxon()){
	                showEmptyPage();
	                return;
	            }
	        }
	        if(firstElement instanceof TreeNode){
	            showViewer(new StructuredSelection(((TreeNode) firstElement).getValue()), activePart, viewer);
	            return;
	        }
	        else if(firstElement!=null
	                && firstElement instanceof IDescribable<?> && !(firstElement instanceof TaxonName)){
	            showViewer(structuredSelection, activePart, viewer);
	            return;
	        }
	    }
	    showEmptyPage();
	    return;
	}

	/** {@inheritDoc} */
	@Override
	public void changed(Object object) {
	    ((AbstractTreeViewer) viewer).expandToLevel(2);
	    viewer.refresh();
	    if(object != null){
	        StructuredSelection selection = new StructuredSelection(object);
	        viewer.setSelection(selection, true);
	    }
	    super.changed(object);
	}

	@Override
	protected String getViewName() {
		return Messages.MediaViewPart_MEDIA;
	}



}
