/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.media.handler;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.model.application.ui.menu.MMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;

import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.e4.TaxonEditorInputE4;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.view.media.e4.MediaViewPartE4;
import eu.etaxonomy.taxeditor.editor.view.media.operation.MoveMediaInListOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.workbench.WorkbenchUtility;

/**
 * @author pplitzner
 * @since Jun 6, 2018
 *
 */
public abstract class AbstractMoveImageHandler {
    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART) MPart activePart,
            MMenuItem menuItem,
            UISynchronize sync) {

        MediaViewPartE4 mediaView = (MediaViewPartE4) activePart.getObject();

        TaxonNameEditorE4 editor = null;
        Object e4WrappedPart = WorkbenchUtility.getE4WrappedPart(mediaView.getSelectionProvidingPart());
        if (e4WrappedPart instanceof TaxonNameEditorE4) {
            editor = (TaxonNameEditorE4) e4WrappedPart;
            TaxonEditorInputE4 input = editor.getEditorInput();
            Taxon taxon =input.getTaxon();

            TreeSelection selection = (TreeSelection) mediaView.getViewer().getSelection();
            TreePath[] paths = selection.getPaths();
            int count = paths[0].getSegmentCount();
            DescriptionBase description = null;
            for (int i = 0; i < count; i++ ) {
                if (paths[0].getSegment(i) instanceof DescriptionBase) {
                    description = (DescriptionBase) paths[0].getSegment(i);
                    break;
                }
            }
            AbstractPostOperation operation = null;
            // TODO use undo context specific to editor
            operation = new MoveMediaInListOperation(menuItem.getLocalizedLabel(),
                    EditorUtil.getUndoContext(),
                    taxon,
                    description,
                    (Media)selection.getFirstElement(),
                    moveImageOperation(),
                    mediaView);
            AbstractUtility.executeOperation(operation, sync);
        }
    }

    public abstract int moveImageOperation();

    @CanExecute
    private boolean canExecute(MHandledMenuItem menuItem,
            @Named(IServiceConstants.ACTIVE_PART) MPart activePart){
        boolean canExecute = false;
        MediaViewPartE4 mediaView = (MediaViewPartE4) activePart.getObject();
        ISelection selection = mediaView.getViewer().getSelection();
        canExecute = selection instanceof TreeSelection
                && ((TreeSelection) selection).size()==1
                && ((TreeSelection) selection).getFirstElement() instanceof Media;
        menuItem.setVisible(canExecute);
        return canExecute;
    }
}
