/**
 *
 */
package eu.etaxonomy.taxeditor.editor.view.media.handler;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditorUtil;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.AddDerivedUnitFacadeMediaOperation;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.CreateTaxonDescriptionOperation;
import eu.etaxonomy.taxeditor.editor.view.media.e4.MediaViewPartE4;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;

/**
 *
 * @author pplitzner
 * @since Aug 14, 2017
 *
 */
public class AddImageGalleryHandler {

    /** {@inheritDoc} */
    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART) MPart activePart, MMenuItem menuItem,
            UISynchronize sync) {

        MediaViewPartE4 mediaView = (MediaViewPartE4) activePart.getObject();

        Object viewerInput = mediaView.getViewer().getInput();
        if(viewerInput instanceof CdmBase){
            if(((CdmBase) viewerInput).isInstanceOf(Taxon.class)){
                Taxon taxon = HibernateProxyHelper.deproxy(viewerInput, Taxon.class);

                AbstractPostOperation<?> operation = new CreateTaxonDescriptionOperation(menuItem.getLocalizedLabel(),
                        EditorUtil.getUndoContext(), taxon, null, mediaView, true);

                AbstractUtility.executeOperation(operation, sync);
            }
            else if(((CdmBase) viewerInput).isInstanceOf(SpecimenOrObservationBase.class)){
                SpecimenOrObservationBase<?> specimen = HibernateProxyHelper.deproxy(viewerInput, SpecimenOrObservationBase.class);

                AbstractPostOperation<?> operation = new AddDerivedUnitFacadeMediaOperation(menuItem.getLocalizedLabel(),
                        BulkEditorUtil.getUndoContext(), specimen, mediaView);

                AbstractUtility.executeOperation(operation, sync);
            }
        }
    }

}
