/**
 *
 */
package eu.etaxonomy.taxeditor.editor.view.media.handler;

import java.util.Iterator;
import java.util.Set;

import javax.inject.Named;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.model.application.ui.menu.MMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.config.MediaDeletionConfigurator;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.SpecimenDescription;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.description.TaxonNameDescription;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.DeleteSpecimenDescriptionOperation;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.DeleteTaxonDescriptionOperation;
import eu.etaxonomy.taxeditor.editor.view.media.e4.MediaViewPartE4;
import eu.etaxonomy.taxeditor.editor.view.media.operation.DeleteMediaOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.ui.dialog.configurator.deleteConfigurator.DeleteConfiguratorDialog;

/**
 *
 * @author pplitzner
 * @since Aug 15, 2017
 *
 */
public class DeleteMediaHandler  {

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART) MPart activePart,
            @Named(IServiceConstants.ACTIVE_SHELL)Shell shell,
            MMenuItem menuItem,
            UISynchronize sync) {

        MediaViewPartE4 mediaView = (MediaViewPartE4) activePart.getObject();
        TreeSelection selection = (TreeSelection) mediaView.getViewer().getSelection();

        for(Iterator iter = selection.iterator();iter.hasNext();){
            Object object = iter.next();
            TreePath[] paths = selection.getPaths();
            int count = paths[0].getSegmentCount();
            DescriptionBase description = null;
            for (int i = 0; i < count; i++ ) {
                if (paths[0].getSegment(i) instanceof DescriptionBase) {
                    description = (DescriptionBase) paths[0].getSegment(i);
                    break;
                }
            }
            // TODO use undo context specific to editor
            MediaDeletionConfigurator config = new MediaDeletionConfigurator();

            config.setDeleteFromDescription(true);
            config.setOnlyRemoveFromGallery(false);

            if (description instanceof SpecimenDescription){
                config.setDeleteFrom(((SpecimenDescription)description).getDescribedSpecimenOrObservation());
            } else if (description instanceof TaxonDescription){
                config.setDeleteFrom(((TaxonDescription)description).getTaxon());
            }else if (description instanceof TaxonNameDescription){
                config.setDeleteFrom(((TaxonNameDescription)description).getTaxonName());
            }
            DeleteConfiguratorDialog dialog;
            dialog = new DeleteConfiguratorDialog(config, shell, Messages.DeleteMediaHandler_CONFIRM,  null,  Messages.DeleteMediaHandler_CONFIRM_MESSAGE, MessageDialog.WARNING, new String[] { Messages.DeleteMediaHandler_DELETE, Messages.DeleteMediaHandler_SKIP }, 0);
            int result_dialog= dialog.open();
            if (result_dialog != IStatus.OK){
                return;
            }
            if(object instanceof Media){
                AbstractPostOperation<?> operation = new DeleteMediaOperation(menuItem.getLocalizedLabel(), EditorUtil.getUndoContext(), description, (Media) object, config, mediaView);
                if (mediaView.getSelectionProvidingPart().getObject() instanceof TaxonNameEditorE4){
                    if (!config.isOnlyRemoveFromGallery()){
                        ((TaxonNameEditorE4)mediaView.getSelectionProvidingPart().getObject()).getEditorInput().addOperation(operation);
                    }
                    Media media = (Media)object;
                    if (description.isImageGallery()){
                        Set<DescriptionElementBase> elements = description.getElements();
                        if (elements.size() == 1){
                              DescriptionElementBase element = elements.iterator().next();
                              element.removeMedia(media);
                              if (element.getMedia().isEmpty()){
                                  description.removeElement(element);
                              }
                        }
                    }

                  ((TaxonNameEditorE4)mediaView.getSelectionProvidingPart().getObject()).setDirty();
                  ((TaxonNameEditorE4)mediaView.getSelectionProvidingPart().getObject()).redraw();
                }else{
                    AbstractUtility.executeOperation(operation, sync);
                }
            }
            if(object instanceof DescriptionBase && ((DescriptionBase) object).isImageGallery()){
                if(object instanceof TaxonDescription){
                    DeleteTaxonDescriptionOperation deleteTaxonDescriptionOperation = new DeleteTaxonDescriptionOperation(menuItem.getLocalizedLabel(), EditorUtil.getUndoContext(), (TaxonDescription)object, mediaView, null);
                    if (mediaView.getSelectionProvidingPart().getObject() instanceof TaxonNameEditorE4){
                        ((TaxonNameEditorE4)mediaView.getSelectionProvidingPart().getObject()).getEditorInput().addOperation(deleteTaxonDescriptionOperation);
                        ((TaxonNameEditorE4)mediaView.getSelectionProvidingPart().getObject()).setDirty();
                        ((TaxonNameEditorE4)mediaView.getSelectionProvidingPart().getObject()).redraw();

                     }else{
                         AbstractUtility.executeOperation(deleteTaxonDescriptionOperation, sync);
                     }
                }
                else if(object instanceof SpecimenDescription){
                    DeleteSpecimenDescriptionOperation deleteTaxonDescriptionOperation = new DeleteSpecimenDescriptionOperation(menuItem.getLocalizedLabel(), EditorUtil.getUndoContext(), (SpecimenDescription)object, mediaView, null);
                    if (mediaView.getSelectionProvidingPart().getObject() instanceof TaxonNameEditorE4){
                        ((TaxonNameEditorE4)mediaView.getSelectionProvidingPart().getObject()).getEditorInput().addOperation(deleteTaxonDescriptionOperation);
                        ((TaxonNameEditorE4)mediaView.getSelectionProvidingPart().getObject()).setDirty();
                        ((TaxonNameEditorE4)mediaView.getSelectionProvidingPart().getObject()).redraw();
                     }else{
                         AbstractUtility.executeOperation(deleteTaxonDescriptionOperation, sync);
                     }
                }
            }
        }
    }

    @CanExecute
    public boolean canExecute(@Named(IServiceConstants.ACTIVE_PART) MPart activePart,
            MHandledMenuItem menuItem){
        MediaViewPartE4 mediaView = (MediaViewPartE4) activePart.getObject();
        ISelection selection = mediaView.getViewer().getSelection();

        boolean canExecute = selection instanceof TreeSelection && !selection.isEmpty();
        for(Iterator iter = ((TreeSelection) selection).iterator();iter.hasNext();){
            Object object = iter.next();
            canExecute &= object instanceof Media
                    || (object instanceof DescriptionBase && ((DescriptionBase) object).isImageGallery());
        }
        menuItem.setVisible(canExecute);
        return canExecute;
    }



}