/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.media.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.utility.ImagesUtility;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 * <p>CreateImageOperation class.</p>
 *
 * @author p.ciardelli
 * @created 31.03.2009
 */
public class CreateImageOperation extends AbstractPostTaxonOperation {

	private ImageFile imageFile;

	private final DescriptionBase<?> description;

	/**
	 * <p>Constructor for CreateImageOperation.</p>
	 *
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param label a {@link java.lang.String} object.
	 * @param description a {@link eu.etaxonomy.cdm.model.description.DescriptionBase} object.
	 * @param imageFile a {@link eu.etaxonomy.cdm.model.media.ImageFile} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public CreateImageOperation(String label, IUndoContext undoContext,
			Taxon taxon, DescriptionBase<?> description, ImageFile imageFile, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, taxon, postOperationEnabled);
		this.imageFile = imageFile;
		this.description = description;
	}

	/**
	 * <p>Constructor for CreateImageOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param description a {@link eu.etaxonomy.cdm.model.description.DescriptionBase} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public CreateImageOperation(String label, IUndoContext undoContext,
			Taxon taxon, DescriptionBase<?> description, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, taxon, postOperationEnabled);
		this.description = description;
	}

	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		if (imageFile == null) {
			imageFile = ImageFile.NewInstance(null, null);
		}
		monitor.worked(20);

		ImagesUtility.addTaxonImage(element, description, imageFile);
		monitor.worked(40);

		return postExecute(imageFile);
	}

	/** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		ImagesUtility.addTaxonImage(element, description, imageFile);

		return postExecute(imageFile);
	}

	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		ImagesUtility.removeTaxonImage(element, description, imageFile);

		return postExecute(null);
	}


}
