/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.media.operation;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.service.DeleteResult;
import eu.etaxonomy.cdm.api.service.IMediaService;
import eu.etaxonomy.cdm.api.service.config.MediaDeletionConfigurator;
import eu.etaxonomy.cdm.api.utility.ImagesUtility;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.bulkeditor.internal.TaxeditorBulkeditorPlugin;
import eu.etaxonomy.taxeditor.model.DeleteResultMessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>DeleteMediaOperation class.</p>
 *
 * @author p.ciardelli
 * @created 31.03.2009
 * @version 1.0
 */
public class DeleteMediaOperation extends AbstractPostTaxonOperation {

	private final Media media;

	private final DescriptionBase<?> description;

	private final MediaDeletionConfigurator config;

	/**
	 * <p>Constructor for DeleteMediaOperation.</p>
	 *
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param label a {@link java.lang.String} object.
	 * @param description a {@link eu.etaxonomy.cdm.model.description.DescriptionBase} object.
	 * @param media a {@link eu.etaxonomy.cdm.model.media.Media} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public DeleteMediaOperation(String label, IUndoContext undoContext,
	        DescriptionBase<?> description, Media media, MediaDeletionConfigurator config, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, postOperationEnabled);
		this.config = config;
		this.media = media;
		this.description = description;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

	   // ImagesUtility.removeMediaFromGallery(description, media);
	    if (monitor != null) {
            monitor.worked(20);
        }
		if (!config.isOnlyRemoveFromGallery() && media.getId() != 0){
			DeleteResult result = CdmStore.getService(IMediaService.class).delete(media.getUuid(), config);
			String errorMessage = "The media ";

			if (!result.isOk()){
				List<String> messages = new ArrayList<String>();
				int i = result.getExceptions().size();
				for (Exception e:result.getExceptions()){
					messages.add(e.getMessage());
				}
				errorMessage += "could not be deleted from database. ";
				//MessagingUtils.errorDialog("test", getClass(), "message", TaxeditorBulkeditorPlugin.PLUGIN_ID, result.getExceptions().iterator().next(),true);
				DeleteResultMessagingUtils.messageDialogWithDetails(result,errorMessage, TaxeditorBulkeditorPlugin.PLUGIN_ID);
//				if (config.isDeleteFromDescription()){
//					ImagesUtility.removeMediaFromGallery(description, media);
//				}
			}else{
//				ImagesUtility.removeMediaFromGallery(description, media);
			}
		} else{
//			ImagesUtility.removeMediaFromGallery(description, media);
		}
		if (monitor != null){
		    monitor.worked(40);
		}

		return postExecute(media);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		ImagesUtility.addMediaToGallery(description, media);

		return postExecute(null);
	}


}
