/**
 * Copyright (C) 2011 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.view.uses.e4;

import javax.annotation.PostConstruct;

import org.eclipse.e4.core.contexts.ContextInjectionFactory;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.services.EMenuService;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;

import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.view.descriptive.DescriptionElementDragListener;
import eu.etaxonomy.taxeditor.editor.view.descriptive.DescriptionElementDropAdapter;
import eu.etaxonomy.taxeditor.editor.view.descriptive.DescriptionElementTransfer;
import eu.etaxonomy.taxeditor.editor.view.descriptive.e4.FactualDataPartE4;
import eu.etaxonomy.taxeditor.editor.view.uses.UsesContentProvider;
import eu.etaxonomy.taxeditor.editor.view.uses.UsesLabelProvider;


/**
 *
 * @author pplitzner
 * @since Sep 8, 2017
 *
 */
public class UsesViewPartE4 extends FactualDataPartE4 {

    //FIXME E4 implement delete handler

    @Override
    @PostConstruct
    public void create(Composite parent, EMenuService menuService, MPart thisPart,
            IEclipseContext context) {

        this.thisPart = thisPart;

        TreeViewer treeViewer = new TreeViewer(new Tree(parent, SWT.H_SCROLL
                | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.MULTI));
        treeViewer.setContentProvider(new UsesContentProvider(featureNodeContainerCache));
        treeViewer.setLabelProvider(new UsesLabelProvider());
        treeViewer.setAutoExpandLevel(2);

        Transfer[] transfers = new Transfer[] { DescriptionElementTransfer.getInstance() };
        treeViewer.addDragSupport(dndOperations, transfers, new DescriptionElementDragListener(
                this));
        DescriptionElementDropAdapter dropListener = new DescriptionElementDropAdapter(treeViewer);
        ContextInjectionFactory.inject(dropListener, context);
        treeViewer.addDropSupport(dndOperations, transfers,
                dropListener);



        // Propagate selection from viewer
        selectionChangedListener = (event -> selService.setSelection(event.getSelection()));
        treeViewer.addSelectionChangedListener(selectionChangedListener);

        //create context menu
        menuService.registerContextMenu(treeViewer.getControl(), "eu.etaxonomy.taxeditor.editor.popupmenu.usesView");

        viewer = treeViewer;
    }

    @Override
    protected String getViewName(){
        return Messages.UsesViewPart_VIEWER_NAME;
    }

    /** {@inheritDoc} */
    @Override
    public boolean postOperation(Object objectAffectedByOperation) {
        viewer.refresh();
        return super.postOperation(objectAffectedByOperation);
    }

}
